@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2001 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,14 +17,21 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: utils.h,v 1.5 2001/11/18 15:58:42 f1rmb Exp $
+ * $Id: utils.h,v 1.16 2003/12/18 00:30:11 f1rmb Exp $
  *
  */
-#ifndef HAVE_UTILS_H
-#define HAVE_UTILS_H
+#ifndef _COMMON_UTILS_H
+#define _COMMON_UTILS_H
+
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 
 #include <sys/time.h>
 
+#ifdef HAVE_X11
+#include <X11/Xlib.h>
+#endif
 
 /* sys/time.h does not define timersub() on all platforms... */
 #ifndef timersub
@@ -39,9 +46,49 @@
   } while (0)
 #endif
 
+#ifndef NAME_MAX
+#define _NAME_MAX          256
+#else
+#define _NAME_MAX          NAME_MAX
+#endif
+
+#ifndef PATH_MAX
+#define _PATH_MAX          768
+#else
+#define _PATH_MAX          PATH_MAX
+#endif
+
 /*
  * Execute a shell command.
  */
 int xine_system(int dont_run_as_root, char *command);
 
+/*
+ * cleanup the str string, take care about '
+ */
+char *atoa(char *str);
+
+/*
+ * Create directories recursively
+ */
+int mkdir_safe(char *path);
+
+/*
+ * return 0/1 regarding to string (1/0, true/false, yes/no, on/off).
+ */
+int get_bool_value(const char *val);
+
+
+const char *get_last_double_semicolon(const char *str);
+int is_ipv6_last_double_semicolon(const char *str);
+
+inline int is_a_dir(char *filename);
+inline int is_a_file(char *filename);
+
+void dump_host_info(void);
+#ifdef HAVE_X11
+void dump_cpu_infos(void);
+void dump_xfree_info(Display *display, int screen, int complete);
+#endif
+
 #endif
