@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: control.c,v 1.58 2002/08/01 08:38:58 f1rmb Exp $
+ * $Id: control.c,v 1.93 2003/10/29 12:32:15 f1rmb Exp $
  *
  *
  */
@@ -33,18 +33,13 @@
 #include <X11/keysym.h>
 #include <pthread.h>
 
-#include <xine.h>
-#include <xine/xineutils.h>
+#ifdef HAVE_ALLOCA_H
+#include <alloca.h>
+#endif
 
-#include "Imlib-light/Imlib.h"
-#include "panel.h"
-#include "event.h"
-#include "actions.h"
-#include "skins.h"
-#include "errors.h"
-#include "i18n.h"
+#include "common.h"
 
-#include "xitk.h"
+#define TEST_VO_VALUE(val)  (val < 65535/3 || val > 65535*2/3) ? (65535 - val) : 0
 
 extern gGui_t          *gGui;
 
@@ -58,7 +53,7 @@ typedef struct {
   xitk_widget_list_t   *widget_list;
 
   xitk_widget_t        *skinlist;
-  char                 *skins[64];
+  const char           *skins[64];
   int                   skins_num;
 
   int                   running;
@@ -70,6 +65,21 @@ typedef struct {
 } _control_t;
 
 static _control_t    *control = NULL;
+static int hue_ena, sat_ena, bright_ena, contr_ena;
+
+
+static void hue_changes_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->video_settings.hue = cfg->num_value;
+}
+static void brightness_changes_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->video_settings.brightness = cfg->num_value;
+}
+static void saturation_changes_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->video_settings.saturation = cfg->num_value;
+}
+static void contrast_changes_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->video_settings.contrast = cfg->num_value;
+}
 
 /*
  *
@@ -85,48 +95,17 @@ void control_show_tips(int enabled, unsigned long timeout) {
 }
 
 /*
- * Get current property 'prop' value from vo_driver.
+ * Get current parameter 'param' value.
  */
-static int get_current_prop(int prop) {
-  return (gGui->vo_driver->get_property(gGui->vo_driver, prop));
+static int get_current_param(int param) {
+  return (xine_get_param(gGui->stream, param));
 }
 
 /*
- * set property 'prop' to  value 'value'.
- * vo_driver return value on success, ~value on failure.
+ * set parameter 'param' to value 'value'.
  */
-static int set_current_prop(int prop, int value) {
-  return (gGui->vo_driver->set_property(gGui->vo_driver, prop, value));
-}
-
-/*
- * Enable or disable video settings sliders.
- */
-static void active_sliders_video_settings(void) {
-  int vidcap;
-  
-  if((vidcap = gGui->vo_driver->get_capabilities(gGui->vo_driver)) > 0) {
-    
-    if(vidcap & VO_CAP_BRIGHTNESS)
-      xitk_enable_widget(control->bright);
-    else
-      xitk_disable_widget(control->bright);
-    
-    if(vidcap & VO_CAP_SATURATION)
-      xitk_enable_widget(control->sat);
-    else
-      xitk_disable_widget(control->sat);
-    
-    if(vidcap & VO_CAP_HUE)
-      xitk_enable_widget(control->hue);
-    else
-      xitk_disable_widget(control->hue);
-    
-    if(vidcap & VO_CAP_CONTRAST)
-      xitk_enable_widget(control->contr);
-    else
-      xitk_disable_widget(control->contr);
-  }
+static void set_current_param(int param, int value) {
+  xine_set_param(gGui->stream, param, value);
 }
 
 /*
@@ -134,21 +113,19 @@ static void active_sliders_video_settings(void) {
  */
 static void update_sliders_video_settings(void) {
 
-  if(xitk_is_widget_enabled(control->hue)) {
-    xitk_slider_set_pos(control->widget_list, control->hue, 
-			get_current_prop(VO_PROP_HUE));
-  }
-  if(xitk_is_widget_enabled(control->sat)) {
-    xitk_slider_set_pos(control->widget_list, control->sat, 
-			get_current_prop(VO_PROP_SATURATION));
-  }
-  if(xitk_is_widget_enabled(control->bright)) {
-    xitk_slider_set_pos(control->widget_list, control->bright, 
-			get_current_prop(VO_PROP_BRIGHTNESS));
-  }
-  if(xitk_is_widget_enabled(control->contr)) {
-    xitk_slider_set_pos(control->widget_list, control->contr, 
-			get_current_prop(VO_PROP_CONTRAST));
+  if(control) {
+    if(xitk_is_widget_enabled(control->hue)) {
+      xitk_slider_set_pos(control->hue, get_current_param(XINE_PARAM_VO_HUE));
+    }
+    if(xitk_is_widget_enabled(control->sat)) {
+      xitk_slider_set_pos(control->sat, get_current_param(XINE_PARAM_VO_SATURATION));
+    }
+    if(xitk_is_widget_enabled(control->bright)) {
+      xitk_slider_set_pos(control->bright, get_current_param(XINE_PARAM_VO_BRIGHTNESS));
+    }
+    if(xitk_is_widget_enabled(control->contr)) {
+      xitk_slider_set_pos(control->contr, get_current_param(XINE_PARAM_VO_CONTRAST));
+    }
   }
 }
 
@@ -156,64 +133,198 @@ static void update_sliders_video_settings(void) {
  * Set hue
  */
 static void set_hue(xitk_widget_t *w, void *data, int value) {
-  int ret = 0;
+  int hue;
 
-  ret = set_current_prop(VO_PROP_HUE, value);
-    
-  if(ret != value)
+  set_current_param(XINE_PARAM_VO_HUE, value);
+  
+  if((hue = get_current_param(XINE_PARAM_VO_HUE)) != value)
     update_sliders_video_settings();
+  
+  if(hue_ena)
+    config_update_num("gui.vo_hue", hue);
 }
 
 /*
  * Set saturation
  */
 static void set_saturation(xitk_widget_t *w, void *data, int value) {
-  int ret = 0;
+  int saturation;
 
-  ret = set_current_prop(VO_PROP_SATURATION, value);
+  set_current_param(XINE_PARAM_VO_SATURATION, value);
    
-  if(ret != value)
+  if((saturation = get_current_param(XINE_PARAM_VO_SATURATION)) != value)
     update_sliders_video_settings();
+
+  if(sat_ena)
+    config_update_num("gui.vo_saturation", saturation);
 }
 
 /*
  * Set brightness
  */
 static void set_brightness(xitk_widget_t *w, void *data, int value) {
-  int ret = 0;
+  int brightness;
 
-  ret = set_current_prop(VO_PROP_BRIGHTNESS, value);
+  set_current_param(XINE_PARAM_VO_BRIGHTNESS, value);
     
-  if(ret != value)
+  if((brightness = get_current_param(XINE_PARAM_VO_BRIGHTNESS)) != value)
     update_sliders_video_settings();
+  
+  if(bright_ena)
+    config_update_num("gui.vo_brightness", brightness);
 }
 
 /*
  * Set contrast
  */
 static void set_contrast(xitk_widget_t *w, void *data, int value) {
-  int ret = 0;
+  int contrast;
 
-  ret = set_current_prop(VO_PROP_CONTRAST, value);
-    
-  if(ret != value)
+  set_current_param(XINE_PARAM_VO_CONTRAST, value);
+  
+  if((contrast = get_current_param(XINE_PARAM_VO_CONTRAST)) != value)
     update_sliders_video_settings();
+  
+  if(contr_ena)
+    config_update_num("gui.vo_contrast", contrast);
+}
+
+void control_set_image_prop(int prop, int value) {
+  switch(prop) {
+  case XINE_PARAM_VO_HUE:
+    set_current_param(XINE_PARAM_VO_HUE, value);
+    break;
+  case XINE_PARAM_VO_SATURATION:
+    set_current_param(XINE_PARAM_VO_SATURATION, value);
+    break;
+  case XINE_PARAM_VO_BRIGHTNESS:
+    set_current_param(XINE_PARAM_VO_BRIGHTNESS, value);
+    break;
+  case XINE_PARAM_VO_CONTRAST:
+    set_current_param(XINE_PARAM_VO_CONTRAST, value);
+    break;
+  default:
+    return;
+    break;
+  }
+  update_sliders_video_settings();
+}
+
+static int test_vo_property(int property) {
+  int cur;
+
+  cur = get_current_param(property);
+  set_current_param(property, TEST_VO_VALUE(cur));
+  if((get_current_param(property)) == cur)
+    return 0;
+  else {
+    set_current_param(property, cur);
+    return 1;
+  }
+}
+
+
+/*
+ * Enable or disable video settings sliders.
+ */
+static void active_sliders_video_settings(void) {
+  
+  if(!bright_ena)
+    xitk_disable_widget(control->bright);
+  else
+    xitk_enable_widget(control->bright);
+  
+  if(!sat_ena)
+    xitk_disable_widget(control->sat);
+  else
+    xitk_enable_widget(control->sat);
+
+  if(!hue_ena)
+    xitk_disable_widget(control->hue);
+  else
+    xitk_enable_widget(control->hue);
+
+  if(!contr_ena)
+    xitk_disable_widget(control->contr);
+  else
+    xitk_enable_widget(control->contr);
+}
+
+static void probe_active_controls(void) {
+  hue_ena    = test_vo_property(XINE_PARAM_VO_HUE);
+  bright_ena = test_vo_property(XINE_PARAM_VO_BRIGHTNESS);
+  sat_ena    = test_vo_property(XINE_PARAM_VO_SATURATION);
+  contr_ena  = test_vo_property(XINE_PARAM_VO_CONTRAST);
+}
+
+void control_config_register(void) {
+  
+  probe_active_controls();
+  
+  if(hue_ena)
+    set_current_param(XINE_PARAM_VO_HUE,
+		    (gGui->video_settings.hue = 
+		     xine_config_register_range(gGui->xine, "gui.vo_hue",
+						(get_current_param(XINE_PARAM_VO_HUE)),
+						0, 65535,
+						_("hue value"),
+						_("Hue value."), 
+						CONFIG_LEVEL_DEB,
+						hue_changes_cb, 
+						CONFIG_NO_DATA)));
+  
+  if(bright_ena)
+    set_current_param(XINE_PARAM_VO_BRIGHTNESS, 
+		    (gGui->video_settings.brightness = 
+		     xine_config_register_range(gGui->xine, "gui.vo_brightness",
+						(get_current_param(XINE_PARAM_VO_BRIGHTNESS)),
+						0, 65535,
+						_("brightness value"),
+						_("Brightness value."), 
+						CONFIG_LEVEL_DEB,
+						brightness_changes_cb, 
+						CONFIG_NO_DATA)));
+
+  if(sat_ena)
+    set_current_param(XINE_PARAM_VO_SATURATION,
+		    (gGui->video_settings.saturation = 
+		     xine_config_register_range(gGui->xine, "gui.vo_saturation",
+						(get_current_param(XINE_PARAM_VO_SATURATION)),
+						0, 65535,
+						_("saturation value"),
+						_("Saturation value."), 
+						CONFIG_LEVEL_DEB,
+						saturation_changes_cb, 
+						CONFIG_NO_DATA)));
+
+  if(contr_ena)
+    set_current_param(XINE_PARAM_VO_CONTRAST,
+		    (gGui->video_settings.contrast = 
+		     xine_config_register_range(gGui->xine, "gui.vo_contrast",
+						(get_current_param(XINE_PARAM_VO_CONTRAST)),
+						0, 65535,
+						_("contrast value"),
+						_("Contrast value."), 
+						CONFIG_LEVEL_DEB,
+						contrast_changes_cb, 
+						CONFIG_NO_DATA)));
 }
 
 /*
  * Leaving control panel, release memory.
  */
 void control_exit(xitk_widget_t *w, void *data) {
-  window_info_t wi;
-  int           i;
 
   if(control) {
+    window_info_t wi;
+    int           i;
+    
     control->running = 0;
     control->visible = 0;
 
     if((xitk_get_window_info(control->widget_key, &wi))) {
-      gGui->config->update_num (gGui->config, "gui.control_x", wi.x);
-      gGui->config->update_num (gGui->config, "gui.control_y", wi.y);
+      config_update_num("gui.control_x", wi.x);
+      config_update_num("gui.control_y", wi.y);
       WINDOW_INFO_ZERO(&wi);
     }
 
@@ -231,16 +342,16 @@ void control_exit(xitk_widget_t *w, void *data) {
     XUnlockDisplay(gGui->display);
 
     control->window = None;
-    xitk_list_free(control->widget_list->l);
+    xitk_list_free((XITK_WIDGET_LIST_LIST(control->widget_list)));
 
     XLockDisplay(gGui->display);
-    XFreeGC(gGui->display, control->widget_list->gc);
+    XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(control->widget_list)));
     XUnlockDisplay(gGui->display);
 
     free(control->widget_list);
 
     for(i = 0; i < control->skins_num; i++)
-      free(control->skins[i]);
+      free((char *)control->skins[i]);
 
     free(control);
     control = NULL;
@@ -263,8 +374,12 @@ int control_is_running(void) {
  */
 int control_is_visible(void) {
 
-  if(control != NULL)
-    return control->visible;
+  if(control != NULL) {
+    if(gGui->use_root_window)
+      return xitk_is_window_visible(gGui->display, control->window);
+    else
+      return control->visible;
+  }
 
   return 0;
 }
@@ -273,66 +388,31 @@ int control_is_visible(void) {
  * Raise control->window
  */
 void control_raise_window(void) {
-  
-  if(control != NULL) {
-    if(control->window) {
-      if(control->visible && control->running) {
-	if(control->running) {
-	  XMapRaised(gGui->display, control->window);
-	  control->visible = 1;
-	  XLockDisplay(gGui->display);
-	  XSetTransientForHint (gGui->display, 
-				control->window, gGui->video_window);
-	  XUnlockDisplay(gGui->display);
-	  layer_above_video(control->window);
-	}
-      } else {
-	XLockDisplay(gGui->display);
-	XUnmapWindow (gGui->display, control->window);
-	XUnlockDisplay(gGui->display);
-	control->visible = 0;
-      }
-    }
-  }
+  if(control != NULL)
+    raise_window(control->window, control->visible, control->running);
 }
 
 /*
  * Hide/show the control panel
  */
 void control_toggle_visibility (xitk_widget_t *w, void *data) {
-  
-  if(control != NULL) {
-    if (control->visible && control->running) {
-      control->visible = 0;
-      xitk_hide_widgets(control->widget_list);
-      XLockDisplay(gGui->display);
-      XUnmapWindow (gGui->display, control->window);
-      XUnlockDisplay(gGui->display);
-    } else {
-      if(control->running) {
-	control->visible = 1;
-	xitk_show_widgets(control->widget_list);
-	XLockDisplay(gGui->display);
-	XMapRaised(gGui->display, control->window); 
-	XSetTransientForHint (gGui->display, 
-			      control->window, gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(control->window);
-      }
-    }
-  }
+  if(control != NULL)
+    toggle_window(control->window, control->widget_list,
+		  &control->visible, control->running);
 }
 
 /*
  * Handle X events here.
  */
 void control_handle_event(XEvent *event, void *data) {
-
-  /*
+  
   switch(event->type) {
-
+    
+  case KeyPress:
+    gui_handle_event(event, data);
+    break;      
   }
-  */
+
 }
 
 /*
@@ -354,10 +434,13 @@ void control_change_skins(void) {
       xine_error(_("%s(): couldn't find image for background\n"), __XINE_FUNCTION__);
       exit(-1);
     }
-    
+    XUnlockDisplay(gGui->display);
+
     hint.width  = new_img->rgb_width;
     hint.height = new_img->rgb_height;
-    hint.flags  = PSize;
+    hint.flags  = PPosition | PSize;
+
+    XLockDisplay(gGui->display);
     XSetWMNormalHints(gGui->display, control->window, &hint);
     
     XResizeWindow (gGui->display, control->window,
@@ -375,8 +458,8 @@ void control_change_skins(void) {
     control->bg_image = new_img;
     
     XLockDisplay(gGui->display);
-    
-    XSetTransientForHint(gGui->display, control->window, gGui->video_window);
+    if(!gGui->use_root_window)
+      XSetTransientForHint(gGui->display, control->window, gGui->video_window);
     
     Imlib_destroy_image(gGui->imlib_data, old_img);
     Imlib_apply_image(gGui->imlib_data, new_img, control->window);
@@ -395,13 +478,19 @@ void control_change_skins(void) {
 /*
  *
  */
-static void control_select_new_skin(xitk_widget_t *w, void *data) {
-  int                 selected = (int)data;
-
+static void control_select_new_skin(xitk_widget_t *w, void *data, int selected) {
   xitk_browser_release_all_buttons(control->skinlist);
   select_new_skin(selected);
 }
 
+void control_deinit(void) {
+  if(control) {
+    if(control_is_visible())
+      control_toggle_visibility(NULL, NULL);
+    xitk_unregister_event_handler(&control->widget_key);
+  }
+}
+
 /*
  * Create control panel window
  */
@@ -411,7 +500,7 @@ void control_panel(void) {
   XWMHints                  *wm_hint;
   XSetWindowAttributes       attr;
   char                      *title;
-  Atom                       prop, XA_WIN_LAYER;
+  Atom                       prop;
   MWMHints                   mwmhints;
   XClassHint                *xclasshint;
   xitk_browser_widget_t      br;
@@ -420,15 +509,8 @@ void control_panel(void) {
   xitk_slider_widget_t       sl;
   xitk_combo_widget_t        cmb;
   xitk_widget_t             *w;
-  long                       data[1];
 
-  /* This shouldn't be happend */
-  if(control != NULL) {
-    if(control->window)
-      return;
-  }
-  
-  xine_strdupa(title, _("Xine Control Panel"));
+  xine_strdupa(title, _("xine Control Window"));
 
   XITK_WIDGET_INIT(&br, gGui->imlib_data);
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
@@ -443,14 +525,27 @@ void control_panel(void) {
   if (!(control->bg_image = 
 	Imlib_load_image(gGui->imlib_data,
 			 xitk_skin_get_skin_filename(gGui->skin_config, "CtlBG")))) {
-    xine_error(_("xine-playlist: couldn't find image for background\n"));
+    xine_error(_("control: couldn't find image for background\n"));
     exit(-1);
   }
 
-  hint.x = gGui->config->register_num (gGui->config, "gui.control_x", 200,
-				       NULL, NULL, NULL, NULL);
-  hint.y = gGui->config->register_num (gGui->config, "gui.control_y", 100,
-				       NULL, NULL, NULL, NULL);
+  XUnlockDisplay(gGui->display);
+
+  hint.x = xine_config_register_num (gGui->xine, "gui.control_x", 
+				     200,
+				     CONFIG_NO_DESC,
+				     CONFIG_NO_HELP,
+				     CONFIG_LEVEL_DEB,
+				     CONFIG_NO_CB,
+				     CONFIG_NO_DATA);
+  hint.y = xine_config_register_num (gGui->xine, "gui.control_y", 
+				     100,
+				     CONFIG_NO_DESC,
+				     CONFIG_NO_HELP,
+				     CONFIG_LEVEL_DEB,
+				     CONFIG_NO_CB,
+				     CONFIG_NO_DATA);
+
   hint.width = control->bg_image->rgb_width;
   hint.height = control->bg_image->rgb_height;
   hint.flags = PPosition | PSize;
@@ -466,6 +561,8 @@ void control_panel(void) {
    * Colormap
    */
   attr.border_pixel      = 1;
+
+  XLockDisplay(gGui->display);
   attr.colormap		 = Imlib_get_colormap(gGui->imlib_data);
 
   control->window = XCreateWindow (gGui->display,
@@ -480,25 +577,16 @@ void control_panel(void) {
  			 None, NULL, 0, &hint);
   
   XSelectInput(gGui->display, control->window, INPUT_MOTION | KeymapStateMask);
+  XUnlockDisplay(gGui->display);
   
-  /*
-   * layer above most other things, like gnome panel
-   * WIN_LAYER_ABOVE_DOCK  = 10
-   *
-   */
-  if(gGui->layer_above) {
-    XA_WIN_LAYER = XInternAtom(gGui->display, "_WIN_LAYER", False);
-    
-    data[0] = 10;
-    XChangeProperty(gGui->display, control->window, XA_WIN_LAYER,
-		    XA_CARDINAL, 32, PropModeReplace, (unsigned char *)data,
-		    1);
-  }
-  
+  if(is_layer_above())
+    xitk_set_layer_above(control->window);
+
   /*
    * wm, no border please
    */
 
+  XLockDisplay(gGui->display);
   prop = XInternAtom(gGui->display, "_MOTIF_WM_HINTS", True);
   mwmhints.flags = MWM_HINTS_DECORATIONS;
   mwmhints.decorations = 0;
@@ -507,47 +595,45 @@ void control_panel(void) {
                   PropModeReplace, (unsigned char *) &mwmhints,
                   PROP_MWM_HINTS_ELEMENTS);
   
-  XSetTransientForHint (gGui->display, control->window, gGui->video_window);
-
   /* set xclass */
 
   if((xclasshint = XAllocClassHint()) != NULL) {
     xclasshint->res_name = title;
-    xclasshint->res_class = "Xine";
+    xclasshint->res_class = "xine";
     XSetClassHint(gGui->display, control->window, xclasshint);
   }
 
   wm_hint = XAllocWMHints();
   if (wm_hint != NULL) {
-    wm_hint->input = True;
+    wm_hint->input         = True;
     wm_hint->initial_state = NormalState;
-    wm_hint->flags = InputHint | StateHint;
+    wm_hint->flags         = InputHint | StateHint;
     XSetWMHints(gGui->display, control->window, wm_hint);
-    XFree(wm_hint); /* CHECKME */
+    XFree(wm_hint);
   }
   
   gc = XCreateGC(gGui->display, control->window, 0, 0);
   
   Imlib_apply_image(gGui->imlib_data, control->bg_image, control->window);
 
+  XUnlockDisplay (gGui->display);
+
   /*
    * Widget-list
    */
-  control->widget_list                = xitk_widget_list_new();
-  control->widget_list->l             = xitk_list_new ();
-  control->widget_list->win           = control->window;
-  control->widget_list->gc            = gc;
+  control->widget_list = xitk_widget_list_new();
+  xitk_widget_list_set(control->widget_list, WIDGET_LIST_LIST, (xitk_list_new()));
+  xitk_widget_list_set(control->widget_list, WIDGET_LIST_WINDOW, (void *) control->window);
+  xitk_widget_list_set(control->widget_list, WIDGET_LIST_GC, gc);
   
   { /* All of sliders are disabled by default*/
-    int min, max, cur;
+    int min = 0, max = 65535, cur;
 
-    lbl.window = control->widget_list->win;
-    lbl.gc     = control->widget_list->gc;
+    lbl.window = XITK_WIDGET_LIST_WINDOW(control->widget_list);
+    lbl.gc     = XITK_WIDGET_LIST_GC(control->widget_list);
 
     /* HUE */
-    gGui->vo_driver->get_property_min_max(gGui->vo_driver, 
-					  VO_PROP_HUE, &min, &max);
-    cur = get_current_prop(VO_PROP_HUE);
+    cur = get_current_param(XINE_PARAM_VO_HUE);
     
     sl.skin_element_name = "SliderCtlHue";
     sl.min               = min;
@@ -557,22 +643,20 @@ void control_panel(void) {
     sl.userdata          = NULL;
     sl.motion_callback   = set_hue;
     sl.motion_userdata   = NULL;
-    xitk_list_append_content(control->widget_list->l,
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(control->widget_list)),
 	     (control->hue = xitk_slider_create(control->widget_list, gGui->skin_config, &sl)));
-    xitk_slider_set_pos(control->widget_list, control->hue, cur);
+    xitk_slider_set_pos(control->hue, cur);
     xitk_set_widget_tips(control->hue, _("Control HUE value"));
-  
+    
     lbl.skin_element_name = "CtlHueLbl";
     lbl.label             = _("Hue");
     lbl.callback          = NULL;
-    xitk_list_append_content(control->widget_list->l,
-			    xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(control->widget_list)),
+			     xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
     xitk_disable_widget(control->hue);
-
+    
     /* SATURATION */
-    gGui->vo_driver->get_property_min_max(gGui->vo_driver, 
-					  VO_PROP_SATURATION, &min, &max);
-    cur = get_current_prop(VO_PROP_SATURATION);
+    cur = get_current_param(XINE_PARAM_VO_SATURATION);
 
     sl.skin_element_name = "SliderCtlSat";
     sl.min               = min;
@@ -582,22 +666,20 @@ void control_panel(void) {
     sl.userdata          = NULL;
     sl.motion_callback   = set_saturation;
     sl.motion_userdata   = NULL;
-    xitk_list_append_content(control->widget_list->l,
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(control->widget_list)),
 	      (control->sat = xitk_slider_create(control->widget_list, gGui->skin_config, &sl)));
-    xitk_slider_set_pos(control->widget_list, control->sat, cur);
+    xitk_slider_set_pos(control->sat, cur);
     xitk_set_widget_tips(control->sat, _("Control SATURATION value"));
 
     lbl.skin_element_name = "CtlSatLbl";
     lbl.label             = _("Sat");
     lbl.callback          = NULL;
-    xitk_list_append_content(control->widget_list->l,
-			    xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(control->widget_list)),
+			     xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
     xitk_disable_widget(control->sat);
-      
+
     /* BRIGHTNESS */
-    gGui->vo_driver->get_property_min_max(gGui->vo_driver, 
-					  VO_PROP_BRIGHTNESS, &min, &max);
-    cur = get_current_prop(VO_PROP_BRIGHTNESS);
+    cur = get_current_param(XINE_PARAM_VO_BRIGHTNESS);
 
     sl.skin_element_name = "SliderCtlBright";
     sl.min               = min;
@@ -607,22 +689,20 @@ void control_panel(void) {
     sl.userdata          = NULL;
     sl.motion_callback   = set_brightness;
     sl.motion_userdata   = NULL;
-    xitk_list_append_content(control->widget_list->l,
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(control->widget_list)),
 	    (control->bright = xitk_slider_create(control->widget_list, gGui->skin_config, &sl)));
-    xitk_slider_set_pos(control->widget_list, control->bright, cur);
+    xitk_slider_set_pos(control->bright, cur);
     xitk_set_widget_tips(control->bright, _("Control BRIGHTNESS value"));
 
     lbl.skin_element_name = "CtlBrightLbl";
     lbl.label             = _("Brt");
     lbl.callback          = NULL;
-    xitk_list_append_content(control->widget_list->l,
-			    xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(control->widget_list)),
+			     xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
     xitk_disable_widget(control->bright);
       
     /* CONTRAST */
-    gGui->vo_driver->get_property_min_max(gGui->vo_driver, 
-					  VO_PROP_CONTRAST, &min, &max);
-    cur = get_current_prop(VO_PROP_CONTRAST);
+    cur = get_current_param(XINE_PARAM_VO_CONTRAST);
 
     sl.skin_element_name = "SliderCtlCont";
     sl.min               = min;
@@ -632,18 +712,19 @@ void control_panel(void) {
     sl.userdata          = NULL;
     sl.motion_callback   = set_contrast;
     sl.motion_userdata   = NULL;
-    xitk_list_append_content(control->widget_list->l,
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(control->widget_list)),
 	      (control->contr = xitk_slider_create(control->widget_list, gGui->skin_config, &sl)));
-    xitk_slider_set_pos(control->widget_list, control->contr, cur);
+    xitk_slider_set_pos(control->contr, cur);
     xitk_set_widget_tips(control->contr, _("Control CONTRAST value"));
 
     lbl.skin_element_name = "CtlContLbl";
     lbl.label             = _("Ctr");
     lbl.callback          = NULL;
-    xitk_list_append_content(control->widget_list->l,
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(control->widget_list)),
 			    xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
     xitk_disable_widget(control->contr);
 
+    probe_active_controls();
     active_sliders_video_settings();
   }
 
@@ -661,37 +742,28 @@ void control_panel(void) {
   lbl.skin_element_name = "CtlSkLbl";
   lbl.label             = _("Choose a Skin");
   lbl.callback          = NULL;
-  xitk_list_append_content(control->widget_list->l,
+  xitk_list_append_content((XITK_WIDGET_LIST_LIST(control->widget_list)),
 			   xitk_label_create(control->widget_list, gGui->skin_config, &lbl));
 
   br.arrow_up.skin_element_name    = "CtlSkUp";
   br.slider.skin_element_name      = "SliderCtlSk";
   br.arrow_dn.skin_element_name    = "CtlSkDn";
+  br.arrow_left.skin_element_name  = "CtlSkLeft";
+  br.slider_h.skin_element_name    = "SliderHCtlSk";
+  br.arrow_right.skin_element_name = "CtlSkRight";
   br.browser.skin_element_name     = "CtlSkItemBtn";
   br.browser.num_entries           = control->skins_num;
   br.browser.entries               = control->skins;
   br.callback                      = control_select_new_skin;
   br.dbl_click_callback            = NULL;
   br.parent_wlist                  = control->widget_list;
-  xitk_list_append_content (control->widget_list->l, 
+  xitk_list_append_content ((XITK_WIDGET_LIST_LIST(control->widget_list)), 
 			   (control->skinlist = 
 			    xitk_browser_create(control->widget_list, gGui->skin_config, &br)));
 
   xitk_browser_update_list(control->skinlist, 
 			   control->skins, control->skins_num, 0);
 
-  /* Temporary test, don't remove me.
-  cmb.skin_element_name = "combotest";
-  cmb.layer_above       = gGui->layer_above;
-  cmb.parent_wlist      = control->widget_list;
-  cmb.entries           = control->skins;
-  cmb.parent_wkey       = &control->widget_key;
-  cmb.callback          = NULL;
-  cmb.userdata          = NULL;
-  xitk_list_append_content(control->widget_list->l, 
-			   (control->combo = xitk_combo_create(gGui->skin_config, &cmb)));
-  xitk_combo_set_select(control->widget_list, control->combo, 3);
-  */
 
   lb.skin_element_name = "CtlDismiss";
   lb.button_type       = CLICK_BUTTON;
@@ -699,16 +771,12 @@ void control_panel(void) {
   lb.callback          = control_exit;
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
-  xitk_list_append_content (control->widget_list->l, 
+  xitk_list_append_content ((XITK_WIDGET_LIST_LIST(control->widget_list)), 
 		    (w = xitk_labelbutton_create (control->widget_list, gGui->skin_config, &lb)));
   xitk_set_widget_tips(w, _("Close control window"));
 
   control_show_tips(panel_get_tips_enable(), panel_get_tips_timeout());
 
-  XMapRaised(gGui->display, control->window); 
-
-  XUnlockDisplay (gGui->display);
-
   control->widget_key = 
     xitk_register_event_handler("control", 
 				control->window, 
@@ -719,4 +787,7 @@ void control_panel(void) {
   
   control->visible = 1;
   control->running = 1;
+  control_raise_window();
+
+  try_to_set_input_focus(control->window);
 }
