@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: file_browser.c,v 1.38 2003/07/25 22:52:57 f1rmb Exp $
+ * $Id: file_browser.c,v 1.44 2003/11/09 13:52:31 f1rmb Exp $
  *
  */
 
@@ -81,24 +81,48 @@ typedef struct {
 } filebrowser_filter_t;
 
 static filebrowser_filter_t __fb_filters[] = {
-  { NULL                  , "*"                        },
-  { NULL                  , "sub,srt"                  }, /* subtitles */
-  { NULL                  , "pls,m3u,sfv,tox,asx,smil" }, /* playlists */
-  { "*.vob"               , "vob"                      }, /* mpeg block */
-  { "*.mpv"               , "mpv"                      }, /* elementary */
-  { "*.mpg, *.mpeg, *.mpe", "mpg,mpeg,mpe"             }, /* mpeg */
-  { "*.avi"               , "avi"                      }, /* avi */
-  { "*.mp3"               , "mp3"                      }, /* mp3 */
-  { "*.asf, *.wma, *.wmv" , "asf,wma,wmv"              }, /* asf */
-  { "*.cpk, *.cak, *.film", "cpk,cak,film"             }, /* film */
-  { "*.ogg"               , "ogg"                      }, /* ogg */
-  { "*.vdr"               , "vdr"                      }, /* pes */
-  { "*.mov, *.mp4, *.qt"  , "mov,mp4,qt"               }, /* QT */
-  { "*.rm"                , "rm"                       }, /* rm */
-  { "*.roq"               , "roq"                      }, /* roq */
-  { "*.m2t, *.ts, *.trp"  , "m2t,ts,trp"               }, /* ts */
-  { "*.dv"                , "dv"                       }, /* dv */
-  { NULL                  , NULL                       }
+  { NULL, "*"                                                                      },
+ /* subtitles */
+  { NULL, ".sub .srt .asc .smi .ssa "                                              },
+  /* playlists */
+  { NULL, ".pls .m3u .sfv .tox .asx .smil "                                        },
+  { "*.4xm", ".4xm "                                                               },
+  { "*.ac3", ".ac3 "                                                               },
+  { "*.aif, *.aiff,", ".aif .aiff "                                                },
+  {"*.asf, *.wmv, *.wma, *.wvx, *.wax", ".asf .wmv .wma .wvx .wax "                },
+  { "*.aud", ".aud "                                                               },
+  { "*.avi", ".avi "                                                               },
+  { "*.cin,", ".cin "                                                              },
+  { "*.cpk, *.cak, *.film,", ".cpk .cak .film "                                    },
+  { "*.dv, *.dif", ".dv .dif "                                                     },
+  { "*.fli, *.flc,", ".fli .flc "                                                  },
+  { "*.mp3, *.mp2, *.mpa, *.mpega", ".mp3 .mp2 .mpa .mpega "                       },
+  { "*.mjpg", ".mjpg "                                                             },
+  { "*.mov, *.qt, *.mp4,", ".mov .qt .mp4 "                                        },
+  { "*.mpg, *.mpeg", ".mpg .mpeg "                                                 },
+  { "*.mpv", ".mpv "                                                               },
+  { "*.mve, *.mv8", ".mve .mv8 "                                                   },
+  { "*.nsf", ".nsf "                                                               },
+  { "*.nsv", ".nsv "                                                               },
+  { "*.ogg, *.ogm, *.spx,", ".ogg .ogm .spx "                                      },
+  { "*.pes", ".pes "                                                               },
+  { "*.png, *.mng,", ".png .mng "                                                  },
+  { "*.pva", ".pva "                                                               },
+  { "*.ra", ".ra "                                                                 },
+  { "*.rm, *.ram", ".rm .ram "                                                     },
+  { "*.roq", ".roq "                                                               },
+  { "*.str, *.iki, *.ik2, *.dps, *.dat, *.xa1, *.xa2, *.xas, *.xap, *.xa,",
+    ".str .iki .ik2 .dps .dat .xa1 .xa2 .xas .xap .xa "                            },
+  { "*.snd, *.au", ".snd .au "                                                     },
+  { "*.ts, *.m2t, *.trp,", ".ts .m2t .trp "                                        },
+  { "*.vqa,", ".vqa "                                                              },
+  { "*.vob", ".vob "                                                               },
+  { "*.voc", ".voc "                                                               },
+  { "*.vox", ".vox "                                                               },
+  { "*.wav", ".wav "                                                               },
+  { "*.wve", ".wve "                                                               },
+  { "*.y4m", ".y4m "                                                               },
+  { NULL, NULL                                                                     }
 };
 
 typedef struct {
@@ -233,7 +257,7 @@ static void fne_destroy(filename_editor_t *fne) {
     xitk_destroy_widgets(fne->widget_list);
     xitk_window_destroy_window(gGui->imlib_data, fne->xwin);
 
-    fne->xwin = None;
+    fne->xwin = NULL;
     xitk_list_free((XITK_WIDGET_LIST_LIST(fne->widget_list)));
     
     XLockDisplay(gGui->display);
@@ -367,12 +391,7 @@ static void fb_create_input_window(char *title, char *text,
   XUnlockDisplay(gGui->display);
   layer_above_video(xitk_window_get_window(fne->xwin));
   
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(fne->xwin)))
-    xine_usec_sleep(5000);
-  
-  XLockDisplay(gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(fne->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay(gGui->display);
+  try_to_set_input_focus(xitk_window_get_window(fne->xwin));
 }
 /*
  * ************************** END OF filename editor **************************
@@ -386,37 +405,24 @@ static void fb_create_input_window(char *title, char *text,
  * Return 1 if file match with current filter, otherwise 0.
  */
 static int is_file_match_to_filter(filebrowser_t *fb, char *file) {
-  char  *filter_ends, *m, *ending;
-  
+
   if(!fb->filter_selected)
     return 1;
-  
-  xine_strdupa(filter_ends, __fb_filters[fb->filter_selected].ending);
-  if((ending = strrchr(file, '.')) != NULL) {
-    
-    while((m = xine_strsep(&filter_ends, ",")) != NULL) {
+  else {
+    char *ending;
+
+    if((ending = strrchr(file, '.'))) {
+      char ext[strlen(ending) + 2];
       
-      if(!strcasecmp((ending + 1), m))
+      sprintf(ext, "%s ", ending);
+      if(strstr(__fb_filters[fb->filter_selected].ending, ext))
 	return 1;
       
     }
   }
-  
   return 0;
 }
 
-/*
- * Return 1 is filepathname is a directory, otherwise 0
- */
-static int is_a_dir(char *filepathname) {
-  struct stat  pstat;
-  
-  if((stat(filepathname, &pstat)) < 0)
-    return 0;
-
-  return (S_ISDIR(pstat.st_mode));
-}
-
 static void fb_update_origin(filebrowser_t *fb) {
   char   buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
   
@@ -440,14 +446,27 @@ static void fb_extract_path_and_file(filebrowser_t *fb, char *filepathname) {
   if(filepathname) {
     char *filename = NULL;
     char *_filepathname;
+    char *tilde;
     
-    xine_strdupa(_filepathname, filepathname);
-    
+    if((tilde = strchr(filepathname, '~')) && (*(tilde - 1) != '\\')) {
+      const char *homedir = xine_get_homedir();
+      int         len = strlen(homedir) + (tilde - filepathname);
+      
+      tilde++;
+      
+      if(*tilde == '/')
+	len += strlen(tilde);
+      
+      _filepathname = (char *) alloca(len + 1);
+      sprintf(_filepathname, "%s%s", homedir, (tilde) ? tilde : "");
+    }
+    else 
+      xine_strdupa(_filepathname, filepathname);
+
     if(!is_a_dir((char *)_filepathname))
       filename = strrchr(_filepathname, '/');
     
     if(filename && (strlen(filename) > 1)) {
-      
       *filename++ = '\0';
       sprintf(fb->filename, "%s", filename);
       
@@ -865,7 +884,7 @@ static void fb_exit(xitk_widget_t *w, void *data) {
     xitk_destroy_widgets(fb->widget_list);
     xitk_window_destroy_window(gGui->imlib_data, fb->xwin);
 
-    fb->xwin = None;
+    fb->xwin = NULL;
     xitk_list_free((XITK_WIDGET_LIST_LIST(fb->widget_list)));
     
     XLockDisplay(gGui->display);
@@ -1072,21 +1091,8 @@ static void fb_handle_events(XEvent *event, void *data) {
 }
 
 void filebrowser_raise_window(filebrowser_t *fb) {
-  if(fb != NULL) {
-    if(fb->xwin) {
-      if(fb->visible && fb->running) {
-	  XLockDisplay(gGui->display);
-	  XUnmapWindow(gGui->display, xitk_window_get_window(fb->xwin));
-	  XRaiseWindow(gGui->display, xitk_window_get_window(fb->xwin));
-	  XMapWindow(gGui->display, xitk_window_get_window(fb->xwin));
-	  if(!gGui->use_root_window)
-	    XSetTransientForHint (gGui->display, 
-				  xitk_window_get_window(fb->xwin), gGui->video_window);
-	  XUnlockDisplay(gGui->display);
-	  layer_above_video(xitk_window_get_window(fb->xwin));
-      }
-    }
-  }
+  if(fb != NULL)
+    raise_window(xitk_window_get_window(fb->xwin), fb->visible, fb->running);
 }
 
 void filebrowser_end(filebrowser_t *fb) {
@@ -1623,12 +1629,7 @@ filebrowser_t *create_filebrowser(char *window_title, char *filepathname,
     
   fb_getdir(fb);
 
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(fb->xwin)))
-    xine_usec_sleep(5000);
-
-  XLockDisplay(gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(fb->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay(gGui->display);
+  try_to_set_input_focus(xitk_window_get_window(fb->xwin));
 
   return fb;
 }
