@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: lang.h,v 1.4 2002/03/24 01:33:33 f1rmb Exp $
+ * $Id: lang.h,v 1.8 2003/11/22 14:03:35 valtri Exp $
  *
  */
 
@@ -33,16 +33,20 @@ typedef enum {
   PORTUGUESE,
   SPANISH,
   POLISH,
-  UKRAINIAN
+  UKRAINIAN,
+  CZECH
 } lang_code_t;
 
 typedef struct {
   char         *lang;
   lang_code_t   code;
   char         *ext;
+  char         *doc_encoding;
 } langs_t;
 
 const langs_t *get_lang(void);
 const langs_t *get_default_lang(void);
 
+const char *get_language_from_iso639_1(char *two_letters);
+
 #endif
