@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: lirc.c,v 1.14 2002/06/02 21:59:55 f1rmb Exp $
+ * $Id: lirc.c,v 1.25 2003/09/15 20:28:21 f1rmb Exp $
  *
  * lirc-specific stuff
  *
@@ -28,16 +28,14 @@
 
 #include <pthread.h>
 #include <signal.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <string.h>
 
-#include "event.h"
-#include "panel.h"
-#include "actions.h"
-#include "playlist.h"
-
-#include "xitk.h"
+#include "common.h"
 
 #ifdef HAVE_LIRC
-#include "lirc/lirc_client.h"
 
 #ifdef DEBUG
 #define LIRC_VERBOSE 1
@@ -59,64 +57,60 @@ static _lirc_t lirc;
 
 static void lirc_get_playlist(char *from) {
   int    i;
-  char **autoscan_plugins = xine_get_autoplay_input_plugin_ids(gGui->xine);
+  char **autoscan_plugins = (char **)xine_get_autoplay_input_plugin_ids(gGui->xine);
 
   for(i = 0; autoscan_plugins[i] != NULL; ++i) {
     if(!strcasecmp(autoscan_plugins[i], from)) {
       int    num_mrls;
       int    j;
-      char **autoplay_mrls = xine_get_autoplay_mrls (gGui->xine,
-						     from,
-						     &num_mrls);
+      char **autoplay_mrls = (char **)xine_get_autoplay_mrls (gGui->xine, from, &num_mrls);
       if(autoplay_mrls) {
 
 	/* First, free existing playlist */
-	for(j = 0; j < gGui->playlist_num; j++)
-	  gGui->playlist[j] = NULL;
-	
-	gGui->playlist_num = 0;
-	gGui->playlist_cur = 0;
+	mediamark_free_mediamarks();
 	
 	for (j = 0; j < num_mrls; j++)
-	  gGui->playlist[j] = autoplay_mrls[j];
+	  mediamark_add_entry((const char *)autoplay_mrls[j], 
+			      (const char *)autoplay_mrls[j], NULL, 0, -1, 0, 0);
 	
-	gGui->playlist_num = j;
-	gGui->playlist_cur = 0;
-	gui_set_current_mrl(gGui->playlist[gGui->playlist_cur]);
-	pl_update_playlist();
+	gGui->playlist.cur = 0;
+	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
+	playlist_update_playlist();
       }    
     }
   }
 }
 
-void *xine_lirc_loop(void *dummy) {
+static void *xine_lirc_loop(void *dummy) {
   char             *code, *c;
   int               ret;
   kbinding_entry_t *k;
-
-  /* I'm a poor lonesome pthread... */
-  pthread_detach(pthread_self());
+  fd_set            set;
+  struct timeval    tv;
 
   while(gGui->running) {
+
+    FD_ZERO(&set);
+    FD_SET(lirc.fd, &set);
     
-    pthread_testcancel();
+    tv.tv_sec  = 0;
+    tv.tv_usec = 500000;
+
+    select(lirc.fd + 1, &set, NULL, NULL, &tv);
     
     while(lirc_nextcode(&code) == 0) {
 
-      pthread_testcancel();
-      
       if(code == NULL) 
 	break;
       
-      pthread_testcancel();
-      
       while((ret = lirc_code2char(lirc.config, code, &c)) == 0 
 	    && c != NULL) {
 #if 0
 	fprintf(stdout, "Command Received = '%s'\n", c);
 #endif
-
+	
 	k = kbindings_lookup_action(gGui->kbindings, c);
+	
 	if(k)
 	  gui_execute_action_id((kbindings_get_action_id(k)));
 	else {
@@ -128,11 +122,14 @@ void *xine_lirc_loop(void *dummy) {
 	      lirc_get_playlist(from);   
 	  }
 	}
-	
       }
-      xitk_paint_widget_list (panel->widget_list);
-      free(code);
-
+      
+      if(panel_is_visible())
+	xitk_paint_widget_list (panel->widget_list);
+      
+      if(code)
+	free(code);
+      
       if(ret == -1) 
 	break;
 
@@ -142,40 +139,38 @@ void *xine_lirc_loop(void *dummy) {
   pthread_exit(NULL);
 }
 
-void init_lirc(void) {
-  /*  int flags; */
-
+void lirc_start(void) {
+  int flags;
+  int err;
+  
   if((lirc.fd = lirc_init("xine", LIRC_VERBOSE)) == -1) {
     gGui->lirc_enable = 0;
     return;
   }
-  /*
-  else {
-    flags = fcntl(gGui->lirc.fd, F_GETFL, 0);
-    if(flags != -1)
-      fcntl(gGui->lirc.fd, F_SETFL, flags|FASYNC|O_NONBLOCK);
-  }
-  */
-
+  
+  fcntl(lirc.fd, F_SETOWN, getpid());
+  flags = fcntl(lirc.fd, F_GETFL, 0);
+  if(flags != -1)
+    fcntl(lirc.fd, F_SETFL, flags|O_NONBLOCK);
+  
   if(lirc_readconfig(NULL, &lirc.config, NULL) != 0) {
     gGui->lirc_enable = 0;
+    lirc_deinit();
     return;
   }
 
-  gGui->lirc_enable = 1;
-
-  if(gGui->lirc_enable)
-    pthread_create (&lirc.thread, NULL, xine_lirc_loop, NULL) ;
-}
-
-void deinit_lirc(void) {
-
-  pthread_cancel(lirc.thread);
-
-  if(gGui->lirc_enable) {
+  if((err = pthread_create(&(lirc.thread), NULL, xine_lirc_loop, NULL)) != 0) {
+    printf(_("%s(): can't create new thread (%s)\n"), __XINE_FUNCTION__, strerror(err));
     lirc_freeconfig(lirc.config);
     lirc_deinit();
+    gGui->lirc_enable = 0;
   }
 }
 
+void lirc_stop(void) {
+  pthread_join(lirc.thread, NULL);
+  lirc_freeconfig(lirc.config);
+  lirc_deinit();
+}
+
 #endif
