@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: main.c,v 1.240 2003/07/22 21:33:08 f1rmb Exp $
+ * $Id: main.c,v 1.275 2003/12/23 16:05:15 siggi Exp $
  *
  * xine main for X11
  *
@@ -97,6 +97,8 @@ typedef struct {
 #define OPTION_DISABLE_POST     1009
 #define OPTION_NO_SPLASH        1010
 #define OPTION_STDCTL           1011
+#define OPTION_LIST_PLUGINS     1012
+#define OPTION_BUG_REPORT       1013
 
 
 /* options args */
@@ -110,7 +112,7 @@ static const char *short_options = "?hHgfvn"
 #ifdef HAVE_XF86VIDMODE
  "F"
 #endif
- "u:a:V:A:p::s:RG:BN:P:l::S:ZDr:c:E";
+ "u:a:V:A:p::s:RG:BN:P:l::S:ZD::r:c:ET:";
 
 static struct option long_options[] = {
   {"help"           , no_argument      , 0, 'h'                      },
@@ -143,7 +145,7 @@ static struct option long_options[] = {
   {"enqueue"        , no_argument      , 0, OPTION_ENQUEUE           },
   {"session"        , required_argument, 0, 'S'                      },
   {"version"        , no_argument      , 0, 'v'                      },
-  {"deinterlace"    , no_argument      , 0, 'D'                      },
+  {"deinterlace"    , optional_argument, 0, 'D'                      },
   {"aspect-ratio"   , required_argument, 0, 'r'                      },
   {"config"         , required_argument, 0, 'c'                      },
   {"verbose"        , optional_argument, 0, OPTION_VERBOSE           },
@@ -156,6 +158,9 @@ static struct option long_options[] = {
   {"post"           , required_argument, 0, OPTION_POST              },
   {"disable-post"   , no_argument      , 0, OPTION_DISABLE_POST      },
   {"no-splash"      , no_argument      , 0, OPTION_NO_SPLASH         },
+  {"tvout"          , required_argument, 0, 'T'                      },
+  {"list-plugins"   , optional_argument, 0, OPTION_LIST_PLUGINS      },
+  {"bug-report"     , optional_argument, 0, OPTION_BUG_REPORT        },
   {0                , no_argument      , 0, 0                        }
 };
 
@@ -225,7 +230,7 @@ static int _rc_file_get_next_line(file_info_t *rcfile) {
  * This allow user to always specify a command line argument.
  */
 static char **build_command_line_args(int argc, char *argv[], int *_argc) {
-  int            i;
+  int            i = 1, j;
   char          *cfgdir = CONFIGDIR;
   char          *xinerc = "xinerc";
   file_info_t   *rcfile;
@@ -233,9 +238,9 @@ static char **build_command_line_args(int argc, char *argv[], int *_argc) {
   
   _argv  = (char **) xine_xmalloc(sizeof(char **) * (argc + 1));
   (*_argc) = argc;
-  for(i = 0; i < argc; i++)
-    _argv[i] = strdup(argv[i]);
 
+  _argv[0] = strdup(argv[0]);
+  
   rcfile           = (file_info_t *) xine_xmalloc(sizeof(file_info_t));
   rcfile->filename = (char *) xine_xmalloc((strlen((xine_get_homedir())) + 
 					    strlen(cfgdir) + strlen(xinerc)) + 3);
@@ -245,12 +250,16 @@ static char **build_command_line_args(int argc, char *argv[], int *_argc) {
     
     while(_rc_file_get_next_line(rcfile)) {
       _argv = (char **) realloc(_argv, sizeof(char **) * ((*_argc) + 2));
-      _argv[(*_argc)] = strdup(rcfile->ln);
+      _argv[i] = strdup(rcfile->ln);
+      i++;
       (*_argc)++;
     }
     
     fclose(rcfile->fd);
   }
+  
+  for(j = 1; i < (*_argc); i++, j++)
+    _argv[i] = strdup(argv[j]);
 
   _argv[(*_argc)] = NULL;
   
@@ -389,11 +398,14 @@ static void xrm_parse(void) {
 static void main_change_logo_cb(void *data, xine_cfg_entry_t *cfg) {
   gGui->logo_mrl = cfg->str_value;
 }
+static void sub_autoload_cb(void *data, xine_cfg_entry_t *cfg) {
+  gGui->subtitle_autoload = cfg->num_value;
+}
 
 /*
  *
  */
-void show_version(void) {
+static void show_version(void) {
   
   printf(_("This is xine (X11 gui) - a free video player v%s"), VERSION);
 #ifdef DEBUG
@@ -405,30 +417,140 @@ void show_version(void) {
 /*
  *
  */
-void show_banner(void) {
+static void show_banner(void) {
   int major, minor, sub;
-
+  
   show_version();
-  printf(_("Built with xine library %d.%d.%d (%s)\n"),
-	 XINE_MAJOR_VERSION, XINE_MINOR_VERSION, XINE_SUB_VERSION, XINE_VERSION);
+
+  if(gGui->verbosity)
+    printf(_("Built with xine library %d.%d.%d (%s)\n"),
+	    XINE_MAJOR_VERSION, XINE_MINOR_VERSION, XINE_SUB_VERSION, XINE_VERSION);
   
   xine_get_version (&major, &minor, &sub);
 
-  printf(_("Found xine library version: %d.%d.%d (%s).\n"), 
-	 major, minor, sub, xine_get_version_string());
+  if(gGui->verbosity)
+    printf(_("Found xine library version: %d.%d.%d (%s).\n"), 
+	    major, minor, sub, xine_get_version_string());
+}
 
+static void print_formatted(char *title, const char *const *plugins) {
+  const char  *plugin;
+  char         buffer[81];
+  int          len;
+  char        *blanks = "     ";
+
+  printf(title);
+  
+  sprintf(buffer, "%s", blanks);
+  plugin = *plugins++;
+  
+  while(plugin) {
+    
+    len = strlen(buffer);
+    
+    if((len + (strlen(plugin) + 3)) < 80) {
+      sprintf(buffer, "%s%s%s", buffer, 
+	      (strlen(buffer) == strlen(blanks)) ? "" : ", ", plugin);
+    }
+    else {
+      printf(buffer);
+      printf(",\n");
+      sprintf(buffer, "%s%s", blanks, plugin);
+    }
+    
+    plugin = *plugins++;
+  }
+  
+  if(strlen(buffer))
+    printf(buffer);
+  
+  printf(".\n\n");
+}
+static void list_plugins(char *type) {
+  typedef struct {
+    const char *const *(*func)(xine_t *);
+    char                *name;
+    char                *type;
+  } _list_plugin_t;
+  const char   *const  *plugins;
+  xine_t               *xine;
+  char                 *cfgdir     = CONFIGDIR;
+  char                 *cfgfile    = CONFIGFILE;
+  char                 *configfile = NULL;
+  int                   i;
+  _list_plugin_t        list_functions[] = {
+    { xine_list_audio_output_plugins,  _("   -Audio output:\n"),    "audio_out"     },
+    { xine_list_video_output_plugins,  _("   -Video output:\n"),    "video_out"     },
+    { xine_list_demuxer_plugins,       _("   -Demuxer:\n"),         "demux"         },
+    { xine_list_input_plugins,         _("   -Input:\n"),           "input"         },
+    { xine_list_spu_plugins,           _("   -Subpicture:\n"),      "sub"           },
+    { xine_list_post_plugins,          _("   -Post processing:\n"), "post"          },
+    { xine_list_audio_decoder_plugins, _("   -Audio decoder:\n"),   "audio_decoder" },
+    { xine_list_video_decoder_plugins, _("   -Video decoder:\n"),   "video_decoder" },
+    { NULL,                            NULL,                        NULL            }
+  };
+    
+  configfile = (char *) xine_xmalloc(strlen(xine_get_homedir())
+				     + strlen(cfgdir) 
+				     + strlen(cfgfile)
+				     + 3);
+  sprintf(configfile, "%s/%s", xine_get_homedir(), cfgdir);
+  mkdir(configfile, 0755);
+  sprintf(configfile + strlen(configfile), "/%s", cfgfile);
+  
+  xine = xine_new();
+  xine_config_load(xine, configfile);
+  xine_init(xine);
+
+  show_version();
+
+  if(type && strlen(type)) {
+    i = 0;
+    
+    while(list_functions[i].name) {
+      
+      if(!strncasecmp(type, list_functions[i].type, strlen(type))) {
+	if((plugins = list_functions[i].func(xine))) {
+	  printf(_("\n Available xine's plugins:\n"));
+	  print_formatted(list_functions[i].name, plugins);
+	  goto __found;
+	}
+      }
+
+      i++;
+    }
+
+    printf(_("No available plugins found of %s type!.\n"), type);
+  __found: ;
+  }
+  else {
+    printf(_("\n Available xine's plugins:\n"));
+    
+    i = 0;
+    
+    while(list_functions[i].name) {
+
+      if((plugins = list_functions[i].func(xine)))
+	print_formatted(list_functions[i].name, plugins);
+      
+      i++;
+    }
+  }
+
+  xine_exit(xine);
 }
 
 /*
  *
  */
-void show_usage (void) {
+static void show_usage (void) {
   const char   *const *driver_ids;
   const char   *driver_id;
   xine_t       *xine;
   char         *cfgdir     = CONFIGDIR;
   char         *cfgfile    = CONFIGFILE;
   char         *configfile = NULL;
+  char        **backends, *backend;
   
   configfile = (char *) xine_xmalloc(strlen(xine_get_homedir())
 				     + strlen(cfgdir) 
@@ -451,36 +573,37 @@ void show_usage (void) {
   printf(_("  -c, --config <file>          Use config file instead of default one.\n"));
   printf(_("  -V, --video-driver <drv>     Select video driver by id. Available drivers: \n"));
   printf("                               ");
-  driver_ids = xine_list_video_output_plugins (xine);
-  driver_id  = *driver_ids++;
-  while (driver_id) {
-    printf ("%s ", driver_id);
+  if((driver_ids = xine_list_video_output_plugins (xine))) {
     driver_id  = *driver_ids++;
+    while (driver_id) {
+      printf ("%s ", driver_id);
+      driver_id  = *driver_ids++;
+    }
   }
   printf ("\n");
 
   printf(_("  -A, --audio-driver <drv>     Select audio driver by id. Available drivers: \n"));
   printf("                               null ");
-  driver_ids = xine_list_audio_output_plugins (xine);
-  driver_id  = *driver_ids++;
-  while (driver_id) {
-    printf ("%s ", driver_id);
+  if((driver_ids = xine_list_audio_output_plugins (xine))) {
     driver_id  = *driver_ids++;
+    while (driver_id) {
+      printf ("%s ", driver_id);
+      driver_id  = *driver_ids++;
+    }
   }
   printf ("\n");
-
   printf(_("  -u, --spu-channel <#>        Select SPU (subtitle) channel '#'.\n"));
   printf(_("  -a, --audio-channel <#>      Select audio channel '#'.\n"));
-  printf(_("  -p, --auto-play [opt]        Play on start. Can be followed by:\n"));
-  printf(_("                    'f': in fullscreen mode.\n"));
+  printf(_("  -p, --auto-play [opt]        Play on start. Can be followed by:\n"
+	   "                    'f': in fullscreen mode.\n"
+	   "                    'h': hide GUI (panel, etc.).\n"
+	   "                    'w': hide video window.\n"
+	   "                    'q': quit when play is done.\n"
+	   "                    'd': retrieve playlist from DVD. (deprecated. use -s DVD)\n"
+	   "                    'v': retrieve playlist from VCD. (deprecated. use -s VCD)\n"));
 #ifdef HAVE_XINERAMA
   printf(_("                    'F': in xinerama fullscreen mode.\n"));
 #endif
-  printf(_("                    'h': hide GUI (panel, etc.).\n"));
-  printf(_("                    'w': hide video window.\n"));
-  printf(_("                    'q': quit when play is done.\n"));
-  printf(_("                    'd': retrieve playlist from DVD. (deprecated. use -s DVD)\n"));
-  printf(_("                    'v': retrieve playlist from VCD. (deprecated. use -s VCD)\n"));
   printf(_("  -s, --auto-scan <plugin>     auto-scan play list from <plugin>\n"));
   printf(_("  -f, --fullscreen             start in fullscreen mode,\n"));
 #ifdef HAVE_XINERAMA
@@ -491,71 +614,98 @@ void show_usage (void) {
 #ifdef HAVE_LIRC
   printf(_("  -L, --no-lirc                Turn off LIRC support.\n"));
 #endif
-  printf(_("      --visual <class-or-id>   Use the specified X11 visual. <class-or-id>\n"));
-  printf(_("                               is either an X11 visual class, or a visual id.\n"));
+  printf(_("      --visual <class-or-id>   Use the specified X11 visual. <class-or-id>\n"
+	   "                               is either an X11 visual class, or a visual id.\n"));
   printf(_("      --install                Install a private colormap.\n"));
-  printf(_("      --keymap [=option]       Display keymap. Option are:\n"));
-  printf(_("                                 'default': display default keymap table,\n"));
-  printf(_("                                 'lirc': display draft of a .lircrc config file.\n"));
-  printf(_("                                 'remapped': user remapped keymap table.\n"));
-  printf(_("                                 'file:<filename>': use <filename> as keymap.\n"));
-  printf(_("                                 -if no option is given, 'default' is selected.\n"));
+  printf(_("      --keymap [=option]       Display keymap. Option are:\n"
+	   "                                 'default': display default keymap table,\n"
+	   "                                 'lirc': display draft of a .lircrc config file.\n"
+	   "                                 'remapped': user remapped keymap table.\n"
+	   "                                 'file:<filename>': use <filename> as keymap.\n"
+	   "                                 -if no option is given, 'default' is selected.\n"));
   printf(_("  -n, --network                Enable network remote control server.\n"));
   printf(_("  -R, --root                   Use root window as video window.\n"));
   printf(_("  -G, --geometry <WxH[+X+Y]>   Set output window geometry (X style).\n"));
   printf(_("  -B, --borderless             Borderless video output window.\n"));
-  printf(_("  -N, --animation <mrl>        Specify mrl to play when video output isn't used.\n"));
-  printf(_("                                 -can be used more than one time.\n"));
+  printf(_("  -N, --animation <mrl>        Specify mrl to play when video output isn't used.\n"
+	   "                                 -can be used more than one time.\n"));
   printf(_("  -P, --playlist <filename>    Load a playlist file.\n"));
-  printf(_("  -l, --loop [=mode]           Set playlist loop mode (default: loop). Modes are:\n"));
-  printf(_("                                 'loop': loop entire playlist.\n"));
-  printf(_("                                 'repeat': repeat current playlist entry.\n"));
-  printf(_("                                 'shuffle': select randomly a yet unplayed entry from playlist\n"));
-  printf(_("                                 'shuffle+': same as 'shuffle', but indefinetely replay the playlist.\n"));
+  printf(_("  -l, --loop [=mode]           Set playlist loop mode (default: loop). Modes are:\n"
+	   "                                 'loop': loop entire playlist.\n"
+	   "                                 'repeat': repeat current playlist entry.\n"
+	   "                                 'shuffle': select randomly a yet unplayed entry from playlist\n"
+	   "                                 'shuffle+': same as 'shuffle', but indefinetely replay the playlist.\n"));
   printf(_("      --skin-server-url <url>  Define the skin server url.\n"));
-  printf(_("      --enqueue <mrl>          Enqueue mrl of a running session (session 0)\n"));
-  printf(_("  -S, --session <option1,option2,...>\n"));
-  printf(_("                               Session managements. Options can be:\n"));
-  printf(_("                    session=n   specify session <n> number,\n"));
-  printf(_("                    mrl=m       add mrl <m> to the playlist,\n"));
-  printf(_("                    audio=c     select audio channel (<c>: 'next' or 'prev'),\n"));
-  printf(_("                    spu=c       select spu channel (<c>: 'next' or 'prev'),\n"));
-  printf(_("                    volume=v    set audio volume,\n"));
-  printf(_("                    amp=v       set amplification level,\n"));
-  printf(_("                    (playlist|pl)=p\n"));
-  printf(_("                                 <p> can be:\n"));
-  printf(_("                                   'clear'  clear the playlist,\n"));
-  printf(_("                                   'next'   play next playlist entry,\n"));
-  printf(_("                                   'prev'   play previous playlist entry,\n"));
-  printf(_("                                   'load:s' load playlist file <s>.\n"));
-  printf(_("                    play, slow2, slow4, pause, fast2,\n"));
-  printf(_("                    fast4, stop, quit, fullscreen, eject.\n"));
+  printf(_("      --enqueue <mrl> ...      Enqueue mrl of a running session (session 0)\n"));
+  printf(_("  -S, --session <option1,option2,...>\n"
+	   "                               Session managements. Options can be:\n"
+	   "                    session=n   specify session <n> number,\n"
+	   "                    mrl=m       add mrl <m> to the playlist,\n"
+	   "                    audio=c     select audio channel (<c>: 'next' or 'prev'),\n"
+	   "                    spu=c       select spu channel (<c>: 'next' or 'prev'),\n"
+	   "                    volume=v    set audio volume,\n"
+	   "                    amp=v       set amplification level,\n"
+	   "                    loop=m      set loop mode (<m>: 'none' 'loop' 'repeat' 'shuffle' or 'shuffle+'),\n"
+	   "                    (playlist|pl)=p\n"
+	   "                                 <p> can be:\n"
+	   "                                   'clear'  clear the playlist,\n"
+	   "                                   'first'  play first entry in the playlist,\n"
+	   "                                   'prev'   play previous playlist entry,\n"
+	   "                                   'next'   play next playlist entry,\n"
+	   "                                   'last'   play last entry in the playlist,\n"
+	   "                                   'load:s' load playlist file <s>,\n"
+	   "                                   'stop'   stop playback at the end of current playback.\n"
+	   "                                   'cont'   continue playback at the end of current playback.\n"
+	   "                    play, slow2, slow4, pause, fast2,\n"
+	   "                    fast4, stop, quit, fullscreen, eject.\n"));
   printf(_("  -Z                           Don't automatically start playback (smart mode).\n"));
-  printf(_("  -D, --deinterlace            Deinterlace video output\n"));
-  printf(_("  -r, --aspect-ratio <mode>    Set aspect ratio of video output. Modes are:\n"));
-  printf(_("                                 'auto', 'square', '4:3', 'anamorphic', 'dvb'.\n"));
+  printf(_("  -D, --deinterlace [post]...  Deinterlace video output. One or more post plugin\n"
+	   "                                 can be specified, with optional parameters.\n"
+	   "                                 Syntax is the same as --post option.\n"));
+  printf(_("  -r, --aspect-ratio <mode>    Set aspect ratio of video output. Modes are:\n"
+	   "                                 'auto', 'square', '4:3', 'anamorphic', 'dvb'.\n"));
 #ifdef XINE_PARAM_BROADCASTER_PORT
-  printf(_("      --broadcast-port <port>  Set port of xine broadcaster (master side)\n"));
-  printf(_("                               Slave is started with 'xine slave://address:port'\n"));
+  printf(_("      --broadcast-port <port>  Set port of xine broadcaster (master side)\n"
+	   "                               Slave is started with 'xine slave://address:port'\n"));
 #endif
   printf(_("      --no-logo                Don't display the logo.\n"));
   printf(_("  -E, --no-reload              Don't reload old playlist.\n"));
-  printf(_("      --post <plugin>[:parameter=value][,...]\n"));
-  printf(_("                               Load a post plugin.\n"));
-  printf(_("                               Parameters are comma separated.\n"));
-  printf(_("                               This option can be used more than one time.\n"));
+  printf(_("      --post <plugin>[:parameter=value][,...][;...]\n"
+	   "                               Load one or many post plugin(s).\n"
+	   "                               Parameters are comma separated.\n"
+	   "                               This option can be used more than one time.\n"));
   printf(_("      --disable-post           Don't enable post plugin(s).\n"));
   printf(_("      --no-splash              Don't display the splash screen.\n"));
   printf(_("      --stdctl                 Control xine by the console, using lirc commands.\n"));
+  printf(_("  -T, --tvout <backend>        Turn on tvout support. Backend can be:\n"));
+  printf("                               ");
+  if((backends = tvout_get_backend_names())) {
+    backend = *backends++;
+    while(backend) {
+      printf("%s ", backend);
+      backend = *backends++;
+    }
+  }
   printf("\n");
-  printf(_("examples for valid MRLs (media resource locator):\n"));
-  printf(_("  File:  'path/foo.vob'\n"));
-  printf(_("         '/path/foo.vob'\n"));
-  printf(_("         'file://path/foo.vob'\n"));
-  printf(_("         'fifo://[[mpeg1|mpeg2]:/]path/foo'\n"));
-  printf(_("         'stdin://[mpeg1|mpeg2]' or '-' (mpeg2 only)\n"));
-  printf(_("  DVD:   'dvd://VTS_01_2.VOB'\n"));
-  printf(_("  VCD:   'vcd://<track number>'\n"));
+  printf(_("      --list-plugins [=type]   Display the list of all available plugins,\n"
+	   "                               Optional <type> can be:\n"
+	   "                    audio_out, video_out, demux, input, sub, post,\n"
+	   "                    audio_decoder, video_decoder.\n"));
+  printf(_("      --bug-report [=mrl]      Enable bug report mode:\n"
+	   "                                 Turn on verbosity, gather all output\n"
+	   "                                 messages and write them into a file named\n"
+	   "                                 BUG-REPORT.TXT.\n"
+	   "                                 If <mrl> is given, xine will play the mrl\n"
+	   "                                 then quit (like -pq).\n"));
+  printf("\n\n");
+  printf(_("examples for valid MRLs (media resource locator):\n"
+	   "  File:  'path/foo.vob'\n"
+	   "         '/path/foo.vob'\n"
+	   "         'file://path/foo.vob'\n"
+	   "         'fifo://[[mpeg1|mpeg2]:/]path/foo'\n"
+	   "         'stdin://[mpeg1|mpeg2]' or '-' (mpeg2 only)\n"
+	   "  DVD:   'dvd://VTS_01_2.VOB'\n"
+	   "  VCD:   'vcd://<track number>'\n"));
   printf("\n");
 
   xine_exit(xine);
@@ -573,11 +723,12 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
   vis.display           = gGui->display;
   vis.screen            = gGui->screen;
   vis.d                 = gGui->video_window;
+  XLockDisplay(gGui->display);
   res_h                 = (DisplayWidth  (gGui->display, gGui->screen)*1000 
 			   / DisplayWidthMM (gGui->display, gGui->screen));
   res_v                 = (DisplayHeight (gGui->display, gGui->screen)*1000
 			   / DisplayHeightMM (gGui->display, gGui->screen));
-  
+  XUnlockDisplay(gGui->display);
   gGui->pixel_aspect    = res_v / res_h;
   
 #ifdef DEBUG
@@ -642,7 +793,8 @@ static xine_video_port_t *load_video_out_driver(int driver_number) {
 
     while (driver_ids[i]) {
       
-      printf (_("main: probing <%s> video output plugin\n"), driver_ids[i]);
+      if(gGui->verbosity)
+	printf (_("main: probing <%s> video output plugin\n"), driver_ids[i]);
       
       video_port = xine_open_video_driver(gGui->xine, 
 					  driver_ids[i],
@@ -742,7 +894,8 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
       
       /* don't want to load an audio driver ? */
       if (!strncasecmp(audio_driver_ids[driver_num], "NULL", 4)) {
-        printf(_("main: not using any audio driver (as requested).\n"));
+	if(gGui->verbosity)
+	  printf(_("main: not using any audio driver (as requested).\n"));
         return NULL;
       }
       
@@ -761,11 +914,14 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
 
     while (driver_ids[i]) {
       
-      if( strcmp(driver_ids[i], "none") )
-        printf (_("main: probing <%s> audio output plugin\n"), driver_ids[i]);
+      if(strcmp(driver_ids[i], "none")) {
+	if(gGui->verbosity)
+	  printf(_("main: probing <%s> audio output plugin\n"), driver_ids[i]);
+      }
       else {
-        printf (_("main: skipping <%s> audio output plugin from auto-probe\n"), driver_ids[i]);
-        i++;
+	if(gGui->verbosity)
+	  printf(_("main: skipping <%s> audio output plugin from auto-probe\n"), driver_ids[i]);
+	i++;
         continue;
       }
       
@@ -786,7 +942,8 @@ static xine_audio_port_t *load_audio_out_driver(int driver_number) {
     /* don't want to load an audio driver ? */
     if (!strncasecmp (audio_driver_ids[driver_number], "NULL", 4)) {
 
-      printf(_("main: not using any audio driver (as requested).\n"));
+      if(gGui->verbosity)
+	printf(_("main: not using any audio driver (as requested).\n"));
 
       /* calling -A null is useful to developers, but we should not save it at
        * config. if user doesn't have a sound card he may go to setup screen
@@ -828,7 +985,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
   gettimeofday (&tv, NULL);
   
   if(abs(tv.tv_sec - event->tv.tv_sec) > 3) {
-    printf("Event too old, discarding\n");
+    fprintf(stderr, "Event too old, discarding\n");
     return;
   }
   
@@ -847,13 +1004,12 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 
   /* frontend can e.g. move on to next playlist entry */
   case XINE_EVENT_UI_PLAYBACK_FINISHED:
-    /*    printf("xitk/main.c: playback finished\n"
-     *	   "             event->stream=%d,\n"
-     *	   "gGui->stream=%d, gGui->visual_anim.stream=%d\n",
-     *	   event->stream, gGui->stream, gGui->visual_anim.stream);
-     */
     if(event->stream == gGui->stream) {
-      /* printf("xitk/main.c: playing next stream...\n"); */
+
+      /* There a reference stream waiting to be played, just wait for */
+      if(gGui->got_reference_stream)
+	return;
+
       gui_playlist_start_next();
     }
     else if(event->stream == gGui->visual_anim.stream) {
@@ -887,6 +1043,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	}
 	gGui->mmk.ident = strdup(uevent->str);
 	
+	video_window_set_mrl(gGui->mmk.ident);
 	playlist_mrlident_toggle();
 	panel_update_mrl_display();
       }
@@ -943,7 +1100,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	  sprintf(buffer, "%s %s %s", buffer,
 		  (char *) data + data->explanation, (char *) data + data->parameters);
 	else
-	  sprintf(buffer, "%s %s", buffer, _("No Informations available."));
+	  sprintf(buffer, "%s %s", buffer, _("No Information available."));
 	  
 	break;
 
@@ -1013,6 +1170,16 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
+	/* (warning message) */
+      case XINE_MSG_SECURITY:
+	report = xine_info;
+	if(data->explanation)
+	  sprintf(buffer, "%s %s",
+		  (char *) data + data->explanation, (char *) data + data->parameters);
+	else
+	  sprintf(buffer, _("No Informations available."));
+	break;
+
       default:
 	sprintf(buffer, "%s", _("*sight*, unkown error."));
 	if(data->explanation)
@@ -1021,7 +1188,7 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	break;
       }
       
-      if(gGui->verbosity >= 2) {
+      if(gGui->verbosity >= XINE_VERBOSITY_DEBUG) {
 	sprintf(buffer, "%s\n\n[", buffer);
 	
 	if(data->explanation)
@@ -1048,7 +1215,10 @@ static void event_listener(void *user_data, const xine_event_t *event) {
       
       gGui->mixer.volume_level = (aevent->left + aevent->right) / 2;
       gGui->mixer.mute = aevent->mute;
-      xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
+
+      if(gGui->mixer.method == SOUND_CARD_MIXER)
+	xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
+      
       xitk_checkbox_set_state(panel->mixer.mute, gGui->mixer.mute);
     }
     break;
@@ -1074,31 +1244,31 @@ static void event_listener(void *user_data, const xine_event_t *event) {
   case XINE_EVENT_MRL_REFERENCE:
     if((event->stream == gGui->stream) && gGui->playlist.num) {
       xine_mrl_reference_data_t *ref = (xine_mrl_reference_data_t *) event->data;
-      
+
       if(ref->alternative == 0) {
 	mediamark_t *mmk = mediamark_clone_mmk((mediamark_t *) mediamark_get_current_mmk());
-	
+
 	if(mmk) {
 	  mediamark_replace_entry(&gGui->playlist.mmk[gGui->playlist.cur],
 				  ref->mrl, 
 				  ((mmk->ident && (strcmp(mmk->mrl, mmk->ident) == 0)) ?
 				   ref->mrl : mmk->ident), 
-				  mmk->sub, mmk->start, mmk->end, mmk->offset);
+				  mmk->sub, mmk->start, mmk->end, mmk->av_offset, mmk->spu_offset);
 	  mediamark_free_mmk(&mmk);
 	  
 	  mmk = (mediamark_t *) mediamark_get_current_mmk();
 	  gui_set_current_mrl(mmk);
+	  
+	  playlist_update_playlist();
+	  panel_update_mrl_display();
+	  
+	  /* We can't do anything more here, otherwise deadlock occur */
+	  gGui->got_reference_stream++;
 	}
       }
-      else {
-	mediamark_add_entry(ref->mrl, ref->mrl, NULL, 0, -1, 0);
-      }
-      
-      playlist_update_playlist();
-      panel_update_mrl_display();
+      else
+	mediamark_add_entry(ref->mrl, ref->mrl, NULL, 0, -1, 0, 0);
 
-      /* We can't do anything more here, otherwise deadlock occur */
-      gGui->got_reference_stream++;
     }
     break;
   }
@@ -1127,7 +1297,12 @@ int main(int argc, char *argv[]) {
   int                     old_playlist_cfg, no_old_playlist = 0;
   char                  **pplugins        = NULL;
   int                     pplugins_num    = 0;
-
+  char                   *tvout            = NULL;
+  char                   *pdeinterlace     = NULL;
+  int                     enable_deinterlace = 0;
+  char                  **session_argv     = NULL;
+  int                     session_argv_num = 0;
+  
 #ifdef HAVE_SETLOCALE
   if((xitk_set_locale()) != NULL)
     setlocale(LC_ALL, "");
@@ -1148,9 +1323,8 @@ int main(int argc, char *argv[]) {
 
   sigemptyset(&vo_mask);
   sigaddset(&vo_mask, SIGALRM);
-  if (sigprocmask (SIG_BLOCK,  &vo_mask, NULL)) {
-    printf (_("sigprocmask() failed.\n"));
-  }
+  if (sigprocmask (SIG_BLOCK,  &vo_mask, NULL))
+    fprintf (stderr, "sigprocmask() failed.\n");
 
   gGui = (gGui_t *) xine_xmalloc(sizeof(gGui_t));
   
@@ -1163,6 +1337,7 @@ int main(int argc, char *argv[]) {
   gGui->install_colormap       = 0;
   gGui->cursor_grabbed         = 0;
   gGui->network                = 0;
+  gGui->video_window           = None;
   gGui->use_root_window        = 0;
 #ifdef HAVE_XF86VIDMODE
   gGui->XF86VidMode_fullscreen = 0;
@@ -1170,6 +1345,7 @@ int main(int argc, char *argv[]) {
   gGui->actions_on_start[aos]  = ACTID_NOKEY;
   gGui->playlist.loop          = PLAYLIST_LOOP_NO_LOOP;
   gGui->playlist.on_start      = NULL;
+  gGui->playlist.control       = 0;
   gGui->skin_server_url        = NULL;
   gGui->verbosity              = 0;
   gGui->broadcast_port         = 0;
@@ -1181,6 +1357,8 @@ int main(int argc, char *argv[]) {
 #ifdef HAVE_LIRC
   gGui->lirc_enable            = 1;
 #endif
+  gGui->deinterlace_enable     = 0;
+  gGui->report                 = stdout;
 
   window_attribute.x     = window_attribute.y      = -8192;
   window_attribute.width = window_attribute.height = -1;
@@ -1246,7 +1424,7 @@ int main(int argc, char *argv[]) {
 	}
 #ifdef HAVE_XINERAMA
 	if(strrchr(optarg, 'F')) {
-	  gGui->actions_on_start[aos++] = ACTID_TOGGLE_XINERAMA_FULLSCR;
+	  gGui->actions_on_start[aos++] = ACTID_TOGGLE_XINERAMA_FULLSCREEN;
 	}
 #endif
 	if(strrchr(optarg, 'h')) {
@@ -1281,12 +1459,19 @@ int main(int argc, char *argv[]) {
 
 #ifdef HAVE_XINERAMA
     case 'F': /* xinerama full screen mode on start */
-      gGui->actions_on_start[aos++] = ACTID_TOGGLE_XINERAMA_FULLSCR;
+      gGui->actions_on_start[aos++] = ACTID_TOGGLE_XINERAMA_FULLSCREEN;
       break;
 #endif
 
     case 'D':
-      gGui->actions_on_start[aos++] = ACTID_TOGGLE_INTERLEAVE;
+      if(!pdeinterlace) {
+	enable_deinterlace++;
+	if(optarg) {
+	  char *p = xine_chomp(optarg);
+	  if(p && strlen(p))
+	    pdeinterlace = strdup(p);
+	}
+      }
       break;
       
     case 'r':
@@ -1412,7 +1597,7 @@ int main(int argc, char *argv[]) {
     case OPTION_SK_SERVER:
       gGui->skin_server_url = strdup(optarg);
       break;
-
+      
     case OPTION_ENQUEUE:
       if(is_remote_running(((session >= 0) ? session : 0))) {
 	if(session < 0)
@@ -1439,7 +1624,18 @@ int main(int argc, char *argv[]) {
       break;
 
     case 'S':
-      session_handle_subopt(optarg, &session);
+      if(is_remote_running(((session >= 0) ? session : 0)))
+	session_handle_subopt(optarg, &session);
+      else {
+
+	if(!session_argv_num)
+	  session_argv = (char **) xine_xmalloc(sizeof(char *) * 2);
+	else
+	  session_argv = (char **) realloc(session_argv, sizeof(char *) * (session_argv_num + 2));
+
+	session_argv[session_argv_num++] = strdup(optarg);
+	session_argv[session_argv_num]   = NULL;
+      }
       break;
 
     case 'Z':
@@ -1467,8 +1663,7 @@ int main(int argc, char *argv[]) {
       else
 	pplugins = (char **) realloc(pplugins, sizeof(char *) * (pplugins_num + 2));
       
-      pplugins[pplugins_num] = optarg;
-      pplugins_num++;
+      pplugins[pplugins_num++] = optarg;
       pplugins[pplugins_num] = NULL;
       break;
 
@@ -1494,7 +1689,68 @@ int main(int argc, char *argv[]) {
     case OPTION_STDCTL:
       gGui->stdctl_enable = 1;
       break;
+
+    case 'T':
+      tvout = optarg;
+      break;
       
+    case OPTION_LIST_PLUGINS:
+      {
+	char *p = NULL;
+	
+	if(optarg)
+	  p = xine_chomp(optarg);
+	
+	list_plugins(p);
+	exit(1);
+      }
+      break;
+
+    case OPTION_BUG_REPORT:
+      {
+	FILE   *f;
+
+	if(!(f = fopen("BUG-REPORT.TXT", "w+")))
+	  fprintf(stderr, "fopen(%s) failed: %s.\n", "BUG-REPORT.TXT", strerror(errno));
+	else {
+	  
+	  printf(_("*** NOTE ***\n"));
+	  printf(_(" Bug Report mode: All output messages will be added in BUG-REPORT.TXT file.\n"));
+
+#ifdef HAVE_SETLOCALE
+	if((xitk_set_locale()) != NULL)
+	  setlocale(LC_ALL, "C");
+#endif
+
+	  if (dup2((fileno(f)), STDOUT_FILENO) < 0)
+	    fprintf(stderr, "dup2() failed: %s.\n", strerror(errno));
+	  else {
+	    if (dup2((fileno(f)), STDERR_FILENO) < 0)
+	      fprintf(stdout, "dup2() failed: %s.\n", strerror(errno));
+	  }
+
+	  gGui->report = f;
+	  gGui->verbosity = 0xff;
+	}
+	
+	if(optarg) {
+	  char *p = xine_chomp(optarg);
+
+	  if(!session_argv_num)
+	    session_argv = (char **) xine_xmalloc(sizeof(char *) * 2);
+	  else
+	    session_argv = (char **) realloc(session_argv, sizeof(char *) * (session_argv_num + 2));
+	  
+	  session_argv[session_argv_num] = (char *) xine_xmalloc(strlen(p) + 5);
+	  sprintf(session_argv[session_argv_num], "mrl=%s", p);
+	  session_argv[++session_argv_num]   = NULL;
+
+	  gGui->actions_on_start[aos++] = ACTID_PLAY;
+	  gGui->actions_on_start[aos++] = ACTID_QUIT;
+	}
+      }
+      break;
+
     default:
       show_usage();
       fprintf (stderr, _("invalid argument %d => exit\n"), c);
@@ -1504,7 +1760,7 @@ int main(int argc, char *argv[]) {
   
   if(session >= 0) {
     /* Session feature was used, say good bye */
-    if(is_remote_running(session)) {
+    if(is_remote_running(session) && (_argc - optind)) {
       /* send all remaining MRL options to session */
       int i;
 
@@ -1571,6 +1827,7 @@ int main(int argc, char *argv[]) {
   }
   
   pthread_mutex_init(&gGui->xe_mutex, NULL);
+
   gGui->xine = xine_new();
   xine_config_load(gGui->xine, gGui->configfile);
   xine_engine_set_param(gGui->xine, XINE_ENGINE_PARAM_VERBOSITY, gGui->verbosity);
@@ -1585,7 +1842,7 @@ int main(int argc, char *argv[]) {
 			      _("If it's enabled and if you don't specify any MRL in command "
 				"line, xine will automatically load previous playlist."), 
 			      CONFIG_LEVEL_BEG,
-			      CONFIG_NO_CB,
+			      dummy_config_cb,
 			      CONFIG_NO_DATA);
 
   if(old_playlist_cfg && (!(_argc - optind)) && (!no_old_playlist)) {
@@ -1595,6 +1852,28 @@ int main(int argc, char *argv[]) {
     mediamark_load_mediamarks(buffer);
   }
 
+  gGui->subtitle_autoload = 
+    xine_config_register_bool(gGui->xine, "gui.subtitle_autoload", 1,
+			      _("Subtitle autoloading"),
+			      _("Automatically load subtitles if they exist."), 
+			      CONFIG_LEVEL_BEG,
+			      sub_autoload_cb,
+			      CONFIG_NO_DATA);
+
+
+  enable_deinterlace +=
+    xine_config_register_bool(gGui->xine, "gui.deinterlace_by_default", 0,
+			      _("Enable deinterlacing by default"),
+			      _("Deinterlace plugin will be enabled on "
+			        "startup. Progressive streams are automaticaly "
+			        "detected with no performance penalty."), 
+			        CONFIG_LEVEL_BEG,
+			        NULL,
+			        CONFIG_NO_DATA);
+
+  if( enable_deinterlace )
+    gGui->actions_on_start[aos++] = ACTID_TOGGLE_INTERLEAVE;
+
   /*
    * init gui
    */
@@ -1709,11 +1988,11 @@ int main(int argc, char *argv[]) {
     gGui->ao_port = load_audio_out_driver(driver_num);
   }
   SAFE_FREE(audio_driver_id);
+
+  post_deinterlace_init(pdeinterlace);
   post_init();
 
   gGui->stream = xine_stream_new(gGui->xine, gGui->ao_port, gGui->vo_port);
-  gGui->spu_stream = xine_stream_new(gGui->xine, NULL, gGui->vo_port);
-  xine_set_param(gGui->spu_stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
 
   osd_init();
 
@@ -1732,12 +2011,15 @@ int main(int argc, char *argv[]) {
   gGui->event_queue = xine_event_new_queue(gGui->stream);
   xine_event_create_listener_thread(gGui->event_queue, event_listener, NULL);
 
-  xine_tvmode_init(gGui->xine);
-  
+  if(tvout && strlen(tvout)) {
+    if((gGui->tvout = tvout_init(gGui->display, tvout)))
+      tvout_setup(gGui->tvout);
+  }
+
   xine_set_param(gGui->stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL, audio_channel);
   xine_set_param(gGui->stream, XINE_PARAM_SPU_CHANNEL, spu_channel);
   xine_set_param(gGui->stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
-  xine_set_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL, gGui->mixer.amp);
+  xine_set_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL, gGui->mixer.amp_level);
 #ifdef XINE_PARAM_BROADCASTER_PORT
   xine_set_param(gGui->stream, XINE_PARAM_BROADCASTER_PORT, gGui->broadcast_port);
 #endif
@@ -1755,9 +2037,13 @@ int main(int argc, char *argv[]) {
   gGui->playlist.scan_stream = xine_stream_new(gGui->xine, gGui->ao_port, gGui->vo_port);
   xine_set_param(gGui->playlist.scan_stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
   xine_set_param(gGui->playlist.scan_stream, XINE_PARAM_SPU_CHANNEL, -2);
-  
+
+  /* subtitle stream */
+  gGui->spu_stream = xine_stream_new(gGui->xine, NULL, gGui->vo_port);
+  xine_set_param(gGui->spu_stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
+
   /* init the video window */
-  video_window_select_visual ();
+  video_window_select_visual();
 
   xine_set_param(gGui->stream, XINE_PARAM_VO_ASPECT_RATIO, aspect_ratio);
   
@@ -1778,10 +2064,25 @@ int main(int argc, char *argv[]) {
     pplugin_rewire_posts();
   }
   
-  gui_run();
+  gui_run(session_argv);
 
   if(pplugins)
     free(pplugins);
   
+  if(session_argv_num) {
+    int i = 0;
+    
+    while(session_argv[i])
+      free(session_argv[i++]);
+    
+    free(session_argv);
+  }
+
+  pthread_mutex_destroy(&gGui->xe_mutex);
+  pthread_mutex_destroy(&gGui->download_mutex);
+
+  if(gGui->report)
+    fclose(gGui->report);
+
   return 0;
 }
