@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: menus.c,v 1.23 2003/05/05 22:15:01 f1rmb Exp $
+ * $Id: menus.c,v 1.38 2003/11/18 22:15:44 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -28,48 +28,72 @@
 
 #include "common.h"
 
-extern gGui_t             *gGui;
-extern _panel_t           *panel;
+extern gGui_t                  *gGui;
+extern _panel_t                *panel;
 
-#define PLAYB_PLAY         1
-#define PLAYB_STOP         2
-#define PLAYB_PAUSE        3
-#define PLAYB_NEXT         4
-#define PLAYB_PREV         5
-#define PLAYB_SPEEDM       6
-#define PLAYB_SPEEDL       7
+#define PLAYB_PLAY              1
+#define PLAYB_STOP              2
+#define PLAYB_PAUSE             3
+#define PLAYB_NEXT              4
+#define PLAYB_PREV              5
+#define PLAYB_SPEEDM            6
+#define PLAYB_SPEEDL            7
 
-#define PLAYL_LOAD        10
-#define PLAYL_SAVE        11
-#define PLAYL_EDIT        12
-#define PLAYL_NO_LOOP     13
-#define PLAYL_LOOP        14
-#define PLAYL_REPEAT      15
-#define PLAYL_SHUFFLE     16
-#define PLAYL_SHUF_PLUS   17
+#define PLAYL_LOAD              10
+#define PLAYL_SAVE              11
+#define PLAYL_EDIT              12
+#define PLAYL_NO_LOOP           13
+#define PLAYL_LOOP              14
+#define PLAYL_REPEAT            15
+#define PLAYL_SHUFFLE           16
+#define PLAYL_SHUF_PLUS         17
+#define PLAYL_CTRL_STOP         18
 
-#define AUDIO_MUTE        20
-#define AUDIO_INCRE_VOL   21
-#define AUDIO_DECRE_VOL   22
+#define AUDIO_MUTE              20
+#define AUDIO_INCRE_VOL         21
+#define AUDIO_DECRE_VOL         22
 
-#define VIDEO_FULLSCR     30
-#define VIDEO_2X          31
-#define VIDEO_1X          32
-#define VIDEO__5X         33
-#define VIDEO_INTERLEAVE  34
+#define VIDEO_FULLSCR           30
+#define VIDEO_2X                31
+#define VIDEO_1X                32
+#define VIDEO__5X               33
+#define VIDEO_INTERLEAVE        34
+#define VIDEO_PPROCESS          35
+#define VIDEO_PPROCESS_ENABLE   36
 
-#define SETS_SETUP        40
-#define SETS_KEYMAP       41
-#define SETS_VIDEO        42
-#define SETS_LOGS         43
-#define SETS_SKINDL       44
+#define SETS_SETUP              40
+#define SETS_KEYMAP             41
+#define SETS_VIDEO              42
+#define SETS_LOGS               43
+#define SETS_SKINDL             44
+#define SETS_TVANALOG           45
 
-#define STREAM_OSDI       50     
-#define STREAM_WINI       51
+#define STREAM_OSDI             50
+#define STREAM_WINI             51
 
 #define IS_CHANNEL_CHECKED(C, N) (C == N) ? "<checked>" : "<check>"
 
-
+static void menu_open_mrlbrowser(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  open_mrlbrowser_from_playlist(w, data);
+}
+static void menu_playlist_mmk_editor(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  playlist_mmk_editor();
+}
+static void menu_playlist_delete_all(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  playlist_delete_all(w, data);
+}
+static void menu_playlist_delete_current(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  playlist_delete_current(w, data);
+}
+static void menu_playlist_play_current(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  playlist_play_current(w, data);
+}
+static void menu_playlist_move_updown(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  playlist_move_current_updown(w, data);
+}
+static void menu_event_sender(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  gui_execute_action_id(ACTID_EVENT_SENDER);
+}
 static void menu_menus_selection(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
   int event     = (int) data;
   int events[7] = {
@@ -172,6 +196,10 @@ static void menu_playlist_ctrl(xitk_widget_t *w, xitk_menu_entry_t *me, void *da
     osd_display_info(_("Playlist: shuffle forever."));
     break;
 
+  case PLAYL_CTRL_STOP:
+    gui_execute_action_id(ACTID_PLAYLIST_STOP);
+    break;
+
   default:
     printf("%s(): unknown control %d\n", __XINE_FUNCTION__, ctrl);
     break;
@@ -195,7 +223,7 @@ static void menu_playlist_from(xitk_widget_t *w, xitk_menu_entry_t *me, void *da
       gGui->playlist.cur = 0;
     
     for (j = 0; j < num_mrls; j++)
-      mediamark_add_entry(autoplay_mrls[j], autoplay_mrls[j], NULL, 0, -1, 0);
+      mediamark_add_entry(autoplay_mrls[j], autoplay_mrls[j], NULL, 0, -1, 0, 0);
     
     if(gGui->playlist.cur == 0)
       gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
@@ -267,6 +295,12 @@ static void menu_audio_ctrl(xitk_widget_t *w, xitk_menu_entry_t *me, void *data)
     break;
   }
 }
+
+static void menu_audio_viz(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  int viz = (int) data;
+  
+  config_update_num("gui.post_audio_plugin", viz);
+}
 static void menu_audio_chan(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
   int channel = (int) data;
 
@@ -307,6 +341,14 @@ static void menu_video_ctrl(xitk_widget_t *w, xitk_menu_entry_t *me, void *data)
     gui_execute_action_id(ACTID_TOGGLE_INTERLEAVE);
     break;
 
+  case VIDEO_PPROCESS:
+    gui_execute_action_id(ACTID_VPP);
+    break;
+
+  case VIDEO_PPROCESS_ENABLE:
+    gui_execute_action_id(ACTID_VPP_ENABLE);
+    break;
+
   default:
     printf("%s(): unknown control %d\n", __XINE_FUNCTION__, ctrl);
     break;
@@ -329,6 +371,10 @@ static void menu_settings(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
     gui_execute_action_id(ACTID_CONTROLSHOW);
     break;
 
+  case SETS_TVANALOG:
+    gui_execute_action_id(ACTID_TVANALOG);
+    break;
+
   case SETS_LOGS:
     gui_execute_action_id(ACTID_VIEWLOG);
     break;
@@ -342,6 +388,9 @@ static void menu_settings(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
     break;
   }
 }
+static void menu_help(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  gui_execute_action_id(ACTID_HELP_SHOW);
+}
 static void menu_quit(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
   gui_execute_action_id(ACTID_QUIT);
 }
@@ -352,199 +401,240 @@ void video_window_menu(xitk_widget_list_t *wl) {
   xitk_menu_widget_t   menu;
   char                 buffer[2048];
   xitk_widget_t       *w;
+#ifdef HAVE_XINERAMA
+  int                  fullscr_mode = (FULLSCR_MODE | FULLSCR_XI_MODE);
+#else
+  int                  fullscr_mode = FULLSCR_MODE;
+#endif
   xitk_menu_entry_t    menu_entries[] = {
     { NULL ,
       "<title>",      
       NULL, NULL                                                                             },
-    { "GUI visiblity",
+    { _("GUI visiblity"),
       panel_is_visible() ? "<checked>" : "<check>",  
       menu_panel_visibility, NULL                                                            },
     { "SEP",  
       "<separator>",  
       NULL,  NULL                                                                            },
-    { "Open",
+    { _("Open"),
       "<branch>",   
       NULL, NULL                                                                             },
-    { "Open/File...",
+    { _("Open/File..."),
       NULL,
       menu_file_selector,        NULL                                                        },
-    { "Open/Location...",
-      NULL,
-      menu_mrl_browser, NULL                                                                 },
-    { "Playback",
-      "<Branch>",
-      NULL, NULL                                                                             },
-    { "Playback/Play",
-      NULL,
-      menu_playback_ctrl, (void *) PLAYB_PLAY                                                },
-    { "Playback/Stop",
-      NULL,
-      menu_playback_ctrl, (void *) PLAYB_STOP                                                },
-    { "Playback/Pause",
-      NULL,
-      menu_playback_ctrl, (void *) PLAYB_PAUSE                                               },
-    { "Playback/Next MRL",
-      NULL,
-      menu_playback_ctrl, (void *) PLAYB_NEXT                                                },
-    { "Playback/Previous MRL",
-      NULL,
-      menu_playback_ctrl, (void *) PLAYB_PREV                                                },
-    { "Playback/Increase Speed",
-      NULL,
-      menu_playback_ctrl, (void *) PLAYB_SPEEDM                                              },
-    { "Playback/Decrease Speed",
-      NULL,
-      menu_playback_ctrl, (void *) PLAYB_SPEEDL                                              },
-    { "Playlist",
-      "<branch>",
-      NULL, NULL                                                                             },
-    { "Playlist/Loop modes",
-      "<branch>",
-      NULL, NULL                                                                             },
-    { "Playlist/Loop modes/Disabled",
-      (gGui->playlist.loop == PLAYLIST_LOOP_NO_LOOP) ? "<checked>" : "<check>",
-      menu_playlist_ctrl, (void *) PLAYL_NO_LOOP                                             },
-    { "Playlist/Loop modes/Loop",
-      (gGui->playlist.loop == PLAYLIST_LOOP_LOOP) ? "<checked>" : "<check>",
-      menu_playlist_ctrl, (void *) PLAYL_LOOP                                                },
-    { "Playlist/Loop modes/Repeat Selection",
-      (gGui->playlist.loop == PLAYLIST_LOOP_REPEAT) ? "<checked>" : "<check>",
-      menu_playlist_ctrl, (void *) PLAYL_REPEAT                                              },
-    { "Playlist/Loop modes/Shuffle",
-      (gGui->playlist.loop == PLAYLIST_LOOP_SHUFFLE) ? "<checked>" : "<check>",
-      menu_playlist_ctrl, (void *) PLAYL_SHUFFLE                                             },
-    { "Playlist/Loop modes/Non-stop Shuffle",
-      (gGui->playlist.loop == PLAYLIST_LOOP_SHUF_PLUS) ? "<checked>" : "<check>",
-      menu_playlist_ctrl, (void *) PLAYL_SHUF_PLUS                                           },
-    { "Playlist/SEP",  
-      "<separator>",  
-      NULL, NULL                                                                             },
-    { "Playlist/Get from...",
-      "<branch>",
-      NULL, NULL                                                                             },
-    /*
-      autoplay input plugin idents      
-    */
-    { "Playlist/Load...",
+    { _("Open/Playlist..."),
       NULL,
       menu_playlist_ctrl, (void *) PLAYL_LOAD                                                },
-    { "Playlist/Editor...",
+    { _("Open/Location..."),
+      NULL,
+      menu_mrl_browser, NULL                                                                 },
+    { _("Playback"),
+      "<Branch>",
+      NULL, NULL                                                                             },
+    { _("Playback/Play"),
+      NULL,
+      menu_playback_ctrl, (void *) PLAYB_PLAY                                                },
+    { _("Playback/Stop"),
+      NULL,
+      menu_playback_ctrl, (void *) PLAYB_STOP                                                },
+    { _("Playback/Pause"),
+      NULL,
+      menu_playback_ctrl, (void *) PLAYB_PAUSE                                               },
+    { _("Playback/SEP"),
+      "<separator>",  
+      NULL,  NULL                                                                            },
+    { _("Playback/Next MRL"),
+      NULL,
+      menu_playback_ctrl, (void *) PLAYB_NEXT                                                },
+    { _("Playback/Previous MRL"),
+      NULL,
+      menu_playback_ctrl, (void *) PLAYB_PREV                                                },
+    { _("Playback/SEP"),
+      "<separator>",  
+      NULL,  NULL                                                                            },
+    { _("Playback/Increase Speed"),
+      NULL,
+      menu_playback_ctrl, (void *) PLAYB_SPEEDM                                              },
+    { _("Playback/Decrease Speed"),
+      NULL,
+      menu_playback_ctrl, (void *) PLAYB_SPEEDL                                              },
+    { _("Playlist"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Playlist/Get from"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Playlist/Load..."),
+      NULL,
+      menu_playlist_ctrl, (void *) PLAYL_LOAD                                                },
+    { _("Playlist/Editor..."),
       NULL,
       menu_playlist_ctrl, (void *) PLAYL_EDIT                                                },
+    { _("Playlist/SEP"),  
+      "<separator>",  
+      NULL, NULL                                                                             },
+    { _("Playlist/Loop modes"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Playlist/Loop modes/Disabled"),
+      (gGui->playlist.loop == PLAYLIST_LOOP_NO_LOOP) ? "<checked>" : "<check>",
+      menu_playlist_ctrl, (void *) PLAYL_NO_LOOP                                             },
+    { _("Playlist/Loop modes/Loop"),
+      (gGui->playlist.loop == PLAYLIST_LOOP_LOOP) ? "<checked>" : "<check>",
+      menu_playlist_ctrl, (void *) PLAYL_LOOP                                                },
+    { _("Playlist/Loop modes/Repeat Selection"),
+      (gGui->playlist.loop == PLAYLIST_LOOP_REPEAT) ? "<checked>" : "<check>",
+      menu_playlist_ctrl, (void *) PLAYL_REPEAT                                              },
+    { _("Playlist/Loop modes/Shuffle"),
+      (gGui->playlist.loop == PLAYLIST_LOOP_SHUFFLE) ? "<checked>" : "<check>",
+      menu_playlist_ctrl, (void *) PLAYL_SHUFFLE                                             },
+    { _("Playlist/Loop modes/Non-stop Shuffle"),
+      (gGui->playlist.loop == PLAYLIST_LOOP_SHUF_PLUS) ? "<checked>" : "<check>",
+      menu_playlist_ctrl, (void *) PLAYL_SHUF_PLUS                                           },
+    { _("Playlist/SEP"),  
+      "<separator>",
+      NULL, NULL                                                                             },
+    { _("Playlist/Continue"),
+      (gGui->playlist.control & PLAYLIST_CONTROL_STOP) ? "<check>" : "<checked>",
+      menu_playlist_ctrl, (void *) PLAYL_CTRL_STOP                                           },
     { "SEP",  
       "<separator>",
       NULL, NULL                                                                             },
-    { "Stream",
+    { _("Menus"),
       "<branch>",
       NULL, NULL                                                                             },
-    { "Stream/OSD Information",
+    { _("Menus/Navigation..."),
       NULL,
-      menu_stream, (void *) STREAM_OSDI                                                      },
-    { "Stream/Window Information...",
+      menu_event_sender, NULL                                                                },
+    { _("Menus/SEP"),  
+      "<separator>",
+      NULL, NULL                                                                             },
+    { "SEP",  
+      "<separator>",
+      NULL, NULL                                                                             },
+    { _("Stream"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Stream/Information..."),
       NULL,
       menu_stream, (void *) STREAM_WINI                                                      },
-    { "SEP",  
-      "<separator>",
-      NULL, NULL                                                                             },
-    { "Menus",
-      "<branch>",
-      NULL, NULL                                                                             },
-    { "SEP",  
-      "<separator>",
-      NULL, NULL                                                                             },
-    { "Volume",
-      "<branch>",
-      NULL, NULL                                                                             },
-    { "Volume/Mute",
-      gGui->mixer.mute ? "<checked>" : "<check>",
-      menu_audio_ctrl, (void *) AUDIO_MUTE                                                   },
-    { "Volume/Increase 10%",
+    { _("Stream/Information (OSD)"),
       NULL,
-      menu_audio_ctrl, (void *) AUDIO_INCRE_VOL                                              },
-    { "Volume/Decrease 10%",
-      NULL,
-      menu_audio_ctrl, (void *) AUDIO_DECRE_VOL                                              },
-    { "Audio Channel",
-      "<branch>",
-      NULL, NULL                                                                             },
-    /*
-      audio channels
-    */
-    { "Sub Channel",
-      "<branch>",
-      NULL, NULL                                                                             },
-    /*
-      spu channels
-    */
-    { "SEP",
-      "<separator>",
-      NULL, NULL                                                                             },
-    { "Fullscreen\\/Window",
-      video_window_get_fullscreen_mode() ? "<checked>" : "<check>",
+      menu_stream, (void *) STREAM_OSDI                                                      },
+    { _("Fullscreen\\/Window"),
+      (video_window_get_fullscreen_mode() & fullscr_mode) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_FULLSCR                                                },
-    { "Video",
+    { _("Video"),
       "<branch>",
       NULL, NULL                                                                             },
-    { "Video/Deinterlace",
+    { _("Video/Deinterlace"),
       (xine_get_param(gGui->stream, XINE_PARAM_VO_DEINTERLACE)) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_INTERLEAVE                                             },
-    { "Video/SEP",
+    { _("Video/SEP"),
       "<separator>",
       NULL, NULL                                                                             },
-    { "Video/Aspect ratio",
+    { _("Video/Aspect ratio"),
       "<branch>",
       NULL, NULL                                                                             },
-    { "Video/Aspect ratio/Automatic",
+    { _("Video/Aspect ratio/Automatic"),
       (aspect == XINE_VO_ASPECT_AUTO) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_AUTO                                              },
-    { "Video/Aspect ratio/Square",
+    { _("Video/Aspect ratio/Square"),
       (aspect == XINE_VO_ASPECT_SQUARE) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_SQUARE                                            },
-    { "Video/Aspect ratio/4:3",
+    { _("Video/Aspect ratio/4:3"),
       (aspect == XINE_VO_ASPECT_4_3) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_4_3                                               },
-    { "Video/Aspect ratio/Anamorphic",
+    { _("Video/Aspect ratio/Anamorphic"),
       (aspect == XINE_VO_ASPECT_ANAMORPHIC) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_ANAMORPHIC                                        },
-    { "Video/Aspect ratio/DVB",
+    { _("Video/Aspect ratio/DVB"),
       (aspect == XINE_VO_ASPECT_DVB) ? "<checked>" : "<check>",
       menu_aspect, (void *) XINE_VO_ASPECT_DVB                                               },
-    { "Video/200%",
+    { _("Video/200%"),
       (video_window_get_mag() == 2.0) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_2X                                                     },
-    { "Video/100%",
+    { _("Video/100%"),
       (video_window_get_mag() == 1.0) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO_1X                                                     },
-    { "Video/50%",
+    { _("Video/50%"),
       (video_window_get_mag() ==  .5) ? "<checked>" : "<check>",
       menu_video_ctrl, (void *) VIDEO__5X                                                    },
-    { "SEP", 
+    { _("Video/SEP"),
       "<separator>",
       NULL, NULL                                                                             },
-    { "Settings",
+    { _("Video/Postprocess"),
       "<branch>",
       NULL, NULL                                                                             },
-    { "Settings/Setup...",
+    { _("Video/Postprocess/Chain Reaction..."),
+      NULL,
+      menu_video_ctrl, (void *) VIDEO_PPROCESS                                               },
+    { _("Video/Postprocess/Enable Postprocessing"),
+      gGui->post_enable ? "<checked>" : "<check>",
+      menu_video_ctrl, (void *) VIDEO_PPROCESS_ENABLE                                        },
+    { _("Audio"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Audio/Volume"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Audio/Volume/Mute"),
+      gGui->mixer.mute ? "<checked>" : "<check>",
+      menu_audio_ctrl, (void *) AUDIO_MUTE                                                   },
+    { _("Audio/Volume/Increase 10%"),
+      NULL,
+      menu_audio_ctrl, (void *) AUDIO_INCRE_VOL                                              },
+    { _("Audio/Volume/Decrease 10%"),
+      NULL,
+      menu_audio_ctrl, (void *) AUDIO_DECRE_VOL                                              },
+    { _("Audio/Channel"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Audio/Visualization"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Subtitle"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Subtitle/Channel"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { "SEP",  
+      "<separator>",
+      NULL, NULL                                                                             },
+    { _("Settings"),
+      "<branch>",
+      NULL, NULL                                                                             },
+    { _("Settings/Setup..."),
       NULL,
       menu_settings, (void *) SETS_SETUP                                                     },
 #ifdef HAVE_CURL
-    { "Settings/Skin Downloader...",
+    { _("Settings/Skin Downloader..."),
       NULL,
       menu_settings, (void *) SETS_SKINDL                                                    },
 #endif
-    { "Settings/Keymap Editor...",
+    { _("Settings/Keymap Editor..."),
       NULL,
       menu_settings, (void *) SETS_KEYMAP                                                    },
-    { "Settings/Video...",
+    { _("Settings/Video..."),
       NULL,
       menu_settings, (void *) SETS_VIDEO                                                     },
-    { "Settings/Logs...",
+    { _("Settings/TV Analog..."),
+      NULL,
+      menu_settings, (void *) SETS_TVANALOG                                                  },
+    { "SEP",
+      "<separator>",
+      NULL, NULL                                                                             },
+    { _("Help..."),
+      NULL,
+      menu_help, NULL                                                                        },
+    { _("Logs..."),
       NULL,
       menu_settings, (void *) SETS_LOGS                                                      },
     { "SEP",
       "<separator>",
       NULL, NULL                                                                             },
-    { "Quit",
+    { _("Quit"),
       NULL,
       menu_quit, NULL                                                                        },
     { NULL,
@@ -569,14 +659,14 @@ void video_window_menu(xitk_widget_list_t *wl) {
   if(gGui->playlist.num) {
     xitk_menu_entry_t   menu_entry;
     
-    menu_entry.menu      = "Open/Subtitle...";
+    menu_entry.menu      = _("Open/Subtitle...");
     menu_entry.type      = NULL;
     menu_entry.cb        = menu_subtitle_selector;
     menu_entry.user_data = NULL;
     xitk_menu_add_entry(w, &menu_entry);
 
     /* Save Playlist */
-    menu_entry.menu      = "Playlist/Save...";
+    menu_entry.menu      = _("Playlist/Save...");
     menu_entry.type      = NULL;
     menu_entry.cb        = menu_playlist_ctrl;
     menu_entry.user_data = (void *) PLAYL_SAVE;
@@ -586,7 +676,7 @@ void video_window_menu(xitk_widget_list_t *wl) {
   { /* Autoplay plugins */
     xitk_menu_entry_t   menu_entry;
     char                buffer[2048];
-    char               *location = "Playlist/Get from...";
+    char               *location = _("Playlist/Get from");
     const char *const  *plug_ids = xine_get_autoplay_input_plugin_ids(gGui->xine);
     const char         *plug_id;
     
@@ -606,27 +696,57 @@ void video_window_menu(xitk_widget_list_t *wl) {
     }
   }
 
+  { /* Audio Viz */
+    xitk_menu_entry_t   menu_entry;
+    const char        **viz_names = post_get_audio_plugins_names();
+
+    if(viz_names && *viz_names) {
+      int                 i = 0;
+      char               *location = _("Audio/Visualization");
+      char                buffer[2048];
+
+      while(viz_names[i]) {
+	sprintf(buffer, "%s/%s", location, viz_names[i]);
+	menu_entry.menu      = buffer;
+	menu_entry.type      = IS_CHANNEL_CHECKED(i, gGui->visual_anim.post_plugin_num);
+	menu_entry.cb        = menu_audio_viz;
+	menu_entry.user_data = (void *) i;
+	xitk_menu_add_entry(w, &menu_entry);
+	i++;
+      }
+
+    }
+    else {
+      menu_entry.menu      = _("Audio/Visualization/None");
+      menu_entry.type      = NULL;
+      menu_entry.cb        = NULL;
+      menu_entry.user_data = NULL;
+      xitk_menu_add_entry(w, &menu_entry);
+    }
+    
+  }
+
   { /* Audio channels */
     xitk_menu_entry_t   menu_entry;
     int                 i;
-    char               *location = "Audio Channel";
+    char               *location = _("Audio/Channel");
     char                buffer[2048];
     int                 channel = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
     
-    menu_entry.menu      = "Audio Channel/Off";
+    menu_entry.menu      = _("Audio/Channel/Off");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -2);
     menu_entry.cb        = menu_audio_chan;
     menu_entry.user_data = (void *) -2;
     xitk_menu_add_entry(w, &menu_entry);
     
-    menu_entry.menu      = "Audio Channel/Auto";
+    menu_entry.menu      = _("Audio/Channel/Auto");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -1);
     menu_entry.cb        = menu_audio_chan;
     menu_entry.user_data = (void *) -1;
     xitk_menu_add_entry(w, &menu_entry);
     
     for(i = 0; i < 32; i++) {
-      char   langbuf[16];
+      char   langbuf[XINE_LANG_MAX];
       
       memset(&langbuf, 0, sizeof(langbuf));
       
@@ -659,24 +779,24 @@ void video_window_menu(xitk_widget_list_t *wl) {
   { /* SPU channels */
     xitk_menu_entry_t   menu_entry;
     int                 i;
-    char               *location = "Sub Channel";
+    char               *location = _("Subtitle/Channel");
     char                buffer[2048];
     int                 channel = xine_get_param(gGui->stream, XINE_PARAM_SPU_CHANNEL);
     
-    menu_entry.menu      = "Sub Channel/Off";
+    menu_entry.menu      = _("Subtitle/Channel/Off");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -2);
     menu_entry.cb        = menu_spu_chan;
     menu_entry.user_data = (void *) -2;
     xitk_menu_add_entry(w, &menu_entry);
     
-    menu_entry.menu      = "Sub Channel/Auto";
+    menu_entry.menu      = _("Subtitle/Channel/Auto");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -1);
     menu_entry.cb        = menu_spu_chan;
     menu_entry.user_data = (void *) -1;
     xitk_menu_add_entry(w, &menu_entry);
     
     for(i = 0; i < 32; i++) {
-      char   langbuf[16];
+      char   langbuf[XINE_LANG_MAX];
       
       memset(&langbuf, 0, sizeof(langbuf));
       
@@ -709,12 +829,14 @@ void video_window_menu(xitk_widget_list_t *wl) {
   { /* Menus access */
     xitk_menu_entry_t   menu_entry;
     char                buffer[2048];
-    char               *location = "Menus";
+    char               *location = _("Menus");
     char               *default_menu[8] = {
-      "Menu 1", "Menu 2", "Menu 3", "Menu 4", "Menu 5", "Menu 6", "Menu 7", NULL
+      _("Menu 1"), _("Menu 2"), _("Menu 3"), _("Menu 4"),
+      _("Menu 5"), _("Menu 6"), _("Menu 7"), NULL
     };
     char               *dvd_menu[8] = {
-      "Menu toggle", "Title", "Root", "Subpicture", "Audio", "Angle", "Part", NULL
+      _("Menu toggle"), _("Title"), _("Root"), _("Subpicture"), 
+      _("Audio"), _("Angle"), _("Part"), NULL
     };
     char              **menu = default_menu;
     int                 i;
@@ -763,20 +885,20 @@ void audio_lang_menu(xitk_widget_list_t *wl, int x, int y) {
     char                buffer[2048];
     int                 channel = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_CHANNEL_LOGICAL);
     
-    menu_entry.menu      = "Off";
+    menu_entry.menu      = _("Off");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -2);
     menu_entry.cb        = menu_audio_chan;
     menu_entry.user_data = (void *) -2;
     xitk_menu_add_entry(w, &menu_entry);
     
-    menu_entry.menu      = "Auto";
+    menu_entry.menu      = _("Auto");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -1);
     menu_entry.cb        = menu_audio_chan;
     menu_entry.user_data = (void *) -1;
     xitk_menu_add_entry(w, &menu_entry);
     
     for(i = 0; i < 32; i++) {
-      char  langbuf[16];
+      char  langbuf[XINE_LANG_MAX];
       
       memset(&langbuf, 0, sizeof(langbuf));
       
@@ -835,20 +957,20 @@ void spu_lang_menu(xitk_widget_list_t *wl, int x, int y) {
     char                buffer[2048];
     int                 channel = xine_get_param(gGui->stream, XINE_PARAM_SPU_CHANNEL);
     
-    menu_entry.menu      = "Off";
+    menu_entry.menu      = _("Off");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -2);
     menu_entry.cb        = menu_spu_chan;
     menu_entry.user_data = (void *) -2;
     xitk_menu_add_entry(w, &menu_entry);
     
-    menu_entry.menu      = "Auto";
+    menu_entry.menu      = _("Auto");
     menu_entry.type      = IS_CHANNEL_CHECKED(channel, -1);
     menu_entry.cb        = menu_spu_chan;
     menu_entry.user_data = (void *) -1;
     xitk_menu_add_entry(w, &menu_entry);
     
     for(i = 0; i < 32; i++) {
-      char   langbuf[16];
+      char   langbuf[XINE_LANG_MAX];
       
       memset(&langbuf, 0, sizeof(langbuf));
       
@@ -879,3 +1001,59 @@ void spu_lang_menu(xitk_widget_list_t *wl, int x, int y) {
 
   xitk_menu_show_menu(w);
 }
+
+void playlist_menu(xitk_widget_list_t *wl, int x, int y, int selected) {
+  xitk_menu_widget_t   menu;
+  xitk_widget_t       *w = NULL;
+  char                 buffer[2048];
+  xitk_menu_entry_t    menu_entries_nosel[] = {
+    { NULL ,           "<title>",     NULL,                         NULL                    },
+    { "SEP",           "<separator>", NULL,                         NULL                    },
+    { _("Add"),        NULL,          menu_open_mrlbrowser,         NULL                    },
+    { NULL,            NULL,          NULL,                         NULL                    }
+  };
+  xitk_menu_entry_t    menu_entries_sel[] = {
+    { NULL ,           "<title>",     NULL,                         NULL                    },
+    { "SEP",           "<separator>", NULL,                         NULL                    },
+    { _("Play"),       NULL,          menu_playlist_play_current,   NULL                    },
+    { "SEP",           "<separator>", NULL,                         NULL                    },
+    { _("Add"),        NULL,          menu_open_mrlbrowser,         NULL                    },
+    { _("Edit"),       NULL,          menu_playlist_mmk_editor,     NULL                    },
+    { _("Delete"),     NULL,          menu_playlist_delete_current, NULL                    },
+    { _("Delete All"), NULL,          menu_playlist_delete_all,     NULL                    },
+    { "SEP",           "<separator>", NULL,                         NULL                    },
+    { _("Move Up"),    NULL,          menu_playlist_move_updown,    (void *) DIRECTION_UP   },
+    { _("Move Down"),  NULL,          menu_playlist_move_updown,    (void *) DIRECTION_DOWN },
+    { NULL,            NULL,          NULL,                         NULL                    }
+  };
+
+  
+  XITK_WIDGET_INIT(&menu, gGui->imlib_data);
+  
+  sprintf(buffer, "%s", _("Playlist"));
+  if(selected) {
+    menu_entries_sel[0].menu = buffer;
+    menu.menu_tree           = &menu_entries_sel[0];
+  }
+  else {
+    menu_entries_nosel[0].menu = buffer;
+    menu.menu_tree             = &menu_entries_nosel[0];
+  }
+  
+  menu.parent_wlist      = wl;
+  menu.skin_element_name = NULL;
+
+  w = xitk_noskin_menu_create(wl, &menu, x, y);
+
+  if(!selected && gGui->playlist.num) {
+    xitk_menu_entry_t   menu_entry;
+    
+    menu_entry.menu      = _("Delete All");
+    menu_entry.type      = NULL;
+    menu_entry.cb        = menu_playlist_delete_all;
+    menu_entry.user_data = NULL;
+    xitk_menu_add_entry(w, &menu_entry);
+  }
+
+  xitk_menu_show_menu(w);
+}
