@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrl_browser.c,v 1.65 2003/05/05 22:15:01 f1rmb Exp $
+ * $Id: mrl_browser.c,v 1.68 2003/10/21 19:17:57 f1rmb Exp $
  *
  * MRL Browser
  *
@@ -345,7 +345,7 @@ static void mrl_add_noautoplay(xitk_widget_t *w, void *data, xine_mrl_t *mrl) {
 	playlist_toggle_visibility(NULL, NULL);
     }
     
-    mediamark_add_entry((char *)mrl->mrl, (char *)mrl->mrl, NULL, 0, -1, 0);
+    mediamark_add_entry((char *)mrl->mrl, (char *)mrl->mrl, NULL, 0, -1, 0, 0);
     
     if((!num) && ((xine_get_status(gGui->stream) == XINE_STATUS_STOP) || gGui->logo_mode)) {
       gGui->playlist.cur = (gGui->playlist.num - 1);
@@ -392,7 +392,7 @@ static void mrl_play(xitk_widget_t *w, void *data, xine_mrl_t *mrl) {
     if(!is_playback_widgets_enabled())
       enable_playback_controls(1);
 
-    if(mrl_looks_playlist(_mrl)) {
+    if(mrl_look_like_playlist(_mrl)) {
       if(mediamark_concat_mediamarks(_mrl)) {
 	gui_set_current_mrl((mediamark_t *)mediamark_get_current_mmk());
 	_mrl = (char *) mediamark_get_current_mrl();
