@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: panel.c,v 1.179 2003/05/02 22:30:58 f1rmb Exp $
+ * $Id: panel.c,v 1.197 2003/11/18 16:48:22 f1rmb Exp $
  *
  * xine panel related stuff
  *
@@ -370,13 +370,22 @@ static void *slider_loop(void *dummy) {
 	    }
 	    
 	    if(!(i % 20)) {
+	      int level = 0;
+	      
 	      panel_update_channel_display();
 	      
-	      if(gGui->mixer.caps & MIXER_CAP_VOL) {
-		gGui->mixer.volume_level = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
-		xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
-		panel_check_mute();
+	      if((gGui->mixer.method == SOUND_CARD_MIXER) &&
+		 (gGui->mixer.caps & MIXER_CAP_VOL)) {
+		level = gGui->mixer.volume_level = 
+		  xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
 	      }
+	      else if(gGui->mixer.method == SOFTWARE_MIXER) {
+		level = gGui->mixer.amp_level = 
+		  xine_get_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL);
+	      }
+	      
+	      xitk_slider_set_pos(panel->mixer.slider, level);
+	      panel_check_mute();
 	      
 	      i = 0;
 	    }
@@ -387,7 +396,7 @@ static void *slider_loop(void *dummy) {
 	    stream_infos_update_infos();
 	}
       }
-
+      
       if(gGui->got_reference_stream) {
 	gGui->got_reference_stream--;
 	gui_play(NULL, NULL);
@@ -396,8 +405,13 @@ static void *slider_loop(void *dummy) {
     }
     
     if(gGui->cursor_visible) {
-      gGui->cursor_visible = !gGui->cursor_visible;
-      video_window_set_cursor_visibility(gGui->cursor_visible);
+      if(!(i % 2))
+	video_window_set_cursor_timer(video_window_get_cursor_timer() + 1);
+      
+      if(video_window_get_cursor_timer() >= 5) {
+	gGui->cursor_visible = !gGui->cursor_visible;
+	video_window_set_cursor_visibility(gGui->cursor_visible);
+      }
     }
     
     if(gGui->logo_has_changed)
@@ -489,6 +503,27 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
       stream_infos_toggle_visibility(NULL, NULL);
   }
 
+  if(!panel->visible && tvset_is_visible()) {
+  }
+  else {
+    if(!gGui->use_root_window || (gGui->use_root_window && visible && tvset_is_visible()))
+      tvset_toggle_visibility(NULL, NULL);
+  }
+
+  if(!panel->visible && pplugin_is_visible()) {
+  }
+  else {
+    if(!gGui->use_root_window || (gGui->use_root_window && visible && pplugin_is_visible()))
+      pplugin_toggle_visibility(NULL, NULL);
+  }
+
+  if(!panel->visible && help_is_visible()) {
+  }
+  else {
+    if(!gGui->use_root_window || (gGui->use_root_window && visible && help_is_visible()))
+      help_toggle_visibility(NULL, NULL);
+  }
+
   if (panel->visible) {
     
     XLockDisplay(gGui->display);
@@ -512,12 +547,12 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
        XGrabPointer(gGui->display, gGui->video_window, 1, None, GrabModeAsync, GrabModeAsync, gGui->video_window, None, CurrentTime);
     
     /* Give focus to video output window */
-    XSetInputFocus(gGui->display, gGui->video_window, RevertToParent, CurrentTime);
+    try_to_set_input_focus(gGui->video_window);
     XUnlockDisplay(gGui->display);
      
   }
   else {
-    
+
     panel->visible = 1;
     xitk_show_widgets(panel->widget_list);
     
@@ -537,27 +572,46 @@ static void _panel_toggle_visibility (xitk_widget_t *w, void *data) {
       XUnlockDisplay (gGui->display);
     }
     
-    /* TODO: Currently this is a quick hack
-     * We should rather test whether screen size has changed
-     * and move the panel on screen if it doesn't fit any longer */
-    if (video_window_get_fullscreen_mode() > 1 
+#if defined(HAVE_XINERAMA) || defined(HAVE_XF86VIDMODE)
+    if(
+#ifdef HAVE_XINERAMA
+       (((video_window_get_fullscreen_mode()) & (WINDOWED_MODE | FULLSCR_MODE)) &&
+	(!((video_window_get_fullscreen_mode()) & FULLSCR_XI_MODE)))
 #ifdef HAVE_XF86VIDMODE
-        || gGui->XF86VidMode_fullscreen
+       ||
 #endif
-	) {
-    /*
-     * necessary to place the panel in a visible area (otherwise it might
-     * appear off the video window while switched to a differenti
-     * modeline or tv mode)
-     */
-      XLockDisplay (gGui->display);
-      XMoveWindow(gGui->display, gGui->panel_window, 40, 40);
-      XUnlockDisplay (gGui->display);
+#endif
+#ifdef HAVE_XF86VIDMODE
+       (gGui->XF86VidMode_fullscreen)
+#endif
+       ) {
+      int x, y, w, h, desktopw, desktoph;
+      
+      xitk_get_window_position(gGui->display, gGui->panel_window, &x, &y, &w, &h);
+      
+      XLockDisplay(gGui->display);
+      desktopw = DisplayWidth(gGui->display, gGui->screen);
+      desktoph = DisplayHeight(gGui->display, gGui->screen);
+      XUnlockDisplay(gGui->display);
+      
+      if(((x + w) <= 0) || ((y + h) <= 0) || (x >= desktopw) || (y >= desktoph)) {
+	int newx, newy;
+	
+	newx = (desktopw - w) >> 1;
+	newy = (desktoph - h) >> 1;
+	
+	XLockDisplay(gGui->display);
+	XMoveWindow(gGui->display, gGui->panel_window, newx, newy);
+	XUnlockDisplay(gGui->display);
+	
+	panel_store_new_position(newx, newy, w, h);
+      }
     }
-
+#endif
+    
     if(gGui->logo_mode == 0) {
       int pos;
-
+      
       if(xitk_is_widget_enabled(panel->playback_widgets.slider_play)) {
 	if(gui_xine_get_pos_length(gGui->stream, &pos, NULL, NULL))
 	  xitk_slider_set_pos(panel->playback_widgets.slider_play, pos);
@@ -619,7 +673,7 @@ void panel_reset_slider (void) {
  */
 void panel_update_channel_display (void) {
   int   channel;
-  char  buffer[10];
+  char  buffer[XINE_LANG_MAX];
   char *lang = NULL;
 
   memset(&buffer, 0, sizeof(buffer));
@@ -740,13 +794,16 @@ static void panel_slider_cb(xitk_widget_t *w, void *data, int pos) {
 	  xitk_slider_set_pos(panel->playback_widgets.slider_play, pos);
 	
 	panel_update_runtime_display();
-        }
       }
+    }
   }
   else if(w == panel->mixer.slider) {
-    gGui->mixer.volume_level = pos;
-    xine_set_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME, gGui->mixer.volume_level);
-    osd_draw_bar(_("Audio Volume"), 0, 100, gGui->mixer.volume_level, OSD_BAR_STEPPER);
+
+    if(gGui->mixer.method == SOUND_CARD_MIXER)
+      change_audio_vol(pos);
+    else if(gGui->mixer.method == SOFTWARE_MIXER)
+      change_amp_vol(pos);
+    
   }
   
   panel_check_pause();
@@ -758,7 +815,15 @@ static void panel_slider_cb(xitk_widget_t *w, void *data, int pos) {
 void panel_handle_event(XEvent *event, void *data) {
 
   switch(event->type) {
+  case ButtonPress:
+    {
+      XButtonEvent *bevent = (XButtonEvent *) event;
 
+      if(bevent->button == Button3)
+	video_window_menu(panel->widget_list);
+    }
+    break;
+    
   case KeyPress:
   case ButtonRelease:
     gui_handle_event(event, data);
@@ -855,11 +920,19 @@ void panel_add_mixer_control(void) {
     if((xine_get_param(gGui->stream, XINE_PARAM_AUDIO_MUTE)) != -1)
       gGui->mixer.caps |= MIXER_CAP_MUTE;
   }
+  
+  if(((gGui->mixer.method == SOUND_CARD_MIXER) && gGui->mixer.caps & MIXER_CAP_VOL) ||
+     (gGui->mixer.method == SOFTWARE_MIXER)) {
+    int level = 0;
 
-  if(gGui->mixer.caps & MIXER_CAP_VOL) { 
     xitk_enable_widget(panel->mixer.slider);
-    gGui->mixer.volume_level = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
-    xitk_slider_set_pos(panel->mixer.slider, gGui->mixer.volume_level);
+    
+    if(gGui->mixer.method == SOUND_CARD_MIXER)
+      level = gGui->mixer.volume_level = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_VOLUME);
+    else if(gGui->mixer.method == SOFTWARE_MIXER)
+      level = gGui->mixer.amp_level = xine_get_param(gGui->stream, XINE_PARAM_AUDIO_AMP_LEVEL);
+    
+    xitk_slider_set_pos(panel->mixer.slider, level);
   }
 
   if(gGui->mixer.caps & MIXER_CAP_MUTE) {
@@ -1227,7 +1300,7 @@ void panel_init (void) {
   /* Mixer volume slider */
   sl.skin_element_name = "SliderVol";
   sl.min               = 0;
-  sl.max               = 100;
+  sl.max               = (gGui->mixer.method == SOUND_CARD_MIXER) ? 100 : 200;
   sl.step              = 1;
   sl.callback          = NULL;
   sl.userdata          = NULL;
@@ -1357,14 +1430,14 @@ void panel_init (void) {
   
   {
     pthread_attr_t       pth_attrs;
-#if ! defined (__OpenBSD__)
+#if !defined (__OpenBSD__)
     struct sched_param   pth_params;
 #endif
     
     pthread_attr_init(&pth_attrs);
 
     /* this won't work on linux, freebsd 5.0 */
-#if ! defined (__OpenBSD__)
+#if !defined (__OpenBSD__)
     pthread_attr_getschedparam(&pth_attrs, &pth_params);
     pth_params.sched_priority = sched_get_priority_min(SCHED_OTHER);
     pthread_attr_setschedparam(&pth_attrs, &pth_params);
@@ -1373,14 +1446,8 @@ void panel_init (void) {
     pthread_create(&panel->slider_thread, &pth_attrs, slider_loop, NULL);
   }
 
-  if(panel->visible) {
-    while(!xitk_is_window_visible(gGui->display, gGui->panel_window))
-      xine_usec_sleep(5000);
-  
-    XLockDisplay (gGui->display);
-    XSetInputFocus(gGui->display, gGui->panel_window, RevertToParent, CurrentTime);
-    XUnlockDisplay (gGui->display);
-  }
+  if(panel->visible)
+    try_to_set_input_focus(gGui->panel_window);
 }
 
 void panel_set_title(char *title) {
