@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: setup.c,v 1.78 2003/02/14 21:25:55 mroi Exp $
+ * $Id: setup.c,v 1.87 2003/11/29 23:18:35 f1rmb Exp $
  *
  *
  */
@@ -26,29 +26,29 @@
 #endif
 
 #include <stdio.h>
-#include <errno.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
-#include <pthread.h>
-#include <assert.h>
+#include <X11/cursorfont.h>
 
 #include "common.h"
 
-extern gGui_t              *gGui;
+extern gGui_t                   *gGui;
 
-static char                *fontname     = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
-static char                *boldfontname = "-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*";
-static char                *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
+static char                     *fontname     = "-*-helvetica-medium-r-*-*-10-*-*-*-*-*-*-*";
+static char                     *boldfontname = "-*-helvetica-bold-r-*-*-10-*-*-*-*-*-*-*";
+static char                     *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
 
-#define WINDOW_WIDTH        500
-#define WINDOW_HEIGHT       480
+#define WINDOW_WIDTH             500
+#define WINDOW_HEIGHT            500
 
-#define FRAME_WIDTH         350
-#define FRAME_HEIGHT        40
+#define FRAME_WIDTH              350
+#define FRAME_HEIGHT             40
 
-#define MAX_DISPLAY_WIDGETS 8
-#define BROWSER_MAX_DISP_ENTRIES 17
+#define MAX_DISPLAY_WIDGETS      8
+
+#define NORMAL_CURS              0
+#define WAIT_CURS                1
 
 #define ADD_FRAME(title) {                                                                      \
     xitk_widget_t       *frame = NULL;                                                          \
@@ -66,7 +66,6 @@ static char                *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*
     xitk_font_unload_font(fs);                                                                  \
                                                                                                 \
     XLockDisplay(gGui->display);                                                                \
-                                                                                                \
     XSetForeground(gGui->display, (XITK_WIDGET_LIST_GC(setup->widget_list)),                    \
 		   xitk_get_pixel_color_gray(gGui->imlib_data));                                \
     XFillRectangle(gGui->display, image->image->pixmap,                                         \
@@ -90,13 +89,16 @@ static char                *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*
   }
 
 #define ADD_LABEL(widget, cb, data) {                                                           \
-    int           wx, wy, wh, fh;                                                               \
-    xitk_font_t  *fs;                                                                           \
+    int            wx, wy, wh, fh;                                                              \
+    xitk_font_t   *fs;                                                                          \
     xitk_widget_t *lbl;                                                                         \
+    char           _labelkey[strlen(labelkey) + 5];                                             \
+                                                                                                \
+    sprintf(_labelkey, "%s%s", labelkey, (!entry->callback) ? " (*)" : "");                     \
                                                                                                 \
     fs = xitk_font_load_font(gGui->display, fontname);                                          \
     xitk_font_set_font(fs, (XITK_WIDGET_LIST_GC(setup->widget_list)));                          \
-    fh = xitk_font_get_string_height(fs, labelkey);                                             \
+    fh = xitk_font_get_string_height(fs, _labelkey);                                            \
     xitk_font_unload_font(fs);                                                                  \
                                                                                                 \
     xitk_get_widget_pos(widget, &wx, &wy);                                                      \
@@ -104,40 +106,35 @@ static char                *tabsfontname = "-*-helvetica-bold-r-*-*-12-*-*-*-*-*
     wh = xitk_get_widget_height(widget);                                                        \
                                                                                                 \
     lbl = setup_add_label (wx + 20, (wy + (wh >> 1)) - (fh>>1),                                 \
-                           (FRAME_WIDTH - wx), labelkey, cb, data);                             \
+                           (FRAME_WIDTH - (xitk_get_widget_width(widget) + 35)),                \
+                           _labelkey, cb, data);                                                \
     wt->label = lbl;                                                                            \
   }
 
 #define DISABLE_ME(wtriplet) {                                                                  \
-    if((wtriplet)->frame) {                                                                     \
-      xitk_disable_widget((wtriplet)->frame);                                                   \
-      xitk_hide_widget((wtriplet)->frame);                                                      \
-    }                                                                                           \
+    if((wtriplet)->frame)                                                                       \
+      xitk_disable_and_hide_widget((wtriplet)->frame);                                          \
+                                                                                                \
     if((wtriplet)->label) {                                                                     \
-      xitk_disable_widget((wtriplet)->label);                                                   \
-      xitk_hide_widget((wtriplet)->label);                                                      \
+      xitk_disable_and_hide_widget((wtriplet)->label);                                          \
       xitk_disable_widget_tips((wtriplet)->label);                                              \
     }                                                                                           \
     if((wtriplet)->widget) {                                                                    \
-      xitk_disable_widget((wtriplet)->widget);                                                  \
-      xitk_hide_widget((wtriplet)->widget);                                                     \
+      xitk_disable_and_hide_widget((wtriplet)->widget);                                         \
       xitk_disable_widget_tips((wtriplet)->widget);                                             \
     }                                                                                           \
 }
 
 #define ENABLE_ME(wtriplet) {                                                                   \
-    if((wtriplet)->frame) {                                                                     \
-      xitk_enable_widget((wtriplet)->frame);                                                    \
-      xitk_show_widget((wtriplet)->frame);                                                      \
-    }                                                                                           \
+    if((wtriplet)->frame)                                                                       \
+      xitk_enable_and_show_widget((wtriplet)->frame);                                           \
+                                                                                                \
     if((wtriplet)->label) {                                                                     \
-      xitk_enable_widget((wtriplet)->label);                                                    \
-      xitk_show_widget((wtriplet)->label);                                                      \
+      xitk_enable_and_show_widget((wtriplet)->label);                                           \
       xitk_enable_widget_tips((wtriplet)->label);                                               \
     }                                                                                           \
     if((wtriplet)->widget) {                                                                    \
-      xitk_enable_widget((wtriplet)->widget);                                                   \
-      xitk_show_widget((wtriplet)->widget);                                                     \
+      xitk_enable_and_show_widget((wtriplet)->widget);                                          \
       xitk_enable_widget_tips((wtriplet)->widget);                                              \
     }                                                                                           \
 }
@@ -148,7 +145,7 @@ typedef struct {
   xitk_widget_t        *label;
   xitk_widget_t        *widget;
   int                   changed;
-  xine_cfg_entry_t     *cfg;
+  xine_cfg_entry_t  *cfg;
 } widget_triplet_t;
 
 typedef struct {
@@ -159,6 +156,7 @@ typedef struct {
   int                   visible;
 
   xitk_widget_t        *tabs;
+  xitk_widget_t        *ok;
 
   char                 *sections[20];
   int                   num_sections;
@@ -171,12 +169,7 @@ typedef struct {
   int                   num_wg;
   int                   first_displayed;
 
-  const char          **config_content;
-  int                   config_lines;
-  const char          **readme_content;
-  int                   readme_lines;
-  const char          **faq_content;
-  int                   faq_lines;
+  Cursor                cursor[2];
 
   xitk_register_key_t   kreg;
 
@@ -185,6 +178,7 @@ typedef struct {
 static _setup_t    *setup = NULL;
 
 static void setup_end(xitk_widget_t *, void *);
+static void setup_change_section(xitk_widget_t *, void *, int);
 
 static void add_widget_to_list(xitk_widget_t *w) {
   xitk_list_append_content(setup->widgets, (void *) w);  
@@ -194,58 +188,39 @@ static void add_widget_to_list(xitk_widget_t *w) {
  * Leaving setup panel, release memory.
  */
 void setup_exit(xitk_widget_t *w, void *data) {
-  window_info_t wi;
-  int           i;
-  
-  setup->running = 0;
-  setup->visible = 0;
 
-  if((xitk_get_window_info(setup->kreg, &wi))) {
-    config_update_num ("gui.setup_x", wi.x);
-    config_update_num ("gui.setup_y", wi.y);
-    WINDOW_INFO_ZERO(&wi);
-  }
-
-  xitk_unregister_event_handler(&setup->kreg);
-
-  xitk_destroy_widgets(setup->widget_list);
-  xitk_window_destroy_window(gGui->imlib_data, setup->xwin);
-
-  setup->xwin = None;
-  xitk_list_free((XITK_WIDGET_LIST_LIST(setup->widget_list)));
-  xitk_list_free(setup->widgets);
-  free(setup->widgets);
-  
-  XLockDisplay(gGui->display);
-  XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(setup->widget_list)));
-  XUnlockDisplay(gGui->display);
-
-  free(setup->widget_list);
-  
-  if(setup->config_content) {
-    for(i = 0; i < setup->config_lines; i++) {
-      free((char *)setup->config_content[i]);
+  if(setup) {
+    window_info_t wi;
+    
+    setup->running = 0;
+    setup->visible = 0;
+    
+    if((xitk_get_window_info(setup->kreg, &wi))) {
+      config_update_num ("gui.setup_x", wi.x);
+      config_update_num ("gui.setup_y", wi.y);
+      WINDOW_INFO_ZERO(&wi);
     }
-    free(setup->config_content);
+    
+    xitk_unregister_event_handler(&setup->kreg);
+    
+    xitk_destroy_widgets(setup->widget_list);
+    xitk_window_destroy_window(gGui->imlib_data, setup->xwin);
+    
+    setup->xwin = NULL;
+    xitk_list_free((XITK_WIDGET_LIST_LIST(setup->widget_list)));
+    xitk_list_free(setup->widgets);
+    
+    XLockDisplay(gGui->display);
+    XFreeCursor(gGui->display, setup->cursor[NORMAL_CURS]);
+    XFreeCursor(gGui->display, setup->cursor[WAIT_CURS]);
+    XFreeGC(gGui->display, (XITK_WIDGET_LIST_GC(setup->widget_list)));
+    XUnlockDisplay(gGui->display);
+    
+    free(setup->widget_list);
+    
+    free(setup);
+    setup = NULL;
   }
-
-  if(setup->faq_content) {
-    for(i = 0; i < setup->faq_lines; i++) {
-      free((char *)setup->faq_content[i]);
-    }
-    free(setup->faq_content);
-  }
-
-  if(setup->readme_content) {
-    for(i = 0; i < setup->readme_lines; i++) {
-      free((char **)setup->readme_content[i]);
-    }
-    free(setup->readme_content);
-  }
-
-  free(setup);
-  setup = NULL;
-
 }
 
 /*
@@ -278,63 +253,21 @@ int setup_is_visible(void) {
  * Raise setup->xwin
  */
 void setup_raise_window(void) {
-  
-  if(setup != NULL) {
-    if(setup->xwin) {
-      if(setup->visible && setup->running) {
-	XLockDisplay(gGui->display);
-	XUnmapWindow(gGui->display, xitk_window_get_window(setup->xwin));
-	XRaiseWindow(gGui->display, xitk_window_get_window(setup->xwin));
-	XMapWindow(gGui->display, xitk_window_get_window(setup->xwin));
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, 
-				xitk_window_get_window(setup->xwin), gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(xitk_window_get_window(setup->xwin));
-      }
-    }
-  }
+  if(setup != NULL)
+    raise_window(xitk_window_get_window(setup->xwin), setup->visible, setup->running);
 }
 
 /*
  * Hide/show the setup panel
  */
 void setup_toggle_visibility (xitk_widget_t *w, void *data) {
-  
-  if(setup != NULL) {
-    if (setup->visible && setup->running) {
-      XLockDisplay(gGui->display);
-      if(gGui->use_root_window) {
-	if(xitk_is_window_visible(gGui->display, xitk_window_get_window(setup->xwin)))
-	  XIconifyWindow(gGui->display, xitk_window_get_window(setup->xwin), gGui->screen);
-	else
-	  XMapWindow(gGui->display, xitk_window_get_window(setup->xwin));
-      }
-      else {
-	setup->visible = 0;
-	xitk_hide_widgets(setup->widget_list);
-	XUnmapWindow (gGui->display, xitk_window_get_window(setup->xwin));
-      }
-      XUnlockDisplay(gGui->display);
-    } 
-    else {
-      if(setup->running) {
-	setup->visible = 1;
-	xitk_show_widgets(setup->widget_list);
-	XLockDisplay(gGui->display);
-	XRaiseWindow(gGui->display, xitk_window_get_window(setup->xwin)); 
-	XMapWindow(gGui->display, xitk_window_get_window(setup->xwin)); 
-	if(!gGui->use_root_window)
-	  XSetTransientForHint (gGui->display, 
-				xitk_window_get_window(setup->xwin), gGui->video_window);
-	XUnlockDisplay(gGui->display);
-	layer_above_video(xitk_window_get_window(setup->xwin));
-      }
-    }
-  }
+  if(setup != NULL)
+    toggle_window(xitk_window_get_window(setup->xwin), setup->widget_list,
+		  &setup->visible, setup->running);
 }
 
 static void setup_apply(xitk_widget_t *w, void *data) {
+  int need_restart = 0;
 
   if(setup->num_wg > 0) {
     int i;
@@ -350,6 +283,11 @@ static void setup_apply(xitk_widget_t *w, void *data) {
 	  int            numval = 0;
 	  char          *strval = NULL;
 	  
+	  if(!need_restart) {
+	    if(setup->wg[i]->changed && (!setup->wg[i]->cfg->callback))
+	      need_restart = 1;
+	  }
+
 	  setup->wg[i]->changed = 0;
 	  
 	  switch(type & WIDGET_TYPE_MASK) {
@@ -396,6 +334,36 @@ static void setup_apply(xitk_widget_t *w, void *data) {
       }
     }
     config_save();
+
+    if(w != setup->ok)
+      setup_change_section(setup->tabs, NULL, xitk_tabs_get_current_selected(setup->tabs));
+
+    if(need_restart) {
+      xitk_window_t *xw;
+      
+      xw = xitk_window_dialog_ok(gGui->imlib_data, _("Important Notice"),
+				 NULL, NULL,
+				 ALIGN_CENTER,
+				 _("You changed some configuration value which require"
+				   " to restart xine to take effect."));
+      if(!gGui->use_root_window) {
+	XLockDisplay(gGui->display);
+	XSetTransientForHint(gGui->display, xitk_window_get_window(xw), gGui->video_window);
+	XUnlockDisplay(gGui->display);
+      }
+      layer_above_video(xitk_window_get_window(xw));
+    }
+
+  }
+}
+
+static void setup_set_cursor(int state) {
+  if(setup) {
+    XLockDisplay(gGui->display);
+    XDefineCursor(gGui->display, (xitk_window_get_window(setup->xwin)), 
+		  setup->cursor[state]);
+    XSync(gGui->display, False);
+    XUnlockDisplay(gGui->display);
   }
 }
 
@@ -411,7 +379,7 @@ static void setup_clear_tab(void) {
   xitk_image_t *im;
 
   im = xitk_image_create_image(gGui->imlib_data, (WINDOW_WIDTH - 40), 
-  			       (WINDOW_HEIGHT - (51 + 57) + 1 + 4));
+  			       (WINDOW_HEIGHT - (51 + 57 + 20) + 1 + 4));
 
   draw_outter(gGui->imlib_data, im->image, im->width, im->height);
 
@@ -472,6 +440,7 @@ static void setup_paint_widgets(void) {
 
   /* Repaint them now */
   xitk_paint_widget_list (setup->widget_list); 
+  setup_set_cursor(NORMAL_CURS);
 }
 
 /*
@@ -634,11 +603,11 @@ static widget_triplet_t *setup_add_nothing_available(const char *title, int x, i
  *
  */
 static widget_triplet_t *setup_add_slider (const char *title, const char *labelkey, 
-					   int x, int y, xine_cfg_entry_t *entry ) {
+					   int x, int y, xine_cfg_entry_t *entry) {
   xitk_slider_widget_t     sl;
   xitk_widget_t           *slider;
-  static widget_triplet_t *wt; 
-  
+  static widget_triplet_t *wt;
+
   wt = (widget_triplet_t *) xine_xmalloc(sizeof(widget_triplet_t));
 
   XITK_WIDGET_INIT(&sl, gGui->imlib_data);
@@ -677,7 +646,7 @@ static widget_triplet_t *setup_add_inputnum(const char *title, const char *label
   xitk_intbox_widget_t      ib;
   xitk_widget_t            *intbox, *wi, *wbu, *wbd;
   static widget_triplet_t  *wt;
-
+  
   wt = (widget_triplet_t *) xine_xmalloc(sizeof(widget_triplet_t));
 
   XITK_WIDGET_INIT(&ib, gGui->imlib_data);
@@ -757,7 +726,7 @@ static widget_triplet_t *setup_add_checkbox (const char *title, const char *labe
   xitk_checkbox_widget_t    cb;
   xitk_widget_t            *checkbox;
   static widget_triplet_t  *wt;
-
+  
   wt = (widget_triplet_t *) xine_xmalloc(sizeof(widget_triplet_t));
 
   XITK_WIDGET_INIT(&cb, gGui->imlib_data);
@@ -799,6 +768,7 @@ static widget_triplet_t *setup_add_combo (const char *title, const char *labelke
   ADD_FRAME(title);
 
   cmb.skin_element_name = NULL;
+  cmb.layer_above       = (is_layer_above());
   cmb.parent_wlist      = setup->widget_list;
   cmb.entries           = entry->enum_values;
   cmb.parent_wkey       = &setup->kreg;
@@ -822,56 +792,13 @@ static widget_triplet_t *setup_add_combo (const char *title, const char *labelke
   return wt;
 }
 
-/*
- * Add a browser (needed for help files display).
- */
-static widget_triplet_t *setup_list_browser(int x, int y, const char **content, int len) {
-  xitk_browser_widget_t     br;
-  xitk_widget_t            *browser;
-  static widget_triplet_t  *wt;
-
-  wt = (widget_triplet_t *) xine_xmalloc(sizeof(widget_triplet_t));
-
-  XITK_WIDGET_INIT(&br, gGui->imlib_data);
-
-  br.arrow_up.skin_element_name    = NULL;
-  br.slider.skin_element_name      = NULL;
-  br.arrow_dn.skin_element_name    = NULL;
-  br.browser.skin_element_name     = NULL;
-  br.browser.max_displayed_entries = BROWSER_MAX_DISP_ENTRIES;
-  br.browser.num_entries           = len;
-  br.browser.entries               = content;
-  br.callback                      = NULL;
-  br.dbl_click_callback            = NULL;
-  br.parent_wlist                  = setup->widget_list;
-  br.userdata                      = NULL;
-  xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)), 
-			   (browser = 
-			    xitk_noskin_browser_create(setup->widget_list, &br,
-						       (XITK_WIDGET_LIST_GC(setup->widget_list)), 
-						       x, y,
-						       (WINDOW_WIDTH - 50) - 16,
-						       20, 16, fontname)));
-  
-  xitk_browser_set_alignment(browser, ALIGN_LEFT);
-  xitk_browser_update_list(browser, content, len, 0);
-
-  add_widget_to_list(browser);
-
-  wt->widget = browser;
-  wt->frame  = NULL;
-  wt->label  = NULL;
-
-  return wt;
-}
-
 /*
  *
  */
 static void setup_section_widgets(int s) {
   int                  x = ((WINDOW_WIDTH>>1) - (FRAME_WIDTH>>1) - 10);
   int                  y = 70;
-  xine_cfg_entry_t    *entry;
+  xine_cfg_entry_t *entry;
   int                  cfg_err_result;
   int                  len;
   char                *section;
@@ -881,119 +808,99 @@ static void setup_section_widgets(int s) {
   xitk_disable_widget(setup->slider_wg);
   xitk_hide_widget(setup->slider_wg);
   
-  /* Selected tab is one of help sections */
-  if(s >= (setup->num_sections - 3)) {
-    if(s == (setup->num_sections - 3)) {
-      setup->wg[setup->num_wg] = setup_list_browser (25, 70, 
-						     setup->config_content, setup->config_lines);
-    }
-    else if(s == (setup->num_sections - 2)) {
-      setup->wg[setup->num_wg] = setup_list_browser (25, 70, 
-						     setup->readme_content, setup->readme_lines);
-    }
-    else if(s == (setup->num_sections - 1)) {
-      setup->wg[setup->num_wg] = setup_list_browser (25, 70, 
-						     setup->faq_content, setup->faq_lines);
-    }
+  section = setup->sections[s];
+  len     = strlen (section);
+  entry   = (xine_cfg_entry_t *)xine_xmalloc(sizeof(xine_cfg_entry_t));
+  cfg_err_result   = xine_config_get_first_entry(gGui->xine, entry);
     
-    DISABLE_ME(setup->wg[setup->num_wg]);
-    setup->num_wg++;
+  while (cfg_err_result) {
+      
+    if((entry->exp_level <= gGui->experience_level) &&
+       ((!strncmp(entry->key, section, len)) && entry->description)) {
+	
+      labelkey = &entry->key[len+1];
+	
+      switch (entry->type) {
+	  
+      case XINE_CONFIG_TYPE_RANGE: /* slider */
+	setup->wg[setup->num_wg] = setup_add_slider (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
+	  
+      case XINE_CONFIG_TYPE_STRING:
+	setup->wg[setup->num_wg] = setup_add_inputtext (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
+	  
+      case XINE_CONFIG_TYPE_ENUM:
+	setup->wg[setup->num_wg] = setup_add_combo (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
+	  
+      case XINE_CONFIG_TYPE_NUM:
+	setup->wg[setup->num_wg] = setup_add_inputnum (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
+	  
+      case XINE_CONFIG_TYPE_BOOL:
+	setup->wg[setup->num_wg] = setup_add_checkbox (entry->description, labelkey, x, y, entry);
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
+			     (entry->help) ?  (char *) entry->help : _("No help available"));
+	DISABLE_ME(setup->wg[setup->num_wg]);
+	setup->num_wg++;
+	break;
+	  
+      }
+      
+    } else
+      free(entry);
+      
+    entry = (xine_cfg_entry_t *)xine_xmalloc(sizeof(xine_cfg_entry_t));
+    cfg_err_result = xine_config_get_next_entry(gGui->xine, entry);
   }
-  else {
-    
-    section = setup->sections[s];
-    len     = strlen (section);
-    entry   = (xine_cfg_entry_t *)xine_xmalloc(sizeof(xine_cfg_entry_t));
-    cfg_err_result   = xine_config_get_first_entry(gGui->xine, entry);
-    
-    while (cfg_err_result) {
-      
-      if((entry->exp_level <= gGui->experience_level) &&
-	 ((!strncmp(entry->key, section, len)) && entry->description)) {
-	
-	labelkey = &entry->key[len+1];
-	
-	switch (entry->type) {
-	  
-	case XINE_CONFIG_TYPE_RANGE: /* slider */
-	  setup->wg[setup->num_wg] = setup_add_slider (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
-	  
-	case XINE_CONFIG_TYPE_STRING:
-	  setup->wg[setup->num_wg] = setup_add_inputtext (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
-	  
-	case XINE_CONFIG_TYPE_ENUM:
-	  setup->wg[setup->num_wg] = setup_add_combo (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
-	  
-	case XINE_CONFIG_TYPE_NUM:
-	  setup->wg[setup->num_wg] = setup_add_inputnum (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
-	  
-	case XINE_CONFIG_TYPE_BOOL:
-	  setup->wg[setup->num_wg] = setup_add_checkbox (entry->description, labelkey, x, y, entry);
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->widget, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  xitk_set_widget_tips(setup->wg[setup->num_wg]->label, 
-			       (entry->help) ?  (char *) entry->help : _("No help available"));
-	  DISABLE_ME(setup->wg[setup->num_wg]);
-	  setup->num_wg++;
-	  break;
-	  
-	}
-	
-      } else
-        free(entry);
-      
-      entry = (xine_cfg_entry_t *)xine_xmalloc(sizeof(xine_cfg_entry_t));
-      cfg_err_result = xine_config_get_next_entry(gGui->xine, entry);
-    }
-    free(entry);
+  free(entry);
 
     
-    if(setup->num_wg == 0) {
-      setup->wg[setup->num_wg++] = 
-	setup_add_nothing_available(_("There is no configuration option available in "
-				      "this user experience level."), x, y);
-    }
-
-
-    if(setup->num_wg > MAX_DISPLAY_WIDGETS) {
-      slidmax = setup->num_wg - MAX_DISPLAY_WIDGETS;
-      xitk_show_widget(setup->slider_wg);
-      xitk_enable_widget(setup->slider_wg);
-    }
-    else
-      slidmax = 1;
-      
-    xitk_slider_set_max(setup->slider_wg, slidmax);
-    xitk_slider_set_pos(setup->slider_wg, slidmax);
+  if(setup->num_wg == 0) {
+    setup->wg[setup->num_wg++] = 
+      setup_add_nothing_available(_("There is no configuration option available in "
+				    "this user experience level."), x, y);
   }
+
+
+  if(setup->num_wg > MAX_DISPLAY_WIDGETS) {
+    slidmax = setup->num_wg - MAX_DISPLAY_WIDGETS;
+    xitk_show_widget(setup->slider_wg);
+    xitk_enable_widget(setup->slider_wg);
+  }
+  else
+    slidmax = 1;
+
+  xitk_slider_set_max(setup->slider_wg, slidmax);
+  xitk_slider_set_pos(setup->slider_wg, slidmax);
+
 }
 
 /*
@@ -1003,9 +910,10 @@ static void setup_change_section(xitk_widget_t *wx, void *data, int section) {
   int i;
   xitk_widget_t *sw;
 
-  for (i = 0; i < setup->num_wg; i++ ) {
+  setup_set_cursor(WAIT_CURS);
+
+  for (i = 0; i < setup->num_wg; i++ )
     free(setup->wg[i]);
-  }
 
   /* remove old widgets */
   sw = (xitk_widget_t *) xitk_list_first_content(setup->widgets);
@@ -1031,7 +939,7 @@ static void setup_change_section(xitk_widget_t *wx, void *data, int section) {
   }
 
   
-  xitk_list_free(setup->widgets);
+  xitk_list_clear(setup->widgets);
   setup->num_wg = 0;
   setup->first_displayed = 0;
 
@@ -1046,8 +954,8 @@ static void setup_change_section(xitk_widget_t *wx, void *data, int section) {
  */
 static void setup_sections (void) {
   xitk_pixmap_t       *bg;
-  xine_cfg_entry_t    entry;
-  int                 cfg_err_result;
+  xine_cfg_entry_t  entry;
+  int                  cfg_err_result;
   xitk_tabs_widget_t   tab;
 
   setup->num_sections = 0;
@@ -1084,13 +992,6 @@ static void setup_sections (void) {
     cfg_err_result = xine_config_get_next_entry(gGui->xine, &entry);
   }
 
-  setup->sections[setup->num_sections] = strdup(_("Help"));
-  setup->num_sections++;
-  setup->sections[setup->num_sections] = strdup(_("README"));
-  setup->num_sections++;
-  setup->sections[setup->num_sections] = strdup(_("FAQ"));
-  setup->num_sections++;
-
   XITK_WIDGET_INIT(&tab, gGui->imlib_data);
 
   tab.skin_element_name = NULL;
@@ -1103,6 +1004,8 @@ static void setup_sections (void) {
     (setup->tabs = 
      xitk_noskin_tabs_create(setup->widget_list, &tab, 20, 24, WINDOW_WIDTH - 40, tabsfontname)));
   
+  xitk_enable_and_show_widget(setup->tabs);
+
   bg = xitk_image_create_xitk_pixmap(gGui->imlib_data, WINDOW_WIDTH, WINDOW_HEIGHT);
   
   XLockDisplay(gGui->display);
@@ -1111,7 +1014,7 @@ static void setup_sections (void) {
   XUnlockDisplay(gGui->display);
   
   draw_rectangular_outter_box(gGui->imlib_data, bg, 20, 51, 
-			      (WINDOW_WIDTH - 40) - 1, (WINDOW_HEIGHT - (51 + 57)) + 4);
+			      (WINDOW_WIDTH - 40) - 1, (WINDOW_HEIGHT - (51 + 57 + 20)) + 4);
   xitk_window_change_background(gGui->imlib_data, setup->xwin, bg->pixmap,
 				WINDOW_WIDTH, WINDOW_HEIGHT);
   
@@ -1145,77 +1048,16 @@ static void setup_nextprev_wg(xitk_widget_t *w, void *data, int pos) {
   }
 }
 
-/*
- * Read adn store some files.
- */
-static const char **_setup_read_given(char *given, int *ndest) {
-  char            buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-  char            buffer[256], *ln;
-  const langs_t  *l;
-  FILE           *fd;
-  int             first_try = 1;
-  char          **dest = NULL;
-
-  if((given == NULL) || (ndest == NULL))
-    return NULL;
-  
-  *ndest = 0;
-  
-  memset(&buf, 0, sizeof(buf));
-  
-  l = get_lang();
-
- __redo:
-
-  sprintf(buf, "%s/%s%s", XINE_DOCDIR, given, l->ext);
-
-  if((fd = fopen(buf, "r")) != NULL) {
-    
-    while((ln = fgets(buffer, 255, fd)) != NULL) {
-      dest = (char **) realloc(dest, sizeof(char *) * (*ndest + 2));
-      
-      /* label widget hate empty labels */
-      dest[*ndest] = strdup(strlen(ln) ? ln : " ");
-      
-      /* Remove newline */
-      if(dest[*ndest][strlen(dest[*ndest]) - 1] == '\n')
-	dest[*ndest][strlen(dest[*ndest]) - 1] = '\0';
-      
-      (*ndest)++;
-    }
-    
-    dest[*ndest] = NULL;
-    
-    fclose(fd);
-  }
-
-  if(first_try && (dest == NULL)) {
-    l = get_default_lang();
-    first_try--;
-    goto __redo;
-  }
-
-  return (const char **)dest;  
-}
-static void setup_read_config(void) {
-  setup->config_content = _setup_read_given("README.config", &setup->config_lines);
-}
-static void setup_read_readme(void) {
-  setup->readme_content = _setup_read_given("README", &setup->readme_lines);
-}
-static void setup_read_faq(void) {
-  setup->faq_content =  _setup_read_given("FAQ", &setup->faq_lines);
-}
-
 /*
  * Create setup panel window
  */
 void setup_panel(void) {
   GC                         gc;
   xitk_labelbutton_widget_t  lb;
-  xitk_button_widget_t       b;
   xitk_slider_widget_t       sl;
+  xitk_label_widget_t        lbl;
   int                        x, y;
+  xitk_widget_t             *w;
 
   setup = (_setup_t *) xine_xmalloc(sizeof(_setup_t));
 
@@ -1238,15 +1080,12 @@ void setup_panel(void) {
   setup->xwin = xitk_window_create_dialog_window(gGui->imlib_data,
 						 _("xine setup"), 
 						 x, y, WINDOW_WIDTH, WINDOW_HEIGHT);
-
-  /* Read some help files */
-  setup_read_config();
-  setup_read_readme();
-  setup_read_faq();
   
   XLockDisplay (gGui->display);
   gc = XCreateGC(gGui->display, 
 		 (xitk_window_get_window(setup->xwin)), None, None);
+  setup->cursor[NORMAL_CURS] = XCreateFontCursor(gGui->display, XC_left_ptr);
+  setup->cursor[WAIT_CURS] = XCreateFontCursor(gGui->display, XC_watch);
   XUnlockDisplay (gGui->display);
 
   setup->widget_list                = xitk_widget_list_new();
@@ -1255,31 +1094,50 @@ void setup_panel(void) {
 		       WIDGET_LIST_WINDOW, (void *) (xitk_window_get_window(setup->xwin)));
   xitk_widget_list_set(setup->widget_list, WIDGET_LIST_GC, gc);
 
-  XITK_WIDGET_INIT(&b, gGui->imlib_data);
   XITK_WIDGET_INIT(&sl, gGui->imlib_data);
 
-  {
-
-    sl.min                      = 0;
-    sl.max                      = 1;
-    sl.step                     = 1;
-    sl.skin_element_name        = NULL;
-    sl.callback                 = NULL;
-    sl.userdata                 = NULL;
-    sl.motion_callback          = setup_nextprev_wg;
-    sl.motion_userdata          = NULL;
-    xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)),
-     (setup->slider_wg = xitk_noskin_slider_create(setup->widget_list, &sl,
-						   (WINDOW_WIDTH - 41), 70, 
-						   16, (WINDOW_HEIGHT - 140), XITK_VSLIDER)));
-    xitk_slider_set_pos(setup->slider_wg, 1);
-    xitk_disable_widget(setup->slider_wg);
-    xitk_hide_widget(setup->slider_wg);
-  }
+  sl.min                      = 0;
+  sl.max                      = 1;
+  sl.step                     = 1;
+  sl.skin_element_name        = NULL;
+  sl.callback                 = NULL;
+  sl.userdata                 = NULL;
+  sl.motion_callback          = setup_nextprev_wg;
+  sl.motion_userdata          = NULL;
+  xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)),
+   (setup->slider_wg = xitk_noskin_slider_create(setup->widget_list, &sl,
+						 (WINDOW_WIDTH - 41), 70, 
+						 16, (WINDOW_HEIGHT - 160), XITK_VSLIDER)));
 
   setup_sections();
   setup_paint_widgets();
 
+  {
+    char         *label = _("(*)  you need to restart xine for this setting to take effect");
+    xitk_font_t  *fs;
+    int           len;
+    
+    fs = xitk_font_load_font(gGui->display, fontname);
+    xitk_font_set_font(fs, (XITK_WIDGET_LIST_GC(setup->widget_list)));
+    len = xitk_font_get_string_length(fs, (const char *) label);
+    xitk_font_unload_font(fs);
+    
+    XITK_WIDGET_INIT(&lbl, gGui->imlib_data);
+    
+    lbl.window              = xitk_window_get_window(setup->xwin);
+    lbl.gc                  = (XITK_WIDGET_LIST_GC(setup->widget_list));
+    lbl.skin_element_name   = NULL;
+    lbl.label               = label;
+    lbl.callback            = NULL;
+    lbl.userdata            = NULL;
+    xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)),
+			     (w = xitk_noskin_label_create(setup->widget_list, &lbl,
+							   (WINDOW_WIDTH - len) >> 1,
+							   (WINDOW_HEIGHT - (57 + 20)) + 1 +4,
+							   len + 1, 15, fontname)));
+    xitk_enable_and_show_widget(w);
+  }
+
   XITK_WIDGET_INIT(&lb, gGui->imlib_data);
 
   x = (WINDOW_WIDTH - (3 * 100))>>2;
@@ -1292,9 +1150,11 @@ void setup_panel(void) {
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)), 
-   xitk_noskin_labelbutton_create(setup->widget_list, &lb, x, WINDOW_HEIGHT - 40, 100, 23,
-				  "Black", "Black", "White", tabsfontname));
-  
+   (setup->ok = xitk_noskin_labelbutton_create(setup->widget_list, &lb, 
+					       x, WINDOW_HEIGHT - 40, 100, 23,
+					       "Black", "Black", "White", tabsfontname)));
+  xitk_enable_and_show_widget(setup->ok);
+
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Apply");
   lb.align             = ALIGN_CENTER;
@@ -1303,9 +1163,11 @@ void setup_panel(void) {
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)), 
-   xitk_noskin_labelbutton_create(setup->widget_list, &lb, (x * 2) + 100, WINDOW_HEIGHT - 40, 100, 23,
-				  "Black", "Black", "White", tabsfontname));
-
+   (w = xitk_noskin_labelbutton_create(setup->widget_list, &lb, 
+				       (x * 2) + 100, WINDOW_HEIGHT - 40, 100, 23, 
+				       "Black", "Black", "White", tabsfontname)));
+  xitk_enable_and_show_widget(w);
+  
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Close");
   lb.align             = ALIGN_CENTER;
@@ -1314,9 +1176,11 @@ void setup_panel(void) {
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
   xitk_list_append_content((XITK_WIDGET_LIST_LIST(setup->widget_list)), 
-   xitk_noskin_labelbutton_create(setup->widget_list, &lb, (x * 3) + (100 * 2), WINDOW_HEIGHT - 40, 100, 23,
-				  "Black", "Black", "White", tabsfontname));
-
+   (w = xitk_noskin_labelbutton_create(setup->widget_list, &lb, 
+				       (x * 3) + (100 * 2), WINDOW_HEIGHT - 40, 100, 23,
+				       "Black", "Black", "White", tabsfontname)));
+  xitk_enable_and_show_widget(w);
+  
   setup->kreg = xitk_register_event_handler("setup", 
 					    (xitk_window_get_window(setup->xwin)),
 					    setup_handle_event,
@@ -1329,10 +1193,5 @@ void setup_panel(void) {
   setup->running = 1;
   setup_raise_window();
 
-  while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(setup->xwin)))
-    xine_usec_sleep(5000);
-
-  XLockDisplay (gGui->display);
-  XSetInputFocus(gGui->display, xitk_window_get_window(setup->xwin), RevertToParent, CurrentTime);
-  XUnlockDisplay (gGui->display);
+  try_to_set_input_focus(xitk_window_get_window(setup->xwin));
 }
