@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skins.c,v 1.52 2003/02/02 16:51:11 f1rmb Exp $
+ * $Id: skins.c,v 1.61 2003/12/01 21:37:24 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -26,7 +26,7 @@
 
 #include <stdio.h>
 #include <dirent.h>
-#include <sys/stat.h>       
+#include <sys/stat.h>
 #include <errno.h>
 
 #include <X11/Xlib.h>
@@ -78,12 +78,10 @@ typedef struct {
 
 static skin_downloader_t  *skdloader = NULL;
 
-/*
-#define SKIN_DEBUG 1
-*/
+#undef SKIN_DEBUG
 
 #define DEFAULT_SKIN        "xinetic"
-#define SKIN_IFACE_VERSION  4
+#define SKIN_IFACE_VERSION  5
 
 extern gGui_t             *gGui;
 
@@ -103,7 +101,6 @@ static void get_available_skins_from(char *path) {
     return;
 
   if((pdir = opendir(path)) != NULL) {
-    struct stat   pstat, sstat;
     char          fullfilename[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     char          skcfgname[XITK_PATH_MAX + XITK_NAME_MAX + 1];
     
@@ -111,9 +108,7 @@ static void get_available_skins_from(char *path) {
       memset(&fullfilename, 0, sizeof(fullfilename));
       sprintf(fullfilename, "%s/%s", path, pdirent->d_name);
       
-      stat(fullfilename, &pstat);
-      
-      if((S_ISDIR(pstat.st_mode))
+      if((is_a_dir(fullfilename))
 	 && (!(strlen(pdirent->d_name) == 1 && pdirent->d_name[0] == '.' )
 	     && !(strlen(pdirent->d_name) == 2 
 		  && (pdirent->d_name[0] == '.' && pdirent->d_name[1] == '.')))) {
@@ -123,7 +118,7 @@ static void get_available_skins_from(char *path) {
 	 */
 	memset(&skcfgname, 0, sizeof(skcfgname));
 	sprintf(skcfgname, "%s/%s", fullfilename, "skinconfig");
-	if(((stat(skcfgname, &sstat)) > -1) && (S_ISREG(sstat.st_mode))) {
+	if(is_a_file(skcfgname)) {
 	  
 	  skins_avail = (skins_locations_t **) realloc(skins_avail, 
 						       (skins_avail_num + 2) * sizeof(skins_locations_t*));
@@ -172,9 +167,9 @@ static void looking_for_available_skins(void) {
  * Return default skindir.
  */
 char *skin_get_skindir(void) {
-  static char         tmp[2048];
-  xine_cfg_entry_t   entry;
-  char               *skin;
+  static char          tmp[2048];
+  xine_cfg_entry_t     entry;
+  char                *skin;
   
   memset(&tmp, 0, 2048);
   memset(&entry, 0, sizeof(xine_cfg_entry_t)); 
@@ -192,9 +187,9 @@ char *skin_get_skindir(void) {
  * Return the full pathname the skin (default) configfile.
  */
 char *skin_get_configfile(void) {
-  static char        tmp[2048];
+  static char          tmp[2048];
   xine_cfg_entry_t  entry;
-  char              *skin;
+  char                *skin;
   
   memset(&tmp, 0, 2048);
   memset(&entry, 0, sizeof(xine_cfg_entry_t)); 
@@ -384,18 +379,37 @@ void select_new_skin(int selected) {
   config_update_num("gui.skin", selected);
 }
 
+/*
+ * Return path of current skin
+ */
+const char *skin_get_current_skin_dir(void) {
+  xine_cfg_entry_t     entry;
+  int                  skin_num = 0;
+  static char          skin_dir[XINE_PATH_MAX + XINE_NAME_MAX];
+  
+  if((skins_avail == NULL) || (skins_avail_num == 0)) {
+    fprintf(stderr, _("No available skin found. Say goodbye.\n"));
+    exit(-1);
+  }
+  
+  memset(&entry, 0, sizeof(xine_cfg_entry_t)); 
+  (void) xine_config_lookup_entry(gGui->xine, "gui.skin", &entry);
+  skin_num = entry.num_value;
+  
+  sprintf(skin_dir, "%s/%s", skins_avail[skin_num]->pathname, skins_avail[skin_num]->skin);
+
+  return (const char *) &skin_dir;
+}
+
 /*
  * Initialize skin support.
  */
-void init_skins_support(void) {
-  skins_locations_t   *sk;
-  char                 buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
-  int                  twice = 0, twice_load = 0;
-  int                  skin_num, i;
-  char                *skin_anim;
-    
-  change_config_entry = 0;
+void preinit_skins_support(void) {
+  int  skin_num;
+  int  i;
 
+  change_config_entry = 0;
+  
   gGui->skin_config = xitk_skin_init_config();
   
   looking_for_available_skins();
@@ -406,10 +420,12 @@ void init_skins_support(void) {
   }
   
   skin_names = (char **) xine_xmalloc(sizeof(char *) * (skins_avail_num + 1));
+
   for(i = 0; i < skins_avail_num; i++)
     skin_names[i] = strdup(skins_avail[i]->skin);
+  
   skin_names[skins_avail_num] = NULL;
-
+  
   skin_num = xine_config_register_enum (gGui->xine, "gui.skin", 
 					(get_skin_offset(DEFAULT_SKIN)),
 					skin_names,
@@ -418,7 +434,24 @@ void init_skins_support(void) {
 					CONFIG_LEVEL_BEG,
 					skin_change_cb, 
 					CONFIG_NO_DATA);
+}
 
+void init_skins_support(void) {
+  skins_locations_t   *sk;
+  char                 buf[XITK_PATH_MAX + XITK_NAME_MAX + 1];
+  int                  twice = 0, twice_load = 0;
+  int                  skin_num;
+  char                *skin_anim;
+  xine_cfg_entry_t     entry;
+
+  memset(&entry, 0, sizeof(xine_cfg_entry_t)); 
+  if(xine_config_lookup_entry(gGui->xine, "gui.skin", &entry))
+    skin_num = entry.num_value;
+  else {
+    fprintf(stderr, _("Ooch, gui.skin config entry isn't registered. Say goodbye.\n"));
+    exit(-1);
+  }
+  
   sk = (skin_num < skins_avail_num) ? skins_avail[skin_num] : NULL;
   
   if(!sk) {
@@ -435,7 +468,7 @@ void init_skins_support(void) {
  __reload_skin:
   memset(&buf, 0, XITK_PATH_MAX + XITK_NAME_MAX);
   sprintf(buf, "%s/%s", sk->pathname, sk->skin);
-
+  
   if(!xitk_skin_load_config(gGui->skin_config, buf, "skinconfig")) {
     if(!twice_load) {
       twice_load++;
@@ -650,7 +683,7 @@ static void download_skin_cancel(xitk_widget_t *w, void *data) {
   xitk_destroy_widgets(skdloader->widget_list);
   xitk_window_destroy_window(gGui->imlib_data, skdloader->xwin);
 
-  skdloader->xwin = None;
+  skdloader->xwin = NULL;
 
   xitk_list_free((XITK_WIDGET_LIST_LIST(skdloader->widget_list)));
   
@@ -810,7 +843,7 @@ static void download_skin_preview(xitk_widget_t *w, void *data, int selected) {
 
 	download_update_preview();
       }
-      
+
       unlink(tmpfile);
     }
   }
@@ -881,7 +914,7 @@ static void download_skin_select(xitk_widget_t *w, void *data) {
 
 	  memset(&buffer, 0, sizeof(buffer));
 	  
-	  sprintf(buffer, "`which tar`||tar -C %s -xzf %s", skindir, tmpskin);
+	  sprintf(buffer, "which tar > /dev/null 2>&1 && tar -C %s -xzf %s", skindir, tmpskin);
 	  xine_system(0, buffer);
 	  unlink(tmpskin);
 	  
@@ -1013,6 +1046,7 @@ void download_skin(char *url) {
   xitk_window_t        *xwin;
   xitk_pixmap_t        *bg;
   int                   x, y, w, h, width, height;
+  xitk_widget_t        *widget;
 
   w = 300;
   h = 50;
@@ -1136,6 +1170,7 @@ void download_skin(char *url) {
     xitk_browser_update_list(skdloader->browser, 
     			     (const char *const *)skdloader->entries, skdloader->num, 0);
     
+    xitk_enable_and_show_widget(skdloader->browser);
     
     draw_rectangular_inner_box(gGui->imlib_data, bg, 8, y - 2, (WINDOW_WIDTH - 20) + 5, (5 * 20) + 10 + 5);
     
@@ -1153,9 +1188,11 @@ void download_skin(char *url) {
     lb.userdata          = NULL;
     lb.skin_element_name = NULL;
     xitk_list_append_content((XITK_WIDGET_LIST_LIST(skdloader->widget_list)), 
-			     xitk_noskin_labelbutton_create(skdloader->widget_list, 
-							    &lb, x, y, 100, 23,
-							    "Black", "Black", "White", btnfontname));
+	     (widget = xitk_noskin_labelbutton_create(skdloader->widget_list, 
+						      &lb, x, y, 100, 23,
+						      "Black", "Black", "White", btnfontname)));
+    xitk_enable_and_show_widget(widget);
+
     x = WINDOW_WIDTH - 115;
     
     lb.button_type       = CLICK_BUTTON;
@@ -1166,10 +1203,11 @@ void download_skin(char *url) {
     lb.userdata          = NULL;
     lb.skin_element_name = NULL;
     xitk_list_append_content((XITK_WIDGET_LIST_LIST(skdloader->widget_list)), 
-			     xitk_noskin_labelbutton_create(skdloader->widget_list, 
-							    &lb, x, y, 100, 23,
-							    "Black", "Black", "White", btnfontname));
-
+	     (widget = xitk_noskin_labelbutton_create(skdloader->widget_list, 
+						      &lb, x, y, 100, 23,
+						      "Black", "Black", "White", btnfontname)));
+    xitk_enable_and_show_widget(widget);
+    
     skdloader->widget_key = xitk_register_event_handler("skdloader", 
 							(xitk_window_get_window(skdloader->xwin)),
 							download_skin_handle_event,
@@ -1187,13 +1225,7 @@ void download_skin(char *url) {
     XUnlockDisplay(gGui->display);
     layer_above_video(xitk_window_get_window(skdloader->xwin));
 
-    while(!xitk_is_window_visible(gGui->display, xitk_window_get_window(skdloader->xwin)))
-      xine_usec_sleep(5000);
-    
-    XLockDisplay (gGui->display);
-    XSetInputFocus(gGui->display, xitk_window_get_window(skdloader->xwin), RevertToParent, CurrentTime);
-    XUnlockDisplay (gGui->display);
-
+    try_to_set_input_focus(xitk_window_get_window(skdloader->xwin));
     download_update_blank_preview();
   }
   else
