@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: skins.h,v 1.4 2002/05/05 09:46:09 f1rmb Exp $
+ * $Id: skins.h,v 1.9 2003/12/01 21:37:24 f1rmb Exp $
  *
  */
 
@@ -27,14 +27,15 @@
 typedef struct {
   const char *pathname;
   const char *skin;
+  int         number;
 } skins_locations_t;
 
-char *gui_get_skindir(void);
-char *gui_get_configfile(void);
 skins_locations_t **get_available_skins(void);
 int get_available_skins_num(void);
+void preinit_skins_support(void);
 void init_skins_support(void);
 void select_new_skin(int selected);
-void change_skin(skins_locations_t *sk);
+void download_skin(char *url);
+const char *skin_get_current_skin_dir(void);
 
 #endif
