@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: videowin.h,v 1.32 2003/07/14 22:44:10 f1rmb Exp $
+ * $Id: videowin.h,v 1.36 2003/11/29 02:24:38 miguelfreitas Exp $
  *
  * video window handling functions
  */
@@ -33,8 +33,14 @@ typedef struct {
   int   borderless;
 } window_attributes_t;
 
-#define CURSOR_ARROW 1
-#define CURSOR_HAND  2
+#define WINDOWED_MODE              0x00000001
+#define FULLSCR_MODE               0x00000002
+#ifdef HAVE_XINERAMA
+#define FULLSCR_XI_MODE            0x00000004
+#endif
+
+#define CURSOR_ARROW               1
+#define CURSOR_HAND                2
 
 void video_window_init (window_attributes_t *window_attribute, int hide_on_start);
 
@@ -60,10 +66,6 @@ void video_window_dest_size_cb (void *this,
 void video_window_set_fullscreen_mode (int req_fullscreen);
 int video_window_get_fullscreen_mode (void);
 
-/* set/check xinerama fullscreen mode */
-void video_window_set_xinerama_fullscreen_mode (int req_fullscreen);
-int video_window_get_xinerama_fullscreen_mode (void);
-
 /* Set cursor */
 void video_window_set_cursor(int cursor);
 /* hide/show cursor in video window*/
@@ -86,4 +88,10 @@ void video_window_change_skins(void);
 void video_window_reset_ssaver(void);
 
 void video_window_get_frame_size(int *w, int *h);
+
+void video_window_get_visible_size(int *w, int *h);
+
+void video_window_get_output_size(int *w, int *h);
+
+void video_window_set_mrl(char *mrl);
 #endif
