@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: viewlog.h,v 1.7 2003/01/05 00:24:34 f1rmb Exp $
+ * $Id: viewlog.h,v 1.9 2003/10/29 12:32:16 f1rmb Exp $
  *
  */
 
@@ -30,7 +30,7 @@
 
 #include "xitk.h"
 
-void viewlog_window(void);
+void viewlog_panel(void);
 void viewlog_exit(xitk_widget_t *, void *);
 int viewlog_is_visible(void);
 int viewlog_is_running(void);
