@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: combo.c,v 1.33 2003/01/05 00:24:35 f1rmb Exp $
+ * $Id: combo.c,v 1.36 2003/12/13 01:09:39 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -36,12 +36,10 @@ static void _combo_rollunroll(xitk_widget_t *w, void *data, int state);
  *
  */
 static void enability(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
 
     if(w->enable == WIDGET_ENABLE) {
       xitk_enable_widget(private_data->label_widget);
@@ -63,12 +61,10 @@ static void enability(xitk_widget_t *w) {
 }
 
 static void notify_destroy(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
-  
+
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
     
     if(private_data->visible)
       _combo_rollunroll(private_data->button_widget, (void *)w, 0);
@@ -83,10 +79,10 @@ static void notify_destroy(xitk_widget_t *w) {
     XFreeGC(private_data->imlibdata->x.disp, private_data->widget_list->gc);
     XUNLOCK(private_data->imlibdata->x.disp);
 
-    XITK_FREE(private_data->widget_list);
+    free(private_data->widget_list);
 
     XITK_FREE(private_data->skin_element_name);
-    XITK_FREE(private_data);
+    free(private_data);
   }
 }
 
@@ -94,12 +90,10 @@ static void notify_destroy(xitk_widget_t *w) {
  *
  */
 static void paint(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
 
     if(private_data->visible == 1 && (w->visible < 1)) {
       xitk_checkbox_set_state(private_data->button_widget, 0);
@@ -123,16 +117,30 @@ static void paint(xitk_widget_t *w) {
   }
 }
 
+/*
+ *
+ */
+void xitk_combo_callback_exec(xitk_widget_t *w) {
+  
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
+    combo_private_data_t *private_data = (combo_private_data_t *)w->private_data;
+    
+    if(private_data->callback)
+      private_data->callback(private_data->combo_widget, 
+			     private_data->userdata, private_data->selected);
+
+  }
+}
+
 /*
  * Called on select action.
  */
 static void combo_select(xitk_widget_t *w, void *data, int selected) {
-  combo_private_data_t *private_data;
-  xitk_widget_t        *c;
 
   if(w && ((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_BROWSER)) {
-    c = (xitk_widget_t *) ((browser_private_data_t *)w->private_data)->userdata;
-    private_data = (combo_private_data_t *)c->private_data;
+    xitk_widget_t        *c = (xitk_widget_t *) ((browser_private_data_t *)w->private_data)->userdata;
+    combo_private_data_t *private_data = (combo_private_data_t *)c->private_data;
     
     private_data->selected = selected;
     
@@ -170,7 +178,7 @@ static void _combo_handle_event(XEvent *event, void *data) {
      * If we try to move the combo window,
      * move it back to right position (under label*
      */
-    if(private_data->visible) { 
+    if(private_data && private_data->visible) { 
       int  x, y;
       xitk_window_get_window_position(private_data->combo_widget->imlibdata, 
 				      private_data->xwin, &x, &y, NULL, NULL);
@@ -185,11 +193,10 @@ static void _combo_handle_event(XEvent *event, void *data) {
  *
  */
 static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
 
     if(private_data->skin_element_name) {
       int x, y;
@@ -364,12 +371,10 @@ void xitk_combo_rollunroll(xitk_widget_t *w) {
    *
    */
 void xitk_combo_set_select(xitk_widget_t *w, int select) {
-  combo_private_data_t *private_data;
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    
-    private_data = (combo_private_data_t *)w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *)w->private_data;
     
     if(private_data->entries && private_data->entries[select]) {
       private_data->selected = select;
@@ -383,15 +388,13 @@ void xitk_combo_set_select(xitk_widget_t *w, int select) {
  *
  */
 void xitk_combo_update_pos(xitk_widget_t *w) {
-  combo_private_data_t  *private_data;
-  int                    xx = 0, yy = 0;
-  window_info_t          wi;
-  XSizeHints             hint;
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    
-    private_data = (combo_private_data_t *)w->private_data;
+    combo_private_data_t  *private_data = (combo_private_data_t *)w->private_data;
+    int                    xx = 0, yy = 0;
+    window_info_t          wi;
+    XSizeHints             hint;
     
     if(private_data->visible) {
       if((xitk_get_window_info(*(private_data->parent_wkey), &wi))) {
@@ -442,11 +445,11 @@ void xitk_combo_update_pos(xitk_widget_t *w) {
  *
  */
 int xitk_combo_get_current_selected(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
+
     return private_data->selected;
   }
 
@@ -457,11 +460,10 @@ int xitk_combo_get_current_selected(xitk_widget_t *w) {
  * 
  */
 char *xitk_combo_get_current_entry_selected(xitk_widget_t *w) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    private_data = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
 
     if(private_data->entries && private_data->selected >= 0)
       return(private_data->entries[private_data->selected]);
@@ -474,12 +476,11 @@ char *xitk_combo_get_current_entry_selected(xitk_widget_t *w) {
  *
  */
 void xitk_combo_update_list(xitk_widget_t *w, char **list, int len) {
-  combo_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
- 
-    private_data              = (combo_private_data_t *) w->private_data;
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
+
     private_data->entries     = list;
     private_data->num_entries = len;
     private_data->selected    = -1;
@@ -490,6 +491,21 @@ void xitk_combo_update_list(xitk_widget_t *w, char **list, int len) {
   }
 }
 
+/*
+ *
+ */
+xitk_widget_t *xitk_combo_get_label_widget(xitk_widget_t *w) {
+
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
+    combo_private_data_t *private_data = (combo_private_data_t *) w->private_data;
+    
+    return private_data->label_widget;
+  }
+
+  return NULL;
+}
+
 /*
  *
  */
@@ -594,6 +610,7 @@ static xitk_widget_t *_xitk_combo_create(xitk_widget_list_t *wl,
 							private_data->gc, 1, 1, 
 							(itemw - slidw), itemh, slidw,
 							DEFAULT_FONT_10)));
+  xitk_enable_and_show_widget(private_data->browser_widget);
   private_data->browser_widget->type |= WIDGET_GROUP | WIDGET_GROUP_COMBO;
   
   xitk_browser_update_list(private_data->browser_widget, 
@@ -774,5 +791,5 @@ xitk_widget_t *xitk_noskin_combo_create(xitk_widget_list_t *wl,
 
   }
 
-  return _xitk_combo_create(wl, NULL, c, NULL, mywidget, private_data, 1, 1);
+  return _xitk_combo_create(wl, NULL, c, NULL, mywidget, private_data, 0, 0);
 }
