@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: intbox.c,v 1.21 2003/01/05 00:24:35 f1rmb Exp $
+ * $Id: intbox.c,v 1.23 2003/09/15 20:28:22 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -29,16 +29,33 @@
 
 #include "_xitk.h"
 
+static void enability(xitk_widget_t *w) {
+  
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
+    intbox_private_data_t *private_data = (intbox_private_data_t *) w->private_data;
+    
+    if(w->enable == WIDGET_ENABLE) {
+      xitk_enable_widget(private_data->input_widget);
+      xitk_enable_widget(private_data->more_widget);
+      xitk_enable_widget(private_data->less_widget);
+    }
+    else {
+      xitk_disable_widget(private_data->input_widget);
+      xitk_disable_widget(private_data->more_widget);
+      xitk_disable_widget(private_data->less_widget);
+    }
+  }
+}
+
 /*
  *
  */
 static void notify_destroy(xitk_widget_t *w) {
-  intbox_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    
-    private_data = (intbox_private_data_t *) w->private_data;
+    intbox_private_data_t *private_data = (intbox_private_data_t *) w->private_data;
     
     XITK_FREE(private_data->skin_element_name);
     XITK_FREE(private_data);
@@ -49,12 +66,10 @@ static void notify_destroy(xitk_widget_t *w) {
  *
  */
 static void paint(xitk_widget_t *w) {
-  intbox_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    
-    private_data = (intbox_private_data_t *) w->private_data;
+    intbox_private_data_t *private_data = (intbox_private_data_t *) w->private_data;
     
     if((w->visible == 1)) {
       int bx, ih, iw;
@@ -82,11 +97,10 @@ static void paint(xitk_widget_t *w) {
  *
  */
 static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
-  intbox_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    private_data = (intbox_private_data_t *) w->private_data;
+    intbox_private_data_t *private_data = (intbox_private_data_t *) w->private_data;
 
     if(private_data->skin_element_name) {
       /*      
@@ -121,6 +135,9 @@ static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_re
   case WIDGET_EVENT_DESTROY:
     notify_destroy(w);
     break;
+  case WIDGET_EVENT_ENABLE:
+    enability(w);
+    break;
   }
   
   return retval;
@@ -131,13 +148,12 @@ static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_re
  */
 static void intbox_change_value(xitk_widget_t *x, void *data, char *string) {
   xitk_widget_t         *w = (xitk_widget_t *)data;
-  intbox_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    char  buf[256];
+    intbox_private_data_t *private_data = (intbox_private_data_t *)w->private_data;
+    char                   buf[256];
     
-    private_data = (intbox_private_data_t *)w->private_data;
     private_data->value = strtol(string, &string, 10);
     
     memset(&buf, 0, sizeof(buf));
@@ -153,13 +169,11 @@ static void intbox_change_value(xitk_widget_t *x, void *data, char *string) {
  *
  */
 void xitk_intbox_set_value(xitk_widget_t *w, int value) {
-  intbox_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    char buf[256];
-    
-    private_data = (intbox_private_data_t *) w->private_data;
+    intbox_private_data_t *private_data = (intbox_private_data_t *) w->private_data;
+    char                   buf[256];
 
     memset(&buf, 0, sizeof(buf));
     snprintf(buf, 256, "%d", value);
@@ -173,13 +187,12 @@ void xitk_intbox_set_value(xitk_widget_t *w, int value) {
  *
  */
 int xitk_intbox_get_value(xitk_widget_t *w) {
-  intbox_private_data_t *private_data;
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    char *strval;
-
-    private_data = (intbox_private_data_t *)w->private_data;
+    intbox_private_data_t *private_data = (intbox_private_data_t *)w->private_data;
+    char                  *strval;
+    
     strval = xitk_inputtext_get_text(private_data->input_widget);
     private_data->value = strtol(strval, &strval, 10);
     
@@ -193,12 +206,11 @@ int xitk_intbox_get_value(xitk_widget_t *w) {
  */
 static void intbox_stepdown(xitk_widget_t *x, void *data) {
   xitk_widget_t *w = (xitk_widget_t *) data;
-  intbox_private_data_t *private_data;
 
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    
-    private_data = (intbox_private_data_t *)w->private_data;
+    intbox_private_data_t *private_data = (intbox_private_data_t *)w->private_data;
+
     private_data->value -= private_data->step;
     xitk_intbox_set_value(w, private_data->value);
     if(private_data->callback)
@@ -211,12 +223,11 @@ static void intbox_stepdown(xitk_widget_t *x, void *data) {
  */
 static void intbox_stepup(xitk_widget_t *x, void *data) {
   xitk_widget_t *w = (xitk_widget_t *) data;
-  intbox_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    
-    private_data = (intbox_private_data_t *)w->private_data;
+    intbox_private_data_t *private_data = (intbox_private_data_t *)w->private_data;
+
     private_data->value += private_data->step;
     xitk_intbox_set_value(w, private_data->value);
     if(private_data->callback)
@@ -224,6 +235,18 @@ static void intbox_stepup(xitk_widget_t *x, void *data) {
   }
 }
 
+xitk_widget_t *xitk_intbox_get_input_widget(xitk_widget_t *w) {
+
+  if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) &&
+	   (w->type & WIDGET_GROUP_WIDGET))) {
+    intbox_private_data_t *private_data = (intbox_private_data_t *)w->private_data;
+    
+    return private_data->input_widget;
+  }
+
+  return NULL;
+}
+
 /*
  *
  */
@@ -348,5 +371,5 @@ xitk_widget_t *xitk_noskin_intbox_create(xitk_widget_list_t *wl,
   mywidget->width = width;
   mywidget->height = height;
   
-  return _xitk_intbox_create(wl, NULL, ib, NULL, mywidget, private_data, 1, 1);
+  return _xitk_intbox_create(wl, NULL, ib, NULL, mywidget, private_data, 0, 0);
 }
