@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: label.c,v 1.33 2002/07/17 20:29:31 f1rmb Exp $
+ * $Id: label.c,v 1.58 2003/12/14 14:10:33 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -30,13 +30,6 @@
 #include <pthread.h>
 #include <unistd.h>
 
-#include "Imlib-light/Imlib.h"
-#include "widget.h"
-#include "font.h"
-#include "image.h"
-#include "label.h"
-#include "widget_types.h"
-#include "utils.h"
 #include "_xitk.h"
 
 static void _create_label_pixmap(xitk_widget_t *w) {
@@ -65,12 +58,9 @@ static void _create_label_pixmap(xitk_widget_t *w) {
   pixwidth = private_data->char_length * 
     ((private_data->length * ((len / private_data->length) + 1)) + 5);
 
-  XLOCK (private_data->imlibdata->x.disp);
-  private_data->labelpix = XCreatePixmap(private_data->imlibdata->x.disp, 
-					 private_data->imlibdata->x.base_window, 
-					 (pixwidth) ? pixwidth : 1, private_data->char_height, 
-					 private_data->imlibdata->x.depth);
-  XUNLOCK (private_data->imlibdata->x.disp);
+  private_data->labelpix = xitk_image_create_xitk_pixmap(private_data->imlibdata,
+							 (pixwidth) ? pixwidth : 1, 
+							 private_data->char_height);
 
   x_dest = 0;
     
@@ -87,8 +77,8 @@ static void _create_label_pixmap(xitk_widget_t *w) {
       py = (c / 32) * private_data->char_height;
       
       XLOCK(private_data->imlibdata->x.disp);
-      XCopyArea(private_data->imlibdata->x.disp, font->image, 
-		private_data->labelpix, private_data->gc, px, py,
+      XCopyArea(private_data->imlibdata->x.disp, font->image->pixmap, 
+		private_data->labelpix->pixmap, private_data->labelpix->gc, px, py,
 		private_data->char_length, private_data->char_height, x_dest, 0);
       XUNLOCK(private_data->imlibdata->x.disp);
      
@@ -102,11 +92,10 @@ static void _create_label_pixmap(xitk_widget_t *w) {
 /*
  *
  */
-static void notify_destroy(xitk_widget_t *w, void *data) {
-  label_private_data_t *private_data;
+static void notify_destroy(xitk_widget_t *w) {
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
-    private_data = (label_private_data_t *) w->private_data;
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
+    label_private_data_t *private_data = (label_private_data_t *) w->private_data;
     
     private_data->on_change = 1;
     
@@ -117,19 +106,20 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
       pthread_join(private_data->thread, &dummy);
     }
     
-    if (private_data->labelpix != None) {
-      XLOCK (private_data->imlibdata->x.disp);
-      XFreePixmap(private_data->imlibdata->x.disp, private_data->labelpix);
-      private_data->labelpix = None;
-      XUNLOCK (private_data->imlibdata->x.disp);
+    if (private_data->labelpix) {
+      xitk_image_destroy_xitk_pixmap(private_data->labelpix);
+      private_data->labelpix = NULL;
     }
     
+    XITK_FREE(private_data->label);
     XITK_FREE(private_data->fontname);
     XITK_FREE(private_data->skin_element_name);
 
     if(private_data->font)
       xitk_image_free_image(private_data->imlibdata, &private_data->font);
 
+    pthread_mutex_destroy(&private_data->paint_mutex);
+
     XITK_FREE(private_data);
   }
 }
@@ -138,10 +128,9 @@ static void notify_destroy(xitk_widget_t *w, void *data) {
  *
  */
 static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
-  label_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
-    private_data = (label_private_data_t *) w->private_data;
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
+    label_private_data_t *private_data = (label_private_data_t *) w->private_data;
 
     if(sk == FOREGROUND_SKIN && private_data->font) {
       return private_data->font;
@@ -155,10 +144,9 @@ static xitk_image_t *get_skin(xitk_widget_t *w, int sk) {
  *
  */
 char *xitk_label_get_label(xitk_widget_t *w) {
-  label_private_data_t *private_data;
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
-    private_data = (label_private_data_t *) w->private_data;
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
+      label_private_data_t *private_data = (label_private_data_t *) w->private_data;
     return private_data->label;
   }
 
@@ -168,15 +156,14 @@ char *xitk_label_get_label(xitk_widget_t *w) {
 /*
  *
  */
-static void paint_label(xitk_widget_t *w, Window win, GC gc) {
-  label_private_data_t  *private_data;
-  xitk_image_t          *font;
+static void paint_label(xitk_widget_t *w) {
 
-  if (w && (((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL) && (w->visible == 1))) {
+  if (w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL) && (w->visible == 1))) {
+    label_private_data_t  *private_data = (label_private_data_t *) w->private_data;
+    xitk_image_t          *font = (xitk_image_t *) private_data->font;
+
+    pthread_mutex_lock(&private_data->paint_mutex);
     
-    private_data = (label_private_data_t *) w->private_data;
-    font = (xitk_image_t *) private_data->font;
-
     /* non skinable widget */
     if(private_data->skin_element_name == NULL) {
       xitk_font_t   *fs = NULL;
@@ -185,45 +172,56 @@ static void paint_label(xitk_widget_t *w, Window win, GC gc) {
 
       /* Clean old */
       XLOCK (private_data->imlibdata->x.disp);
-      XCopyArea (private_data->imlibdata->x.disp, private_data->font->image, win, 
-		 gc, 0, 0, private_data->font->width, private_data->font->height, w->x, w->y);
+      XCopyArea (private_data->imlibdata->x.disp, font->image->pixmap, w->wl->win, font->image->gc, 
+		 0, 0, private_data->font->width, font->height, w->x, w->y);
       XUNLOCK (private_data->imlibdata->x.disp);
 
       fs = xitk_font_load_font(private_data->imlibdata->x.disp, private_data->fontname);
-      xitk_font_set_font(fs, gc);
-      xitk_font_string_extent(fs, private_data->label, &lbear, &rbear, &wid, &asc, &des);
+      xitk_font_set_font(fs, private_data->font->image->gc);
+      xitk_font_string_extent(fs, (private_data->label && strlen(private_data->label)) ? private_data->label : "Label",
+			      &lbear, &rbear, &wid, &asc, &des);
 
       bg = xitk_image_create_image(private_data->imlibdata, w->width, w->height);
 
       XLOCK (private_data->imlibdata->x.disp);
-      XCopyArea (private_data->imlibdata->x.disp, private_data->font->image, bg->image, 
-		 gc, 0, 0, private_data->font->width, private_data->font->height, 0, 0);
-      XSetForeground(private_data->imlibdata->x.disp, gc, 
+      XCopyArea (private_data->imlibdata->x.disp, font->image->pixmap, bg->image->pixmap, 
+		 font->image->gc,
+		 0, 0, private_data->font->width, private_data->font->height, 0, 0);
+      XSetForeground(private_data->imlibdata->x.disp, font->image->gc, 
 		     xitk_get_pixel_color_black(private_data->imlibdata));
-      XDrawString(private_data->imlibdata->x.disp, bg->image, gc,
+      xitk_font_draw_string(fs, bg->image->pixmap, font->image->gc,
 		  2, ((private_data->font->height + asc + des)>>1) - des,
 		  private_data->label, strlen(private_data->label));
-      XCopyArea (private_data->imlibdata->x.disp, bg->image, win, 
-		 gc, 0, 0, private_data->font->width, private_data->font->height, w->x, w->y);
+      XCopyArea (private_data->imlibdata->x.disp, bg->image->pixmap, w->wl->win, 
+		 font->image->gc, 0, 0, font->width, font->height, w->x, w->y);
       XUNLOCK (private_data->imlibdata->x.disp);
 
       xitk_image_free_image(private_data->imlibdata, &bg);
       
       xitk_font_unload_font(fs);
       
+      pthread_mutex_unlock(&private_data->paint_mutex);
       return;
     }
     else {
       int width = private_data->char_length * private_data->length;
       
+
       XLOCK(private_data->imlibdata->x.disp);
-      XCopyArea(private_data->imlibdata->x.disp, private_data->labelpix, win, gc, 
-		private_data->anim_offset, 0, 
-		width, private_data->char_height, 
+      XCopyArea(private_data->imlibdata->x.disp, 
+		private_data->labelpix->pixmap, w->wl->win, font->image->gc, 
+		private_data->anim_offset, 0, width, private_data->char_height, 
 		w->x, w->y);
-      XUNLOCK(private_data->imlibdata->x.disp);
+
+      /* We can't wait here, otherwise the rolling effect is really jerky */
+      if(private_data->animation && (private_data->on_change == 0))
+	XSync(private_data->imlibdata->x.disp, False);
       
+      XUNLOCK(private_data->imlibdata->x.disp);
+
     }
+    
+    pthread_mutex_unlock(&private_data->paint_mutex);
   }
 }
 
@@ -233,35 +231,31 @@ static void paint_label(xitk_widget_t *w, Window win, GC gc) {
 void *xitk_label_animation_loop(void *data) {
   label_private_data_t *private_data = (label_private_data_t *)data;
   xitk_widget_t        *w            = private_data->lWidget;
-  unsigned long         t_anim       = xitk_get_timer_label_animation();
+  unsigned long         t_anim       = private_data->anim_timer;
+  
+  if(t_anim == 0)
+    t_anim = xitk_get_timer_label_animation();
   
   do {
     
     if((w->visible == 1)) {
       
-      private_data->anim_offset++;
+      private_data->anim_offset += private_data->anim_step;
 
-      if (private_data->anim_offset >
+      if (private_data->anim_offset > 
 	  (private_data->char_length * (strlen(private_data->label) + 5)))
 	private_data->anim_offset = 1;
-
+      
       /* 
        * Label will change sooner, don't try to paint it till the change,
        * otherwise a deadlock will happened.
        */
-      if(!private_data->on_change) {
-	paint_label(private_data->lWidget,
-		    private_data->window, private_data->gc);
-	
-	/* We can't wait here, otherwise the rolling effect is really jerky */
-	XLOCK (private_data->imlibdata->x.disp);
-	XSync(private_data->imlibdata->x.disp, False);
-	XUNLOCK (private_data->imlibdata->x.disp);
-      }
+      if(private_data->on_change == 0)
+	paint_label(private_data->lWidget);
+
     }
-    
     xitk_usec_sleep(t_anim);
-    
+
   } while(w->running && private_data->anim_running);
   
   pthread_exit(NULL);
@@ -275,17 +269,22 @@ static void label_setup_label(xitk_widget_t *w, char *label_) {
   int label_len;
   
   /* Inform animation thread to not paint the label */
+  pthread_mutex_lock(&private_data->paint_mutex);
   private_data->on_change = 1;
   
-  if (private_data->label) {
-    XITK_FREE(private_data->label);
+  if(private_data->anim_running) {
+    void *dummy;
+    
+    private_data->anim_running = 0;
+    pthread_join (private_data->thread, &dummy);
   }
+
+  if(private_data->label)
+    XITK_FREE(private_data->label);
   
-  if (private_data->labelpix != None) {
-    XLOCK (private_data->imlibdata->x.disp);
-    XFreePixmap(private_data->imlibdata->x.disp, private_data->labelpix);
-    XUNLOCK (private_data->imlibdata->x.disp);
-    private_data->labelpix = None;
+  if(private_data->labelpix) {
+    xitk_image_destroy_xitk_pixmap(private_data->labelpix);
+    private_data->labelpix = NULL;
   }
 
   label_len = strlen(label_);
@@ -297,53 +296,47 @@ static void label_setup_label(xitk_widget_t *w, char *label_) {
   else
     private_data->anim_offset = 0;
 
-  if (private_data->animation) {
+  if(private_data->animation) {
     
-    if (private_data->anim_running) {
-      void *dummy;
-      
-      private_data->anim_running = 0;
-      pthread_join (private_data->thread, &dummy);
-    }
-    
-    if (label_len > private_data->length) {
+    if(label_len > private_data->length) {
       pthread_attr_t       pth_attrs;
+#if ! defined (__OpenBSD__)
       struct sched_param   pth_params;
+#endif
       
       private_data->anim_running = 1;
       
       pthread_attr_init(&pth_attrs);
-      
+
       /* this won't work on linux, freebsd 5.0 */
+#if ! defined (__OpenBSD__)
       pthread_attr_getschedparam(&pth_attrs, &pth_params);
       pth_params.sched_priority = sched_get_priority_min(SCHED_OTHER);
       pthread_attr_setschedparam(&pth_attrs, &pth_params);
+#endif
       
       pthread_create(&private_data->thread, &pth_attrs, 
-		     xitk_label_animation_loop, (void *)private_data);
-      
-      pthread_attr_destroy(&pth_attrs);
+      		     xitk_label_animation_loop, (void *)private_data);
     }
   }
-  
+
   private_data->on_change = 0;
+  pthread_mutex_unlock(&private_data->paint_mutex);
 }
 
 /*
  *
  */
-static void notify_change_skin(xitk_widget_list_t *wl, 
-			       xitk_widget_t *w, xitk_skin_config_t *skonfig) {
-  label_private_data_t *private_data;
+static void notify_change_skin(xitk_widget_t *w, xitk_skin_config_t *skonfig) {
   
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
-    private_data = (label_private_data_t *) w->private_data;
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
+    label_private_data_t *private_data = (label_private_data_t *) w->private_data;
     
     if(private_data->skin_element_name) {
       
       xitk_skin_lock(skonfig);
 
-      XITK_FREE_XITK_IMAGE(private_data->imlibdata->x.disp, private_data->font);
+      xitk_image_free_image(private_data->imlibdata, &private_data->font);
       private_data->font        = xitk_image_load_image(private_data->imlibdata, 
 							xitk_skin_get_label_skinfont_filename(skonfig, private_data->skin_element_name));
       private_data->char_length = (private_data->font->width/32);
@@ -351,7 +344,8 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       
       private_data->length      = xitk_skin_get_label_length(skonfig, private_data->skin_element_name);
       private_data->animation   = xitk_skin_get_label_animation(skonfig, private_data->skin_element_name);
-      
+      private_data->anim_step   = xitk_skin_get_label_animation_step(skonfig, private_data->skin_element_name);
+      private_data->anim_timer  = xitk_skin_get_label_animation_timer(skonfig, private_data->skin_element_name);
       w->x                      = xitk_skin_get_coord_x(skonfig, private_data->skin_element_name);
       w->y                      = xitk_skin_get_coord_y(skonfig, private_data->skin_element_name);
       w->width                  = private_data->char_length * private_data->length;
@@ -359,10 +353,17 @@ static void notify_change_skin(xitk_widget_list_t *wl,
       w->visible                = (xitk_skin_get_visibility(skonfig, private_data->skin_element_name)) ? 1 : -1;
       w->enable                 = xitk_skin_get_enability(skonfig, private_data->skin_element_name);
       
-      xitk_skin_unlock(skonfig);
+      {
+	char *label;
+	
+	xitk_strdupa(label, private_data->label);
+	label_setup_label(w, label);
+      }
+      
+      if(private_data->on_change == 0)
+	paint_label(w);
 
-      if(private_data->skin_element_name != NULL)
-	_create_label_pixmap(w);
+      xitk_skin_unlock(skonfig);
 
       xitk_set_widget_pos(w, w->x, w->y);
     }
@@ -372,14 +373,20 @@ static void notify_change_skin(xitk_widget_list_t *wl,
 /*
  *
  */
-int xitk_label_change_label(xitk_widget_list_t *wl, xitk_widget_t *w, char *newlabel) {
-
-  if(w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
+int xitk_label_change_label(xitk_widget_t *w, char *newlabel) {
+  
+  if(w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
+    label_private_data_t *private_data = (label_private_data_t *) w->private_data;
     
-    label_setup_label(w, newlabel);
-    
-    paint_label(w, wl->win, wl->gc);
+    if(!newlabel || !private_data->label ||
+       (newlabel && (strcmp(private_data->label, newlabel)))) {
 
+      /* Don't change label if another thread is currently doing the same thing */
+      if(!private_data->on_change) {
+	label_setup_label(w, newlabel);
+	paint_label(w);
+      }
+    }
     return 1;
   }
 
@@ -389,21 +396,59 @@ int xitk_label_change_label(xitk_widget_list_t *wl, xitk_widget_t *w, char *newl
 /*
  *
  */
-static int notify_click_label(xitk_widget_list_t *wl, xitk_widget_t *w, int bUp, int x, int y) {
-  label_private_data_t *private_data;
-  
-  if (w && ((w->widget_type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
-    private_data = (label_private_data_t *) w->private_data;
-    
-    if(private_data->callback) {
-      if(bUp)
-	private_data->callback(private_data->lWidget, private_data->userdata);
-      return 1;
-    }
+static int notify_click_label(xitk_widget_t *w, int button, int bUp, int x, int y) {
+  int  ret = 0;
 
+  if (w && ((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL)) {
+    if(button == Button1) {
+      label_private_data_t *private_data = (label_private_data_t *) w->private_data;
+      
+      if(private_data->callback) {
+	if(bUp)
+	  private_data->callback(private_data->lWidget, private_data->userdata);
+	ret = 1;
+      }
+    }
   }
   
-  return 0;
+  return ret;
+}
+
+static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_result_t *result) {
+  int retval = 0;
+  
+  switch(event->type) {
+  case WIDGET_EVENT_PAINT:
+    {
+      if(w && (((w->type & WIDGET_TYPE_MASK) == WIDGET_TYPE_LABEL))) {
+	label_private_data_t  *private_data = (label_private_data_t *) w->private_data;
+
+	if(private_data->on_change == 0)
+	  paint_label(w);
+
+      }
+    }
+    break;
+  case WIDGET_EVENT_CLICK:
+    result->value = notify_click_label(w, event->button, 
+				       event->button_pressed, event->x, event->y);
+    retval = 1;
+    break;
+  case WIDGET_EVENT_CHANGE_SKIN:
+    notify_change_skin(w, event->skonfig);
+    break;
+  case WIDGET_EVENT_DESTROY:
+    notify_destroy(w);
+    break;
+  case WIDGET_EVENT_GET_SKIN:
+    if(result) {
+      result->image = get_skin(w, event->skin_layer);
+      retval = 1;
+    }
+    break;
+  }
+  
+  return retval;
 }
 
 /*
@@ -451,14 +496,15 @@ static xitk_widget_t *_xitk_label_create(xitk_widget_list_t *wl,
   private_data->label          = NULL;
   private_data->animation      = (skin_element_name == NULL) ? 0 : xitk_skin_get_label_animation(skonfig, private_data->skin_element_name);
   private_data->anim_running   = 0;
-  private_data->window         = l->window;
-  private_data->gc             = l->gc;
+  private_data->anim_step      = (skin_element_name == NULL) ? 1 : xitk_skin_get_label_animation_step(skonfig, private_data->skin_element_name);
+  private_data->anim_timer     = (skin_element_name == NULL) ? 0 : xitk_skin_get_label_animation_timer(skonfig, private_data->skin_element_name);
+  private_data->labelpix       = NULL;
 
-  private_data->labelpix       = None;
+  pthread_mutex_init(&private_data->paint_mutex, NULL);
 
   mywidget->private_data       = private_data;
 
-  mywidget->widget_list        = wl;
+  mywidget->wl                 = wl;
 
   mywidget->enable             = enable;
   mywidget->running            = 1;
@@ -476,16 +522,8 @@ static xitk_widget_t *_xitk_label_create(xitk_widget_list_t *wl,
     mywidget->width            = private_data->char_length * private_data->length;
     mywidget->height           = private_data->char_height;
   }
-  mywidget->widget_type        = WIDGET_TYPE_LABEL;
-  mywidget->paint              = paint_label;
-  mywidget->notify_click       = notify_click_label;
-  mywidget->notify_focus       = NULL;
-  mywidget->notify_keyevent    = NULL;
-  mywidget->notify_inside      = NULL;
-  mywidget->notify_change_skin = (skin_element_name == NULL) ? NULL : notify_change_skin;
-  mywidget->notify_destroy     = notify_destroy;
-  mywidget->get_skin           = get_skin;
-  
+  mywidget->type               = WIDGET_TYPE_LABEL | WIDGET_CLICKABLE | WIDGET_KEYABLE;
+  mywidget->event              = notify_event;
   mywidget->tips_timeout       = 0;
   mywidget->tips_string        = NULL;
 
@@ -521,5 +559,5 @@ xitk_widget_t *xitk_noskin_label_create(xitk_widget_list_t *wl,
 					int x, int y, int width, int height, char *fontname) {
   XITK_CHECK_CONSTITENCY(l);
 
-  return _xitk_label_create(wl, NULL, l, x, y, width, height, NULL, fontname, 1, 1);
+  return _xitk_label_create(wl, NULL, l, x, y, width, height, NULL, fontname, 0, 0);
 }
