@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: labelbutton.c,v 1.50 2003/03/27 20:52:05 f1rmb Exp $
+ * $Id: labelbutton.c,v 1.53 2003/12/14 14:10:33 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -121,7 +121,7 @@ static void create_labelofbutton(xitk_widget_t *lb,
     XITK_DIE("%s()@%d: xitk_font_load_font() failed. Exiting\n", __FUNCTION__, __LINE__);
   
   xitk_font_set_font(fs, gc);
-  xitk_font_string_extent(fs, label, &lbear, &rbear, &width, &asc, &des);
+  xitk_font_string_extent(fs, (label && strlen(label)) ? label : "Button", &lbear, &rbear, &width, &asc, &des);
 
   if((state == CLICK) && (private_data->label_static == 0)) {
     xoff = -4;
@@ -184,19 +184,19 @@ static void create_labelofbutton(xitk_widget_t *lb,
   
   /*  Put text in the right place */
   if(private_data->align == ALIGN_CENTER) {
-    XDrawString(private_data->imlibdata->x.disp, pix, gc, 
+    xitk_font_draw_string(fs, pix, gc, 
 		((xsize-(width+xoff))>>1) + private_data->label_offset, 
 		((ysize+asc+des+yoff)>>1)-des, 
 		label, strlen(label));
   }
   else if(private_data->align == ALIGN_LEFT) {
-    XDrawString(private_data->imlibdata->x.disp, pix, gc, 
+    xitk_font_draw_string(fs, pix, gc, 
 		(((state != CLICK) ? 1 : 5)) + private_data->label_offset, 
 		((ysize+asc+des+yoff)>>1)-des, 
 		label, strlen(label));
   }
   else if(private_data->align == ALIGN_RIGHT) {
-    XDrawString(private_data->imlibdata->x.disp, pix, gc, 
+    xitk_font_draw_string(fs, pix, gc, 
     		(xsize - (width + ((state != CLICK) ? 5 : 1))) + private_data->label_offset,
     		((ysize+asc+des+yoff)>>1)-des, 
     		label, strlen(label));
@@ -746,5 +746,5 @@ xitk_widget_t *xitk_noskin_labelbutton_create (xitk_widget_list_t *wl,
   i = xitk_image_create_image(b->imlibdata, width * 3, height);
   draw_bevel_three_state(b->imlibdata, i);
 
-  return _xitk_labelbutton_create(wl, NULL, b, x, y, NULL, i, ncolor, fcolor, ccolor, fname, 1, 0, 1, 1);
+  return _xitk_labelbutton_create(wl, NULL, b, x, y, NULL, i, ncolor, fcolor, ccolor, fname, 1, 0, 0, 0);
 }
