@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: mrlbrowser.c,v 1.68 2003/01/05 00:24:35 f1rmb Exp $
+ * $Id: mrlbrowser.c,v 1.72 2003/11/10 21:04:42 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -32,7 +32,6 @@
 #include <string.h>
 #include <sys/types.h>
 #include <unistd.h>                                                      
-#include <assert.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
@@ -194,7 +193,7 @@ static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
 
     private_data->mc->mrls_to_disp = 0;
 
-    xitk_strdupa(filter_ends, private_data->mrl_filters[private_data->filter_selected]->ending);
+    filter_ends = strdup(private_data->mrl_filters[private_data->filter_selected]->ending);
     xitk_strdupa(p, filter_ends);
 
     for(i = 0; i < private_data->mrls_num; i++) {
@@ -222,10 +221,15 @@ static void mrlbrowser_filter_mrls(mrlbrowser_private_data_t *private_data) {
 	      private_data->mc->mrls_to_disp++;
 	    }
 	  }
-	  xitk_strdupa(filter_ends, p);
+	  free(filter_ends);
+	  filter_ends = strdup(p);
 	}
       }    
     }
+    
+    if(filter_ends)
+      free(filter_ends);
+    
   }
   else { /* no filtering */
 
@@ -1274,6 +1278,7 @@ xitk_widget_t *xitk_mrlbrowser_create(xitk_widget_list_t *wl,
   _duplicate_mrl_filters(private_data, mb->mrl_filters);
 
   cmb.skin_element_name = mb->combo.skin_element_name;
+  cmb.layer_above       = mb->layer_above;
   cmb.parent_wlist      = private_data->widget_list;
   cmb.entries           = private_data->filters;
   cmb.parent_wkey       = &private_data->widget_key;
