@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tabs.c,v 1.23 2003/02/10 17:59:15 f1rmb Exp $
+ * $Id: tabs.c,v 1.26 2003/12/21 14:22:19 mroi Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -29,28 +29,21 @@
 #include "_xitk.h"
 
 
-typedef struct {
-  xitk_widget_t    *itemlist;
-  int               sel;
-} btnlist_t;
-
 static void tabs_arrange(xitk_widget_t *);
 
 /*
  *
  */
 static void enability(xitk_widget_t *w) {
-  tabs_private_data_t *private_data;
   
   if(w && (((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_TABS) &&
 	   (w->type & WIDGET_GROUP_WIDGET))) {
-    int i;
-    
-    private_data = (tabs_private_data_t *) w->private_data;
+    tabs_private_data_t *private_data = (tabs_private_data_t *) w->private_data;
+    int                  i;
 
     if(w->enable == WIDGET_ENABLE) {
-      xitk_enable_widget(private_data->left);
-      xitk_enable_widget(private_data->right);
+      xitk_enable_and_show_widget(private_data->left);
+      xitk_enable_and_show_widget(private_data->right);
       for(i = 0; i < private_data->num_entries; i++)
 	xitk_enable_widget(private_data->tabs[i]);
     }
@@ -64,7 +57,6 @@ static void enability(xitk_widget_t *w) {
   }
 }
   
-/*
 static void notify_destroy(xitk_widget_t *w) {
   tabs_private_data_t *private_data;
   
@@ -74,14 +66,14 @@ static void notify_destroy(xitk_widget_t *w) {
     
     private_data = (tabs_private_data_t *) w->private_data;
 
+    for(i = 0; i <= private_data->num_entries; i++)
+      XITK_FREE(private_data->bt[i]);
+
     XITK_FREE(private_data->skin_element_name);
-    for(i = 0; i <= private_data->num_entries; i++) {
-      xitk_destroy_widget(private_data->tabs[i]);
-    }
     XITK_FREE(private_data);
   }
 }
-*/
+
 /*
  *
  */
@@ -198,6 +190,9 @@ static int notify_event(xitk_widget_t *w, widget_event_t *event, widget_event_re
   case WIDGET_EVENT_PAINT:
     paint(w);
     break;
+  case WIDGET_EVENT_DESTROY:
+    notify_destroy(w);
+    break;
   case WIDGET_EVENT_ENABLE:
     enability(w);
     break;
@@ -363,7 +358,6 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_widget_list_t *wl,
     xitk_labelbutton_widget_t  lb;
     xitk_button_widget_t       b;
     int                        xx = x;
-    btnlist_t                 *bt;
       
     fs = xitk_font_load_font(t->imlibdata->x.disp, fontname);
 
@@ -377,9 +371,9 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_widget_list_t *wl,
 
     for(i = 0; i < t->num_entries; i++) {
 
-      bt = (btnlist_t *) xitk_xmalloc(sizeof(btnlist_t));
-      bt->itemlist = mywidget;
-      bt->sel = i;
+      private_data->bt[i]           = (btnlist_t *) xitk_xmalloc(sizeof(btnlist_t));
+      private_data->bt[i]->itemlist = mywidget;
+      private_data->bt[i]->sel      = i;
 
       fwidth = xitk_font_get_string_length(fs, t->entries[i]);
 
@@ -389,7 +383,7 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_widget_list_t *wl,
       lb.label             = t->entries[i];
       lb.callback          = NULL;
       lb.state_callback    = tabs_select;
-      lb.userdata          = (void *)bt;
+      lb.userdata          = (void *) (private_data->bt[i]);
       xitk_list_append_content (t->parent_wlist->l, 
 			(private_data->tabs[i] = 
 			 xitk_noskin_labelbutton_create (t->parent_wlist, &lb, xx, y, fwidth + 20, 
@@ -451,8 +445,8 @@ xitk_widget_t *xitk_noskin_tabs_create(xitk_widget_list_t *wl,
   mywidget->wl                    = wl;
 
   mywidget->enable                = 1;
-  mywidget->running               = 1;
-  mywidget->visible               = 1;
+  mywidget->running               = 0;
+  mywidget->visible               = 0;
 
   mywidget->have_focus            = FOCUS_LOST; 
   mywidget->imlibdata             = private_data->imlibdata;
