@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 2001-2002 Daniel Caujolle-Bert <segfault@club-internet.fr>
+** Copyright (C) 2001-2003 Daniel Caujolle-Bert <segfault@club-internet.fr>
 **  
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -82,9 +82,16 @@ typedef struct {
   xitk_widget_t        *intbox;
   int                   oldintvalue;
 
+  /* Doublebox */
+  xitk_widget_t        *doublebox;
+  double                olddoublevalue;
+
   /* checkbox */
   xitk_widget_t        *checkbox;
 
+  /* menu */
+  xitk_widget_t        *menu;
+
   xitk_widget_list_t   *widget_list;
   xitk_register_key_t   kreg;
 } test_t;
@@ -92,14 +99,14 @@ typedef struct {
 
 static test_t *test;
 static int nlab = 0;
-static int align = LABEL_ALIGN_LEFT;
+static int align = ALIGN_LEFT;
 #define FONT_HEIGHT_MODEL "azertyuiopqsdfghjklmwxcvbnAZERTYUIOPQSDFGHJKLMWXCVBN&(-_)=~#{[|`\\^@]}%"
 
+static void create_menu(void);
 /*
  *
  */
 static int init_test(void) {
-  char              *display_name = ":0.0";
   ImlibInitParams    imlib_init;
   int                screen;
   int		     depth = 0;
@@ -112,10 +119,7 @@ static int init_test(void) {
 
   test = (test_t *) xitk_xmalloc(sizeof(test_t));
   
-  if(getenv("DISPLAY"))
-    display_name = getenv("DISPLAY");
-  
-  if((test->display = XOpenDisplay(display_name)) == NULL) {
+  if((test->display = XOpenDisplay((getenv("DISPLAY")))) == NULL) {
     fprintf(stderr, _("Cannot open display\n"));
     return 0;
   }
@@ -134,7 +138,7 @@ static int init_test(void) {
     return 0;
   }
 
-  xitk_init(test->display);
+  xitk_init(test->display, 1);
 
   XUnlockDisplay (test->display);
 
@@ -145,14 +149,28 @@ static int init_test(void) {
  *
  */
 static void deinit_test(void) {
+  int i = 0;
+
+  while(test->entries[i]) {
+    free(test->entries[i]);
+    i++;
+  }
 }
 
 /*
  *
  */
 static void test_end(xitk_widget_t *w, void *data) {
-  XUnmapWindow(test->display, (xitk_window_get_window(test->xwin)));
+  xitk_unregister_event_handler(&test->kreg);
+
   xitk_destroy_widgets(test->widget_list);
+  xitk_window_destroy_window(test->imlibdata, test->xwin);
+
+  xitk_list_free((XITK_WIDGET_LIST_LIST(test->widget_list)));
+  XFreeGC(test->display, (XITK_WIDGET_LIST_GC(test->widget_list)));
+
+  free(test->widget_list);
+
   xitk_stop();
 }
 
@@ -173,6 +191,15 @@ void test_handle_event(XEvent *event, void *data) {
     XUnlockDisplay(test->display);
     break;
 
+  case ButtonPress:
+    {
+      XButtonEvent *bevent = (XButtonEvent *) event;
+      
+      if(bevent->button == Button3)
+	create_menu();
+    }
+    break;
+
   case KeyPress: {
     int modifier;
     
@@ -213,16 +240,16 @@ void test_handle_event(XEvent *event, void *data) {
  */
 static void move_sliders(xitk_widget_t *w, void *data, int pos) {
   if(w == test->vslider) {
-    xitk_slider_set_pos(test->widget_list, test->hslider, pos);
-    xitk_slider_set_pos(test->widget_list, test->rslider, pos);
+    xitk_slider_set_pos(test->hslider, pos);
+    xitk_slider_set_pos(test->rslider, pos);
   }
   else if(w == test->hslider) {
-    xitk_slider_set_pos(test->widget_list, test->vslider, pos);
-    xitk_slider_set_pos(test->widget_list, test->rslider, pos);
+    xitk_slider_set_pos(test->vslider, pos);
+    xitk_slider_set_pos(test->rslider, pos);
   }
   else if(w == test->rslider) {
-    xitk_slider_set_pos(test->widget_list, test->vslider, pos);
-    xitk_slider_set_pos(test->widget_list, test->hslider, pos);
+    xitk_slider_set_pos(test->vslider, pos);
+    xitk_slider_set_pos(test->hslider, pos);
   }
 
 }
@@ -259,17 +286,18 @@ static void window_message_cb(xitk_widget_t *w, void *data, int btn) {
  *
  */
 static void change_label(xitk_widget_t *w, void *data) {
+  xitk_window_t *xw;
   static char *labels[] = {
     "A Label",
     "Boom"
   };
 
-  if((++align) > LABEL_ALIGN_RIGHT)
-    align = LABEL_ALIGN_CENTER;
+  if((++align) > ALIGN_RIGHT)
+    align = ALIGN_CENTER;
 
   nlab = !nlab;
 
-  xitk_label_change_label(test->widget_list, test->label, labels[nlab]);
+  xitk_label_change_label(test->label, labels[nlab]);
 
   xitk_browser_set_alignment(test->browser, align);
   
@@ -278,18 +306,17 @@ static void change_label(xitk_widget_t *w, void *data) {
   //xitk_window_dialog_ok_with_width(test->imlibdata, "Long error message", window_message_cb, NULL, 500, ALIGN_DEFAULT, "** This program is free software; you can redistribute it and/or modify\n** it under the terms of the GNU General Public License as published by\n** the Free Software Foundation; either version 2 of the License, or\n** (at your option) any later version.");
   // xitk_window_dialog_yesno(test->imlibdata, NULL, NULL, NULL, NULL, ALIGN_LEFT, "Le programme <linux kernel> a provoqu une faute de protection dans le module <unknown>  l'adresse 0x00001234.\nCitron dump:\nAX:0x00\t\tBX:0x00\nCX:0x00\t\tGS:0x00;-)");
   //  xitk_window_dialog_ok_with_width(test->imlibdata, "Long error message", window_message_cb, NULL, 500, ALIGN_DEFAULT, "**Thisprogramisfreesoftware;youcanredistributeitand/ormodify**itunderthetermsoftheGNUGeneralPublicLicenseaspublishedby**TheFreeSoftwareFoundation;eitherversion2oftheLicense,or**(atyouroption)anylaterversion.");
-  xitk_window_dialog_error(test->imlibdata, 
-			   "Stream number %d <%s.mpg> is not valid.\n", nlab, labels[nlab]);
+  xw = xitk_window_dialog_error(test->imlibdata, 
+				"Stream number %d <%s.mpg> is not valid.\n", nlab, labels[nlab]);
 }
 
 /*
  *
  */
-static void change_inputtext(xitk_widget_t *w, void *data) {
-  int selected = (int)data;
-  
+static void change_inputtext(xitk_widget_t *w, void *data, int selected) {
+
   if(test->entries[selected] != NULL)
-    xitk_inputtext_change_text(test->widget_list, test->input, test->entries[selected]);
+    xitk_inputtext_change_text(test->input, test->entries[selected]);
 
 }
 
@@ -299,7 +326,7 @@ static void change_inputtext(xitk_widget_t *w, void *data) {
 static void change_inputtext_dbl_click(xitk_widget_t *w, void *data, int selected) {
 
   if(test->entries[selected] != NULL)
-    xitk_inputtext_change_text(test->widget_list, test->input, test->entries[selected]);
+    xitk_inputtext_change_text(test->input, test->entries[selected]);
 
 }
 
@@ -361,13 +388,65 @@ static void create_intbox(void) {
   ib.parent_wlist      = test->widget_list;
   ib.callback          = notify_intbox_change;
   ib.userdata          = NULL;
-  xitk_list_append_content (test->widget_list->l,
+  xitk_list_append_content (XITK_WIDGET_LIST_LIST(test->widget_list),
 	    (test->intbox = 
 	     xitk_noskin_intbox_create(test->widget_list, &ib, x, y, 60, 20, NULL, NULL, NULL)));
-
+  xitk_enable_and_show_widget(test->intbox);
   xitk_set_widget_tips_default(test->intbox, "This is a intbox");
 }
 
+/*
+ *
+ */
+static void doublechange_cb(xitk_widget_t *w, void *data, int btn) {
+
+  switch(btn) {
+  case XITK_WINDOW_ANSWER_YES:
+    test->olddoublevalue = xitk_doublebox_get_value(test->doublebox);
+    break;
+    
+  case XITK_WINDOW_ANSWER_NO:
+  case XITK_WINDOW_ANSWER_CANCEL:
+    xitk_doublebox_set_value(test->doublebox, test->olddoublevalue);
+    break;
+  }
+}
+
+/*
+ *
+ */
+static void notify_doublebox_change(xitk_widget_t *w, void *data, double value) {
+
+  xitk_window_dialog_yesnocancel(test->imlibdata, NULL,
+				 doublechange_cb, 
+				 doublechange_cb, 
+				 doublechange_cb, 
+				 NULL, ALIGN_DEFAULT, 
+				 _("New double value is: %e. Confirm ?"), value);
+}
+
+/*
+ *
+ */
+static void create_doublebox(void) {
+  int x = 150, y = 330;
+  xitk_doublebox_widget_t ib;
+
+  XITK_WIDGET_INIT(&ib, test->imlibdata);
+
+  ib.skin_element_name = NULL;
+  ib.value             = test->olddoublevalue = 1.500;
+  ib.step              = .5;
+  ib.parent_wlist      = test->widget_list;
+  ib.callback          = notify_doublebox_change;
+  ib.userdata          = NULL;
+  xitk_list_append_content (XITK_WIDGET_LIST_LIST(test->widget_list),
+	    (test->doublebox = 
+	     xitk_noskin_doublebox_create(test->widget_list, &ib, x, y, 60, 20, NULL, NULL, NULL)));
+  xitk_enable_and_show_widget(test->doublebox);
+  xitk_set_widget_tips_default(test->doublebox, "This is a doublebox");
+}
+
 /*
  *
  */
@@ -380,9 +459,10 @@ static void create_checkbox(void) {
   cb.skin_element_name = NULL;
   cb.callback          = NULL;
   cb.userdata          = NULL;
-  xitk_list_append_content (test->widget_list->l,
+  xitk_list_append_content (XITK_WIDGET_LIST_LIST(test->widget_list),
 		    (test->checkbox = 
 		     xitk_noskin_checkbox_create(test->widget_list, &cb, x, y, 20, 20)));
+  xitk_enable_and_show_widget(test->checkbox);
 
   xitk_set_widget_tips_default(test->checkbox, "This is a checkbox");
 }
@@ -391,7 +471,7 @@ static void create_checkbox(void) {
  *
  */
 static void create_tabs(void) {
-  Pixmap              bg;
+  xitk_pixmap_t      *bg;
   xitk_tabs_widget_t  t;
   char               *fontname = "*-helvetica-medium-r-*-*-12-*-*-*-*-*-*-*";
   int                 x = 150, y = 200, w = 300;
@@ -403,13 +483,13 @@ static void create_tabs(void) {
   XITK_WIDGET_INIT(&t, test->imlibdata);
 
   xitk_window_get_window_size(test->xwin, &width, &height);
-  bg = xitk_image_create_pixmap(test->imlibdata, width, height);
-  XCopyArea(test->display, (xitk_window_get_background(test->xwin)), bg,
-	    test->widget_list->gc, 0, 0, width, height, 0, 0);
+  bg = xitk_image_create_xitk_pixmap(test->imlibdata, width, height);
+  XCopyArea(test->display, (xitk_window_get_background(test->xwin)), bg->pixmap,
+	    bg->gc, 0, 0, width, height, 0, 0);
   
   draw_rectangular_outter_box(test->imlibdata, bg, x, y+20, (w-1), 60);
-  xitk_window_change_background(test->imlibdata, test->xwin, bg, width, height);
-  XFreePixmap(test->display, bg);
+  xitk_window_change_background(test->imlibdata, test->xwin, bg->pixmap, width, height);
+  xitk_image_destroy_xitk_pixmap(bg);
   
   t.skin_element_name = NULL;
   t.num_entries       = 9;
@@ -417,9 +497,10 @@ static void create_tabs(void) {
   t.parent_wlist      = test->widget_list;
   t.callback          = NULL;
   t.userdata          = NULL;
-  xitk_list_append_content (test->widget_list->l,
+  xitk_list_append_content (XITK_WIDGET_LIST_LIST(test->widget_list),
 		    (test->tabs = 
 		     xitk_noskin_tabs_create(test->widget_list, &t, x, y, w, fontname)));
+  xitk_enable_and_show_widget(test->tabs);
 
 }
 
@@ -427,21 +508,21 @@ static void create_tabs(void) {
  *
  */
 static void create_frame(void) {
-  Pixmap    bg;
-  int       width, height;
-  char     *fontname = "*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
-  int       x = 350, y = 50, w = 200, h = 150;
+  xitk_pixmap_t  *bg;
+  int             width, height;
+  char           *fontname = "*-helvetica-bold-r-*-*-12-*-*-*-*-*-*-*";
+  int              x = 350, y = 50, w = 200, h = 150;
   
   xitk_window_get_window_size(test->xwin, &width, &height);
-  bg = xitk_image_create_pixmap(test->imlibdata, width, height);
-  XCopyArea(test->display, (xitk_window_get_background(test->xwin)), bg,
-	    test->widget_list->gc, 0, 0, width, height, 0, 0);
+  bg = xitk_image_create_xitk_pixmap(test->imlibdata, width, height);
+  XCopyArea(test->display, (xitk_window_get_background(test->xwin)), bg->pixmap,
+	    bg->gc, 0, 0, width, height, 0, 0);
   
   draw_outter_frame(test->imlibdata, bg, _("My Frame"), fontname, x, y, w, h);
   draw_inner_frame(test->imlibdata, bg, NULL, NULL, x+(w>>2), y+(h>>2), w>>1, h>>1);
   
-  xitk_window_change_background(test->imlibdata, test->xwin, bg, width, height);
-  XFreePixmap(test->display, bg);
+  xitk_window_change_background(test->imlibdata, test->xwin, bg->pixmap, width, height);
+  xitk_image_destroy_xitk_pixmap(bg);
 }
 
 /*
@@ -458,11 +539,12 @@ static void create_inputtext(void) {
   inp.max_length        = 256;
   inp.callback          = change_browser_entry;
   inp.userdata          = NULL;
-  xitk_list_append_content (test->widget_list->l,
+  xitk_list_append_content (XITK_WIDGET_LIST_LIST(test->widget_list),
 	   (test->input = 
 	    xitk_noskin_inputtext_create(test->widget_list, &inp,
 					 150, 150, 150, 20,
 					 "Black", "Black", fontname)));
+  xitk_enable_and_show_widget(test->input);
 
   xitk_set_widget_tips_default(test->input, "This is an inputtext");
 }
@@ -481,19 +563,20 @@ static void create_label(void) {
   XITK_WIDGET_INIT(&lbl, test->imlibdata);
 
   fs = xitk_font_load_font(test->display, fontname);
-  xitk_font_set_font(fs, test->widget_list->gc);
+  xitk_font_set_font(fs, XITK_WIDGET_LIST_GC(test->widget_list));
   xitk_font_string_extent(fs, label, &lbear, &rbear, &wid, &asc, &des);
   xitk_font_unload_font(fs);
 
   lbl.window            = xitk_window_get_window(test->xwin);
-  lbl.gc                = test->widget_list->gc;
+  lbl.gc                = XITK_WIDGET_LIST_GC(test->widget_list);
   lbl.skin_element_name = NULL;
   lbl.label             = label;
   lbl.callback          = NULL;
-  xitk_list_append_content(test->widget_list->l, 
+  xitk_list_append_content(XITK_WIDGET_LIST_LIST(test->widget_list), 
 	   (test->label = 
 	    xitk_noskin_label_create(test->widget_list, &lbl,
 				     x, y, len, (asc+des)*2, fontname)));
+  xitk_enable_and_show_widget(test->label);
 
   xitk_set_widget_tips_default(test->label, "This is a label");
 
@@ -501,7 +584,8 @@ static void create_label(void) {
     xitk_image_t *wimage = xitk_get_widget_foreground_skin(test->label);
     
     if(wimage) {
-      draw_rectangular_inner_box(test->imlibdata, wimage->image, 0, 0, wimage->width-1, wimage->height-1);
+      draw_rectangular_inner_box(test->imlibdata, 
+				 wimage->image, 0, 0, wimage->width-1, wimage->height-1);
     }
   }
 }
@@ -518,11 +602,12 @@ static void create_button(void) {
   b.skin_element_name = NULL;
   b.callback          = change_label;
   b.userdata          = NULL;
-  xitk_list_append_content(test->widget_list->l, 
+  xitk_list_append_content(XITK_WIDGET_LIST_LIST(test->widget_list), 
 	   (test->button = 
 	    xitk_noskin_button_create(test->widget_list, &b,
 				      x, y, width, height)));
-  
+  xitk_enable_and_show_widget(test->button);
+
   xitk_set_widget_tips_default(test->button, "This is a button");
 
   { /* Draw red spot. */
@@ -538,36 +623,36 @@ static void create_button(void) {
 
       col = xitk_get_pixel_color_from_rgb(test->imlibdata, 255, 0, 0);
 
-      XSetForeground(test->display, test->widget_list->gc, col);
-      XSetBackground(test->display, test->widget_list->gc, col);
-      XFillArc(test->display, wimage->image, test->widget_list->gc,
+      XSetForeground(test->display, XITK_WIDGET_LIST_GC(test->widget_list), col);
+      XSetBackground(test->display, XITK_WIDGET_LIST_GC(test->widget_list), col);
+      XFillArc(test->display, wimage->image->pixmap, XITK_WIDGET_LIST_GC(test->widget_list),
 	       10, (height >> 1) - 13, 26, 26, 
 	       (0 * 64), (360 * 64));
-      XFillArc(test->display, wimage->image, test->widget_list->gc,
+      XFillArc(test->display, wimage->image->pixmap, XITK_WIDGET_LIST_GC(test->widget_list),
 	       (width) + 10, (height >> 1) - 13, 26, 26, 
 	       (0 * 64), (360 * 64));
-      XFillArc(test->display, wimage->image, test->widget_list->gc,
+      XFillArc(test->display, wimage->image->pixmap, XITK_WIDGET_LIST_GC(test->widget_list),
 	       ((width*2) + 10) + 1, ((height >> 1) - 13) + 1, 26, 26, 
 	       (0 * 64), (360 * 64));
 
 
       fs = xitk_font_load_font(test->display, fontname);
-      xitk_font_set_font(fs, test->widget_list->gc);
+      xitk_font_set_font(fs, XITK_WIDGET_LIST_GC(test->widget_list));
       xitk_font_string_extent(fs, label, &lbear, &rbear, &wid, &asc, &des);
       
       col = xitk_get_pixel_color_black(test->imlibdata);
 
-      XSetForeground(test->display, test->widget_list->gc, col);
-      XDrawString(test->display, wimage->image, test->widget_list->gc, 
+      XSetForeground(test->display, XITK_WIDGET_LIST_GC(test->widget_list), col);
+      xitk_font_draw_string(fs, wimage->image->pixmap, XITK_WIDGET_LIST_GC(test->widget_list), 
 		  50, ((height+asc+des) >> 1) - des, label, strlen(label));
-      XDrawString(test->display, wimage->image, test->widget_list->gc, 
+      xitk_font_draw_string(fs, wimage->image->pixmap, XITK_WIDGET_LIST_GC(test->widget_list), 
 		  (width) + 50, ((height+asc+des) >> 1) - des, label, strlen(label));
       
       {
 	char *nlabel = _("!BOOM!");
 
       xitk_font_string_extent(fs, nlabel, &lbear, &rbear, &wid, &asc, &des);
-      XDrawString(test->display, wimage->image, test->widget_list->gc, 
+      xitk_font_draw_string(fs, wimage->image->pixmap, XITK_WIDGET_LIST_GC(test->widget_list), 
 		  ((width * 2) + 50) + 1, (((height+asc+des) >> 1) - des) + 1, 
 		  nlabel, strlen(nlabel));
       }
@@ -596,11 +681,12 @@ static void create_sliders(void) {
   sl.userdata                 = NULL;
   sl.motion_callback          = move_sliders;
   sl.motion_userdata          = NULL;
-  xitk_list_append_content(test->widget_list->l,
+  xitk_list_append_content(XITK_WIDGET_LIST_LIST(test->widget_list),
 		   (test->hslider = xitk_noskin_slider_create(test->widget_list, &sl,
-							      17, 200, 117, 20,
+							      17, 208, 117, 20,
 							      XITK_HSLIDER)));
-  xitk_slider_set_pos(test->widget_list, test->hslider, 0);
+  xitk_enable_and_show_widget(test->hslider);
+  xitk_slider_set_pos(test->hslider, 0);
 
   xitk_set_widget_tips_default(test->hslider, "This is an horizontal slider");
 
@@ -612,11 +698,12 @@ static void create_sliders(void) {
   sl.userdata                 = NULL;
   sl.motion_callback          = move_sliders;
   sl.motion_userdata          = NULL;
-  xitk_list_append_content(test->widget_list->l,
+  xitk_list_append_content(XITK_WIDGET_LIST_LIST(test->widget_list),
 		   (test->vslider = xitk_noskin_slider_create(test->widget_list, &sl,
 							      17, 230, 20, 117,
 							      XITK_VSLIDER)));
-  xitk_slider_set_pos(test->widget_list, test->vslider, 0);
+  xitk_enable_and_show_widget(test->vslider);
+  xitk_slider_set_pos(test->vslider, 0);
 
   xitk_set_widget_tips_default(test->vslider, "This is a vertical slider");
 
@@ -628,11 +715,12 @@ static void create_sliders(void) {
   sl.userdata                 = NULL;
   sl.motion_callback          = move_sliders;
   sl.motion_userdata          = NULL;
-  xitk_list_append_content(test->widget_list->l,
+  xitk_list_append_content(XITK_WIDGET_LIST_LIST(test->widget_list),
 		   (test->rslider = xitk_noskin_slider_create(test->widget_list, &sl,
 							      50, 240, 80, 80,
 							      XITK_RSLIDER)));
-  xitk_slider_set_pos(test->widget_list, test->rslider, 0);
+  xitk_enable_and_show_widget(test->rslider);
+  xitk_slider_set_pos(test->rslider, 0);
   
   xitk_set_widget_tips_default(test->rslider, "This is a rotate button");
 
@@ -671,11 +759,11 @@ static void create_combo(void) {
   /*
   xitk_window_get_window_size(test->xwin, &wwidth, &wheight);
   bg = xitk_image_create_pixmap(test->imlibdata, wwidth, wheight);
-  XCopyArea(test->display, (xitk_window_get_background(test->xwin)), bg, test->widget_list->gc,
+  XCopyArea(test->display, (xitk_window_get_background(test->xwin)), bg, XITK_WIDGET_LIST_GC(test->widget_list),
 	    0, 0, wwidth, wheight, 0, 0);
   */
   fs = xitk_font_load_font(test->display, fontname);
-  xitk_font_set_font(fs, test->widget_list->gc);
+  xitk_font_set_font(fs, XITK_WIDGET_LIST_GC(test->widget_list));
   height = xitk_font_get_string_height(fs, FONT_HEIGHT_MODEL);
   xitk_font_unload_font(fs);
   /*
@@ -688,13 +776,16 @@ static void create_combo(void) {
   cmb.skin_element_name = NULL;
   cmb.parent_wlist      = test->widget_list;
   cmb.entries           = test->entries;
+  cmb.layer_above       = 0;
   cmb.parent_wkey       = &test->kreg;
   cmb.callback          = combo_select;
   cmb.userdata          = NULL;
-  xitk_list_append_content(test->widget_list->l, 
+  xitk_list_append_content(XITK_WIDGET_LIST_LIST(test->widget_list), 
 		   (test->combo = 
 		    xitk_noskin_combo_create(test->widget_list, &cmb,
 					     x, y, width, NULL, NULL)));
+  xitk_set_widget_tips_default(test->combo, "This is a combo box.");
+  xitk_enable_and_show_widget(test->combo);
 }
 
 /*
@@ -710,12 +801,12 @@ static void create_browser(void) {
 
   xitk_window_get_window_size(test->xwin, &width, &height);
   bg = xitk_image_create_pixmap(test->imlibdata, width, height);
-  XCopyArea(test->display, (xitk_window_get_background(test->xwin)), bg, test->widget_list->gc,
+  XCopyArea(test->display, (xitk_window_get_background(test->xwin)), bg, XITK_WIDGET_LIST_GC(test->widget_list),
 	    0, 0, width, height, 0, 0);
 
-  XSetForeground(test->display, test->widget_list->gc, 
+  XSetForeground(test->display, XITK_WIDGET_LIST_GC(test->widget_list), 
 		 xitk_get_pixel_color_black(test->imlibdata));
-  XDrawRectangle(test->display, bg, test->widget_list->gc, 17, 27, 117, 165);
+  XDrawRectangle(test->display, bg, XITK_WIDGET_LIST_GC(test->widget_list), 17, 27, 117, 176);
 
   xitk_window_change_background(test->imlibdata, 
 				test->xwin, bg, width, height);
@@ -731,6 +822,7 @@ static void create_browser(void) {
       test->entries[i] = strdup(buf);
     }
     
+    test->entries[i] = NULL;
     test->num_entries = i;
   }
 
@@ -740,21 +832,105 @@ static void create_browser(void) {
   browser.browser.skin_element_name     = NULL;
   browser.browser.max_displayed_entries = 8;
   browser.browser.num_entries           = test->num_entries;
-  browser.browser.entries               = test->entries;
+  browser.browser.entries               = (const char *const *)test->entries;
   browser.callback                      = change_inputtext;
   browser.dbl_click_callback            = change_inputtext_dbl_click;
   browser.parent_wlist                  = test->widget_list;
   browser.userdata                      = NULL;
-  xitk_list_append_content (test->widget_list->l, 
+  xitk_list_append_content (XITK_WIDGET_LIST_LIST(test->widget_list), 
 		    (test->browser = 
 		     xitk_noskin_browser_create(test->widget_list, &browser,
-						test->widget_list->gc, 20, 30, 
+						XITK_WIDGET_LIST_GC(test->widget_list), 20, 30, 
 						100, 20, 12, fontname)));
+  xitk_enable_and_show_widget(test->browser);
   
-  xitk_browser_update_list(test->browser, test->entries, test->num_entries, 0);
+  xitk_browser_update_list(test->browser, 
+			   (const char *const *)test->entries, test->num_entries, 0);
 
 }
 
+static void menu_test_end(xitk_widget_t *w, xitk_menu_entry_t *me, void *data) {
+  test_end(NULL, NULL);
+}
+static void create_menu(void) {
+  xitk_menu_widget_t   menu;
+  int                  x = 10, y = 10;
+  xitk_menu_entry_t    menu_entries[] = {
+    { "Popup menu:-------------------",  "<title>",       NULL,     NULL },
+    { "Load a file",                     NULL,            NULL,     NULL },
+    { "Reload",                          NULL,            NULL,     NULL },
+    { "Auto save",                       "<check>",       NULL,     NULL },
+    { "SEP",                             "<separator>",   NULL,     NULL },
+    { "Empty",                           "<branch>",      NULL,     NULL },
+    { "Playlist",                        "<branch>",      NULL,     NULL },
+    { "Playlist/Playlist Management",    "<title>",       NULL,     NULL },
+    { "Playlist/Load",                   NULL,            NULL,     NULL },
+    { "Playlist/Loop",                   "<branch>",      NULL,     NULL },
+    { "Playlist/Loop/no loop",           "<checked>",     NULL,     NULL },
+    { "Playlist/Loop/simple loop",       "<check>",       NULL,     NULL },
+    { "Playlist/Loop/SEP",               "<separator>",   NULL,     NULL },
+    { "Playlist/Loop/repeat entry",      "<check>",       NULL,     NULL },
+    { "Playlist/Loop/SEP",               "<separator>",   NULL,     NULL },
+    { "Playlist/Loop/shuffle",           "<check>",       NULL,     NULL },
+    { "Playlist/Loop/infinite shuffle",  "<check>",       NULL,     NULL },
+    { "Playlist/Loop/Extra",             "<branch>",      NULL,     NULL },
+    { "Playlist/Save",                   NULL,            NULL,     NULL },
+    { "Another branch",                  "<branch>",      NULL,     NULL },
+    { "Another branch/dummy",            NULL,            NULL,     NULL },
+    { "Another branch/vice\\/versa",     NULL,            NULL,     NULL },
+    { "SEP",                             "<separator>",   NULL,     NULL },
+    /* Testing deep sub-branches */
+    { "Deep",                            "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2",                     "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2/Deep 3",              "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4",       "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4/Deep 5",
+                                         "<branch>",      NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4/Deep 5/Entry number 1",
+                                         NULL,            NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4/Deep 5/SEP",
+                                         "<separator>",   NULL,     NULL },
+    { "Deep/Deep 2/Deep 3/Deep 4/Deep 5/Entry number 2",
+                                         NULL,            NULL,     NULL },
+    { "SEP",                             "<separator>",   NULL,     NULL },
+    { "Save a file",                     NULL,            NULL,     NULL },
+    { "SEP",                             "<separator>",   NULL,     NULL },
+    { "Video",                           "<branch>",      NULL,     NULL },
+    { "Video/premier",                   "<branch>",      NULL,     NULL },
+    { "Video/premier/Le Voili",          NULL,            NULL,     NULL },
+    { "Video/second",                    NULL,            NULL,     NULL },
+    { "Video/troisieme",                 NULL,            NULL,     NULL },
+    { "Video/quatrieme",                 NULL,            NULL,     NULL },
+    { "SEP",                             "<separator>",   NULL,     NULL },
+    { "Quit",                            NULL,            menu_test_end, NULL },
+    { NULL,                              NULL,            NULL,     NULL }
+  };
+  
+  XITK_WIDGET_INIT(&menu, test->imlibdata);
+
+  (void) xitk_get_mouse_coords(test->display, 
+			       (xitk_window_get_window(test->xwin)), NULL, NULL, &x, &y);
+  
+  menu.menu_tree         = &menu_entries[0];
+  menu.parent_wlist      = test->widget_list;
+  menu.skin_element_name = NULL;
+  
+  test->menu = xitk_noskin_menu_create(test->widget_list, &menu, x, y);
+
+  {
+    xitk_menu_entry_t menu_entry;
+
+    menu_entry.menu      = "Playlist/Loop/Extra/errrr?";
+    menu_entry.type      = NULL;
+    menu_entry.cb        = NULL;
+    menu_entry.user_data = NULL;
+    xitk_menu_add_entry(test->menu, &menu_entry);
+  }
+  
+
+  xitk_menu_show_menu(test->menu);
+}
+
 /*
  *
  */
@@ -790,25 +966,27 @@ int main(int argc, char **argv) {
   gc = XCreateGC(test->display, 
 		 (xitk_window_get_window(test->xwin)), None, None);
 
-  test->widget_list                = xitk_widget_list_new();
-  test->widget_list->l             = xitk_list_new ();
-  test->widget_list->win           = (xitk_window_get_window(test->xwin));
-  test->widget_list->gc            = gc;
-
+  test->widget_list  = xitk_widget_list_new();
+  xitk_widget_list_set(test->widget_list, WIDGET_LIST_LIST, (xitk_list_new()));
+  xitk_widget_list_set(test->widget_list, 
+		       WIDGET_LIST_WINDOW, (void *) (xitk_window_get_window(test->xwin)));
+  xitk_widget_list_set(test->widget_list, WIDGET_LIST_GC, gc);
+  
   XITK_WIDGET_INIT(&lb, test->imlibdata);
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = _("Quit");
-  lb.align             = LABEL_ALIGN_CENTER;
+  lb.align             = ALIGN_CENTER;
   lb.callback          = test_end;
   lb.state_callback    = NULL;
   lb.userdata          = NULL;
   lb.skin_element_name = NULL;
-  xitk_list_append_content(test->widget_list->l, 
+  xitk_list_append_content(XITK_WIDGET_LIST_LIST(test->widget_list),
 	   (w = xitk_noskin_labelbutton_create(test->widget_list, &lb,
 					       (windoww / 2) - 50, windowh - 50,
 					       100, 30,
 					       "Black", "Black", "White", fontname)));
+  xitk_enable_and_show_widget(w);
   xitk_set_widget_tips_default(w, "Do you really want to leave me ?");
 
   create_browser();
@@ -820,6 +998,7 @@ int main(int argc, char **argv) {
   create_frame();
   create_tabs();
   create_intbox();
+  create_doublebox();
   create_checkbox();
   
   test->kreg = xitk_register_event_handler("test", 
@@ -834,13 +1013,14 @@ int main(int argc, char **argv) {
 
   XMapRaised(test->display, xitk_window_get_window(test->xwin));
 
-  xitk_run();
+  xitk_run(NULL, NULL);
 
   deinit_test();
 
-  XLockDisplay (test->display);
-  XFreeGC(test->display, gc);
-  XUnlockDisplay (test->display);
+  XCloseDisplay(test->display);
+
+  free(test);
+  test = NULL;
 
   return 1;
 }
