@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: tips.c,v 1.23 2003/03/03 14:43:48 f1rmb Exp $
+ * $Id: tips.c,v 1.26 2003/12/13 01:09:40 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -55,8 +55,8 @@ static void _tips_kill_running(void) {
 
     pthread_cancel(disptips->thread);
     
-    xitk_window_destroy_window(disptips->w->imlibdata, disptips->xwin);
     xitk_unregister_event_handler(&disptips->key);
+    xitk_window_destroy_window(disptips->w->imlibdata, disptips->xwin);
     
     XLOCK(disptips->w->imlibdata->x.disp);
     XSync(disptips->w->imlibdata->x.disp, False);
@@ -105,8 +105,8 @@ static void *_tips_destroy_thread(void *data) {
   pthread_mutex_lock(&tp->mutex);
 
   /* Kill tips window */
-  xitk_window_destroy_window(tp->w->imlibdata, tp->xwin);
   xitk_unregister_event_handler(&tp->key);
+  xitk_window_destroy_window(tp->w->imlibdata, tp->xwin);
 
   /* We are flushing here, otherwise tips window will stay displayed */
   XLOCK(tp->w->imlibdata->x.disp);
@@ -334,6 +334,25 @@ void xitk_tips_set_tips(xitk_widget_t *w, char *str) {
   XITK_FREE(w->tips_string);
   w->tips_string = strdup(str);
 
+  /* Special GROUP widget case */
+  if(w->type & WIDGET_GROUP) {
+    if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_INTBOX) {
+      xitk_widget_t *widget = xitk_intbox_get_input_widget(w);
+
+      xitk_tips_set_tips(widget, str);
+    }
+    else if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_DOUBLEBOX) {
+      xitk_widget_t *widget = xitk_doublebox_get_input_widget(w);
+      
+      xitk_tips_set_tips(widget, str);
+    }
+    else if((w->type & WIDGET_GROUP_MASK) & WIDGET_GROUP_COMBO) {
+      xitk_widget_t *widget = xitk_combo_get_label_widget(w);
+      
+      xitk_tips_set_tips(widget, str);
+    }
+  }
+  
   /* No timeout, set it to default */
   if(!w->tips_timeout)
     xitk_tips_set_timeout(w, TIPS_TIMEOUT);
