@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: widget.c,v 1.53 2003/03/27 20:52:09 f1rmb Exp $
+ * $Id: widget.c,v 1.57 2003/12/13 01:09:40 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -1103,7 +1103,7 @@ int xitk_click_notify_widget_list (xitk_widget_list_t *wl, int x, int y, int but
     }
   }
   
-  if((wl->widget_focused == NULL) && menu)
+  if(!(wl->widget_focused) && menu)
     wl->widget_focused = menu;
 
   return((bRepaint == 1));
@@ -1232,7 +1232,10 @@ void xitk_set_focus_to_next_widget(xitk_widget_list_t *wl, int backward) {
     }
     
     /* next widget will be focused */
-    if(widget == wl->widget_focused) {
+    if((widget == wl->widget_focused) ||
+       (widget && wl->widget_focused && 
+	(((wl->widget_focused->type & WIDGET_GROUP_MASK) == WIDGET_GROUP_MENU)
+	 && (wl->widget_focused->type & WIDGET_GROUP_WIDGET)))) {
       xitk_widget_t *next_widget;
       
       if ((wl->widget_focused->type & WIDGET_FOCUSABLE) &&
@@ -1564,7 +1567,7 @@ void xitk_destroy_widget(xitk_widget_t *w) {
 }
 
 /*
- * Show widgets from widget list.
+ * Destroy widgets from widget list.
  */
 void xitk_destroy_widgets(xitk_widget_list_t *wl) {
   xitk_widget_t *mywidget;
@@ -1698,7 +1701,7 @@ void xitk_start_widget(xitk_widget_t *w) {
 }
 
 /*
- * Stop a widgets from widget list.
+ * Stop widgets from widget list.
  */
 void xitk_stop_widgets(xitk_widget_list_t *wl) {
   xitk_widget_t *mywidget;
@@ -1762,6 +1765,17 @@ void xitk_show_widgets(xitk_widget_list_t *wl) {
   }
 }
 
+void xitk_enable_and_show_widget(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  xitk_enable_widget(w);
+  xitk_show_widget(w);
+}
+
 /*
  * Hide a widget.
  */
@@ -1808,6 +1822,17 @@ void xitk_hide_widgets(xitk_widget_list_t *wl) {
   }
 }
 
+void xitk_disable_and_hide_widget(xitk_widget_t *w) {
+
+  if(!w) {
+    XITK_WARNING("widget is NULL\n");
+    return;
+  }
+
+  xitk_disable_widget(w);
+  xitk_hide_widget(w);
+}
+
 /*
  *
  */
