@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: window.c,v 1.21 2002/07/17 20:29:32 f1rmb Exp $
+ * $Id: window.c,v 1.39 2003/12/14 14:10:33 f1rmb Exp $
  *
  */
 #ifdef HAVE_CONFIG_H
@@ -29,42 +29,13 @@
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 
-#include "Imlib-light/Imlib.h"
-#include "labelbutton.h"
-#include "image.h"
-#include "font.h"
-#include "window.h"
-#include "widget.h"
-#include "widget_types.h"
-
 #include "_xitk.h"
 
 #define DIALOG_TYPE_UNKNOWN        0
 #define DIALOG_TYPE_OK             1
 #define DIALOG_TYPE_YESNO          2
 #define DIALOG_TYPE_YESNOCANCEL    3
-
-typedef struct {
-  ImlibData              *imlibdata;
-  xitk_window_t          *xwin;
-  xitk_register_key_t     key;
-  xitk_widget_list_t     *widget_list;
-
-  int                     type;
-
-  xitk_widget_t          *wyes;
-  xitk_widget_t          *wno;
-  xitk_widget_t          *wcancel;
-
-  xitk_widget_t          *default_button;
-
-  xitk_state_callback_t  yescallback;
-  xitk_state_callback_t  nocallback;
-  xitk_state_callback_t  cancelcallback;
-
-  void                   *userdata;
-
-} xitk_dialog_t;
+#define DIALOG_TYPE_BUTTONLESS     4
 
 static void _xitk_window_destroy_window(xitk_widget_t *, void *);
 
@@ -102,6 +73,7 @@ int xitk_is_window_size(Display *display, Window window, int width, int height)
   XLOCK(display);
   if(!XGetWindowAttributes(display, window, &wattr)) {
     XITK_WARNING("XGetWindowAttributes() failed.n");
+    XUNLOCK(display);
     return -1;
   }
   XUNLOCK(display);
@@ -232,7 +204,7 @@ xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width,
   XSizeHints             hint;
   XWMHints              *wm_hint;
   XSetWindowAttributes   attr;
-  Atom                   prop, XA_WIN_LAYER;
+  Atom                   prop, XA_WIN_LAYER, XA_DELETE_WINDOW;
   XColor                 black, dummy;
   MWMHints               mwmhints;
   XClassHint            *xclasshint;
@@ -242,12 +214,11 @@ xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width,
     return NULL;
 
   xwin             = (xitk_window_t *) xitk_xmalloc(sizeof(xitk_window_t));
-  xwin->background = None;
+  xwin->background = NULL;
   xwin->width      = width;
   xwin->height     = height;
+  xwin->parent     = NULL;
   
-  XLOCK(im->x.disp);
-
   memset(&hint, 0, sizeof(hint));
   hint.x               = x;
   hint.y               = y;
@@ -260,7 +231,9 @@ xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width,
   hint.win_gravity     = NorthWestGravity;
   hint.flags           = PWinGravity | PBaseSize | PMinSize | USSize | USPosition;
   
+  XLOCK(im->x.disp);
   XAllocNamedColor(im->x.disp, Imlib_get_colormap(im), "black", &black, &dummy);
+  XUNLOCK(im->x.disp);
 
   attr.override_redirect = False;
   attr.background_pixel  = black.pixel;
@@ -268,6 +241,7 @@ xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width,
   attr.colormap          = Imlib_get_colormap(im);
   attr.win_gravity       = NorthWestGravity;
 
+  XLOCK(im->x.disp);
   xwin->window = XCreateWindow(im->x.disp, im->x.root, hint.x, hint.y, hint.width, hint.height,
 			       0, im->x.depth,  InputOutput, im->x.visual,
 			       CWBackPixel | CWBorderPixel | CWColormap
@@ -293,6 +267,9 @@ xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width,
                   PropModeReplace, (unsigned char *) &mwmhints,
                   PROP_MWM_HINTS_ELEMENTS);
   
+  XA_DELETE_WINDOW = XInternAtom(im->x.disp, "WM_DELETE_WINDOW", False);
+  XSetWMProtocols(im->x.disp, xwin->window, &XA_DELETE_WINDOW, 1);
+
   if((xclasshint = XAllocClassHint()) != NULL) {
     xclasshint->res_name = "Xine Window";
     xclasshint->res_class = "Xitk";
@@ -318,8 +295,6 @@ xitk_window_t *xitk_window_create_window(ImlibData *im, int x, int y, int width,
  * Create a simple painted window.
  */
 xitk_window_t *xitk_window_create_simple_window(ImlibData *im, int x, int y, int width, int height) {
-  GC             gc;
-  XGCValues      gcv;
   xitk_window_t *xwin;
   
   if((im == NULL) || (width == 0 || height == 0))
@@ -329,19 +304,10 @@ xitk_window_t *xitk_window_create_simple_window(ImlibData *im, int x, int y, int
   xwin->width = width;
   xwin->height = height;
   
-  XLOCK(im->x.disp);
-  gcv.graphics_exposures = False;
-  gc = XCreateGC(im->x.disp, xwin->window, GCGraphicsExposures, &gcv);
-  XUNLOCK(im->x.disp);
-  
-  xwin->background = xitk_image_create_pixmap(im, width, height);
+  xwin->background = xitk_image_create_xitk_pixmap(im, width, height);
   draw_outter(im, xwin->background, width, height);
   xitk_window_apply_background(im, xwin);
 
-  XLOCK(im->x.disp);
-  XFreeGC(im->x.disp, gc);
-  XUNLOCK(im->x.disp);
-
   xitk_window_move_window(im, xwin, x, y);
 
   return xwin;
@@ -352,13 +318,12 @@ xitk_window_t *xitk_window_create_simple_window(ImlibData *im, int x, int y, int
  */
 xitk_window_t *xitk_window_create_dialog_window(ImlibData *im, char *title, 
 						int x, int y, int width, int height) {
-  GC             gc;
-  XGCValues      gcv;
   xitk_window_t *xwin;
-  Pixmap         bar, pix_bg;
-  unsigned int   colorblack, colorwhite, colorgray;
+  xitk_pixmap_t  *bar, *pix_bg;
+  unsigned int   colorblack, colorwhite, colorgray, colordgray;
   xitk_font_t   *fs = NULL;
   int            lbear, rbear, wid, asc, des;
+  int            bar_style = xitk_get_barstyle_feature();
   
   if((im == NULL) || (title == NULL) || (width == 0 || height == 0))
     return NULL;
@@ -367,65 +332,113 @@ xitk_window_t *xitk_window_create_dialog_window(ImlibData *im, char *title,
 
   xitk_window_set_window_title(im, xwin, title);
 
-  bar = xitk_image_create_pixmap(im, width, TITLE_BAR_HEIGHT);
-  pix_bg = xitk_image_create_pixmap(im, width, height);
+  bar = xitk_image_create_xitk_pixmap(im, width, TITLE_BAR_HEIGHT);
+  pix_bg = xitk_image_create_xitk_pixmap(im, width, height);
+
+  fs = xitk_font_load_font(im->x.disp, DEFAULT_BOLD_FONT_12);
+  xitk_font_set_font(fs, bar->gc);
+  xitk_font_string_extent(fs, (title && strlen(title)) ? title : "Window", &lbear, &rbear, &wid, &asc, &des);
 
   XLOCK(im->x.disp);
-
-  gcv.graphics_exposures = False;
-  gc = XCreateGC(im->x.disp, xwin->window, GCGraphicsExposures, &gcv);
-
-  XCopyArea(im->x.disp, xwin->background, pix_bg, gc, 0, 0, width, height, 0, 0);
+  XCopyArea(im->x.disp, xwin->background->pixmap, pix_bg->pixmap, xwin->background->gc,
+	    0, 0, width, height, 0, 0);
+  XUNLOCK(im->x.disp);
 
   colorblack = xitk_get_pixel_color_black(im);
   colorwhite = xitk_get_pixel_color_white(im);
   colorgray = xitk_get_pixel_color_gray(im);
+  colordgray = xitk_get_pixel_color_darkgray(im);
 
-  { /* Draw window title bar background */
+ /* Draw window title bar background */
+  if(bar_style) {
     int s, bl = 255;
     unsigned int colorblue;
 
     colorblue = xitk_get_pixel_color_from_rgb(im, 0, 0, bl);
+    XLOCK(im->x.disp);
     for(s = 0; s <= TITLE_BAR_HEIGHT; s++, bl -= 8) {
-      XSetForeground(im->x.disp, gc, colorblue);
-      XDrawLine(im->x.disp, bar, gc, 0, s, width, s);
+      XSetForeground(im->x.disp, bar->gc, colorblue);
+      XDrawLine(im->x.disp, bar->pixmap, bar->gc, 0, s, width, s);
       colorblue = xitk_get_pixel_color_from_rgb(im, 0, 0, bl);
     }
+    XUNLOCK(im->x.disp);
   }
+  else {
+    int s;
+    unsigned int c, cd;
 
-  XSetForeground(im->x.disp, gc, colorwhite);
-  XDrawLine(im->x.disp, bar, gc, 0, 0, width, 0);
-  XDrawLine(im->x.disp, bar, gc, 0, 0, 0, TITLE_BAR_HEIGHT - 1);
-  XSetForeground(im->x.disp, gc, colorgray);
-  XDrawLine(im->x.disp, bar, gc, width - 1, 0, width - 1, TITLE_BAR_HEIGHT - 1);
-  XDrawLine(im->x.disp, bar, gc, 0, TITLE_BAR_HEIGHT - 1, width - 1, TITLE_BAR_HEIGHT - 1);
+    cd = xitk_get_pixel_color_from_rgb(im, 115, 12, 206);
+    c = xitk_get_pixel_color_from_rgb(im, 135, 97, 168);
+
+    draw_flat_with_color(im, bar, width, TITLE_BAR_HEIGHT, colorgray);
+    draw_rectangular_inner_box(im, bar, 2, 2, width - 6, (TITLE_BAR_HEIGHT - 1 - 4));
+    
+    XLOCK(im->x.disp);
+    for(s = 6; s <= (TITLE_BAR_HEIGHT - 6); s += 3) {
+      XSetForeground(im->x.disp, bar->gc, c);
+      XDrawLine(im->x.disp, bar->pixmap, bar->gc, 5, s, (width - 8), s);
+      XSetForeground(im->x.disp, bar->gc, cd);
+      XDrawLine(im->x.disp, bar->pixmap, bar->gc, 5, s+1, (width - 8), s+1);
+    }
+    
+    XSetForeground(im->x.disp, bar->gc, colorgray);
+    XFillRectangle(im->x.disp, bar->pixmap, bar->gc, 
+		   ((width - wid) - TITLE_BAR_HEIGHT) - 10, 6, 
+		   wid + 20, TITLE_BAR_HEIGHT - 1 - 8);
+    XUNLOCK(im->x.disp);
+  }
+  
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, bar->gc, colorwhite);
+  XDrawLine(im->x.disp, bar->pixmap, bar->gc, 0, 0, width, 0);
+  XDrawLine(im->x.disp, bar->pixmap, bar->gc, 0, 0, 0, TITLE_BAR_HEIGHT - 1);
   XUNLOCK(im->x.disp);
 
-  fs = xitk_font_load_font(im->x.disp, DEFAULT_BOLD_FONT_12);
-  xitk_font_set_font(fs, gc);
-  xitk_font_string_extent(fs, title, &lbear, &rbear, &wid, &asc, &des);
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, bar->gc, colorblack);
+  XDrawLine(im->x.disp, bar->pixmap, bar->gc, width - 1, 0, width - 1, TITLE_BAR_HEIGHT - 1);
+
+  XDrawLine(im->x.disp, bar->pixmap, bar->gc, 2, TITLE_BAR_HEIGHT - 1, width - 2, TITLE_BAR_HEIGHT - 1);
+  XUNLOCK(im->x.disp);
 
   XLOCK(im->x.disp);
-  XSetForeground(im->x.disp, gc, colorwhite);
-  XDrawString(im->x.disp, bar, gc, 
+  XSetForeground(im->x.disp, bar->gc, colordgray);
+  XDrawLine(im->x.disp, bar->pixmap, bar->gc, width - 2, 2, width - 2, TITLE_BAR_HEIGHT - 1);
+  XUNLOCK(im->x.disp);
+
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, bar->gc, colorblack);
+  XDrawLine(im->x.disp, pix_bg->pixmap, bar->gc, width - 1, 0, width - 1, height - 1);
+  XDrawLine(im->x.disp, pix_bg->pixmap, bar->gc, 0, height - 1, width - 1, height - 1);
+  XUNLOCK(im->x.disp);
+
+  XLOCK(im->x.disp);
+  XSetForeground(im->x.disp, bar->gc, colordgray);
+  XDrawLine(im->x.disp, pix_bg->pixmap, bar->gc, width - 2, 0, width - 2, height - 2);
+  XDrawLine(im->x.disp, pix_bg->pixmap, bar->gc, 2, height - 2, width - 2, height - 2);
+  XUNLOCK(im->x.disp);
+
+  XLOCK(im->x.disp);
+  if(bar_style)
+    XSetForeground(im->x.disp, bar->gc, colorwhite);
+  else
+    XSetForeground(im->x.disp, bar->gc, (xitk_get_pixel_color_from_rgb(im, 85, 12, 135)));
+  
+  xitk_font_draw_string(fs, bar->pixmap, bar->gc, 
 	      (width - wid) - TITLE_BAR_HEIGHT, ((TITLE_BAR_HEIGHT+asc+des) >> 1) - des, 
 	      title, strlen(title));
-
   XUNLOCK(im->x.disp);
 
   xitk_font_unload_font(fs);
 
   XLOCK(im->x.disp);
-  XCopyArea(im->x.disp, bar, pix_bg, gc, 0, 0, width, TITLE_BAR_HEIGHT, 0, 0);
+  XCopyArea(im->x.disp, bar->pixmap, pix_bg->pixmap, bar->gc, 0, 0, width, TITLE_BAR_HEIGHT, 0, 0);
   XUNLOCK(im->x.disp);
   
-  xitk_window_change_background(im, xwin, pix_bg, width, height);
+  xitk_window_change_background(im, xwin, pix_bg->pixmap, width, height);
   
-  XUNLOCK(im->x.disp);
-  XFreePixmap(im->x.disp, bar);
-  XFreePixmap(im->x.disp, pix_bg);
-  XFreeGC(im->x.disp, gc);
-  XUNLOCK(im->x.disp);
+  xitk_image_destroy_xitk_pixmap(bar);
+  xitk_image_destroy_xitk_pixmap(pix_bg);
 
   return xwin;
 }
@@ -461,7 +474,7 @@ Pixmap xitk_window_get_background(xitk_window_t *w) {
   if(w == NULL)
     return None;
 
-  return w->background;
+  return w->background->pixmap;
 }
 
 /*
@@ -473,7 +486,7 @@ void xitk_window_apply_background(ImlibData *im, xitk_window_t *w) {
     return;
 
   XLOCK(im->x.disp);
-  XSetWindowBackgroundPixmap(im->x.disp, w->window, w->background);
+  XSetWindowBackgroundPixmap(im->x.disp, w->window, w->background->pixmap);
   XClearWindow(im->x.disp, w->window);
   XUNLOCK(im->x.disp);
 }
@@ -483,8 +496,6 @@ void xitk_window_apply_background(ImlibData *im, xitk_window_t *w) {
  */
 int xitk_window_change_background(ImlibData *im,
 				  xitk_window_t *w, Pixmap bg, int width, int height) {
-  GC            gc;
-  XGCValues     gcv;
   Window        rootwin;
   int           xwin, ywin;
   unsigned int  wwin, hwin, bwin, dwin;
@@ -492,11 +503,8 @@ int xitk_window_change_background(ImlibData *im,
   if((im == NULL) || (w == NULL) || (bg == None) || (width == 0 || height == 0))
     return 0;
 
-  XLOCK(im->x.disp);
-  XFreePixmap(im->x.disp, w->background);
-  XUNLOCK(im->x.disp);
-
-  w->background = xitk_image_create_pixmap(im, width, height);
+  xitk_image_destroy_xitk_pixmap(w->background);
+  w->background = xitk_image_create_xitk_pixmap(im, width, height);
   
   XLOCK(im->x.disp);
   if(XGetGeometry(im->x.disp, w->window, &rootwin, 
@@ -507,17 +515,11 @@ int xitk_window_change_background(ImlibData *im,
   else
     return 0;
 
-  gcv.graphics_exposures = False;
-  gc = XCreateGC(im->x.disp, bg, GCGraphicsExposures, &gcv);
-  XCopyArea(im->x.disp, bg, w->background, gc, 0, 0, width, height, 0, 0);
+  XCopyArea(im->x.disp, bg, w->background->pixmap, w->background->gc, 0, 0, width, height, 0, 0);
   XUNLOCK(im->x.disp);
 
   xitk_window_apply_background(im, w);
 
-  XLOCK(im->x.disp);
-  XFreeGC(im->x.disp, gc);
-  XUNLOCK(im->x.disp);
-  
   return 1;
 }
 
@@ -530,16 +532,22 @@ static void _window_handle_event(XEvent *event, void *data) {
   switch(event->type) {
 
   case Expose:
-    wd->widget_list->widget_focused = wd->default_button;
-    if (wd->widget_list->widget_focused->notify_focus 
-	&& wd->widget_list->widget_focused->enable == WIDGET_ENABLE) {
-      xitk_widget_t *w = wd->widget_list->widget_focused;
+    if(wd->widget_list) {
+      wd->widget_list->widget_focused = wd->default_button;
+      if ((wd->widget_list->widget_focused->type & WIDGET_FOCUSABLE) && 
+	  wd->widget_list->widget_focused->enable == WIDGET_ENABLE) {
+	xitk_widget_t *w = wd->widget_list->widget_focused;
+	widget_event_t event;
 
-      (void) (w->notify_focus) (wd->widget_list, w, FOCUS_RECEIVED);
-      w->have_focus = FOCUS_RECEIVED;
-
-      if(w->paint)
-	(w->paint) (w, wd->widget_list->win, wd->widget_list->gc);
+	event.type = WIDGET_EVENT_FOCUS;
+	event.focus = FOCUS_RECEIVED;
+	
+	(void) w->event(w, &event, NULL);
+	w->have_focus = FOCUS_RECEIVED;
+	
+	event.type = WIDGET_EVENT_PAINT;
+	(void) w->event(w, &event, NULL);
+      }
     }
     break;
     
@@ -564,14 +572,19 @@ static void _window_handle_event(XEvent *event, void *data) {
     switch (mykey) {
 
     case XK_Return:
-      _xitk_window_destroy_window(wd->default_button, (void *)wd);
-      break;
-
-    case XK_Escape:
-      if((wd->type == DIALOG_TYPE_YESNO) || (wd->type == DIALOG_TYPE_YESNOCANCEL))
+      if(wd->default_button)
 	_xitk_window_destroy_window(wd->default_button, (void *)wd);
-      else
-	_xitk_window_destroy_window(NULL, (void *)wd);
+      break;
+      
+    case XK_Escape:
+      if((wd->type == DIALOG_TYPE_YESNO) || (wd->type == DIALOG_TYPE_YESNOCANCEL)) {
+	if(wd->default_button)
+	  _xitk_window_destroy_window(wd->default_button, (void *)wd);
+      }
+      else {
+	if(wd->type != DIALOG_TYPE_BUTTONLESS)
+	  _xitk_window_destroy_window(NULL, (void *)wd);
+      }
       break;
 
     }
@@ -581,20 +594,55 @@ static void _window_handle_event(XEvent *event, void *data) {
   }
 }
 
+void xitk_window_set_modal(xitk_window_t *w) {
+  xitk_modal_window(w->window);  
+}
+void xitk_window_dialog_set_modal(xitk_window_t *w) {
+  xitk_dialog_t *wd = w->parent;
+  xitk_window_set_modal(wd->xwin);
+}
 void xitk_window_destroy_window(ImlibData *im, xitk_window_t *w) {
 
   XLOCK(im->x.disp);
   XUnmapWindow(im->x.disp, w->window);
-    if(w->background != None)
-    XFreePixmap(im->x.disp, w->background);
+  XUNLOCK(im->x.disp);
+
+  if(w->background)
+    xitk_image_destroy_xitk_pixmap(w->background);
+
   w->width = -1;
   w->height = -1;
+
+  xitk_unmodal_window(w->window);
+
+  XLOCK(im->x.disp);
   XDestroyWindow(im->x.disp, w->window);
   XUNLOCK(im->x.disp);
 
   XITK_FREE(w);
 }
 
+void xitk_window_dialog_destroy(xitk_window_t *w) {
+  xitk_dialog_t *wd = w->parent;
+
+  if(wd) {
+    xitk_window_destroy_window(wd->imlibdata, wd->xwin);
+    xitk_unregister_event_handler(&wd->key);
+    
+    if(wd->widget_list) {
+      xitk_destroy_widgets(wd->widget_list);
+      XLOCK(wd->imlibdata->x.disp);
+      XFreeGC(wd->imlibdata->x.disp, wd->widget_list->gc);
+      XUNLOCK(wd->imlibdata->x.disp);
+      
+      xitk_list_free(wd->widget_list->l);
+      
+      XITK_FREE(wd->widget_list);
+    }
+    XITK_FREE(wd);
+  }  
+}
+
 /*
  * Unmap and free window components.
  */
@@ -602,7 +650,6 @@ static void _xitk_window_destroy_window(xitk_widget_t *w, void *data) {
   xitk_dialog_t *wd = (xitk_dialog_t *)data;
 
   xitk_window_destroy_window(wd->imlibdata, wd->xwin);
-  xitk_unregister_event_handler(&wd->key);
 
   switch(wd->type) {
 
@@ -627,39 +674,42 @@ static void _xitk_window_destroy_window(xitk_widget_t *w, void *data) {
     }
     break;
     
+  case DIALOG_TYPE_BUTTONLESS:
+    /* NOOP */
+    break;
+
   default:
     XITK_WARNING("window dialog type unknown: %d\n", wd->type);
     break;
 
   }
 
-  xitk_destroy_widgets(wd->widget_list);
-
-  XLOCK(wd->imlibdata->x.disp);
-  XFreeGC(wd->imlibdata->x.disp, wd->widget_list->gc);
-  XUNLOCK(wd->imlibdata->x.disp);
+  xitk_unregister_event_handler(&wd->key);
   
-  xitk_list_free(wd->widget_list->l);
+  if(wd->widget_list) {
+    xitk_destroy_widgets(wd->widget_list);
+    
+    XLOCK(wd->imlibdata->x.disp);
+    XFreeGC(wd->imlibdata->x.disp, wd->widget_list->gc);
+    XUNLOCK(wd->imlibdata->x.disp);
+    
+    xitk_list_free(wd->widget_list->l);
 
-  XITK_FREE(wd->widget_list);
+    free(wd->widget_list);
+  }
+  
   XITK_FREE(wd);
   
 }
 
-/*
- * Create a window error, containing an error message.
- */
-void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *button_label,
-					      xitk_state_callback_t cb, void *userdata, 
-					      int window_width, int align, char *message, ...) {
+xitk_window_t *xitk_window_dialog_button_free_with_width(ImlibData *im, char *title,
+							 int window_width, int align, char *message, ...) {
   xitk_dialog_t              *wd;
-  xitk_labelbutton_widget_t   lb;
   int                         windoww = window_width, windowh;
   xitk_image_t               *i;
-  int                         bwidth = 100, bx, by;
 
   if((im == NULL) || (window_width == 0) || (message == NULL))
-    return;
+    return NULL;
 
   wd = (xitk_dialog_t *) xitk_xmalloc(sizeof(xitk_dialog_t));
 
@@ -669,7 +719,7 @@ void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *
     int       n, size = 100;
     
     if((buf = xitk_xmalloc(size)) == NULL) 
-      return;
+      return NULL;
     
     while(1) {
 
@@ -686,7 +736,111 @@ void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *
 	size *= 2;
 
       if((buf = realloc(buf, size)) == NULL)
-	return;
+	return NULL;
+    }
+    
+    i = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
+    XITK_FREE(buf);
+  }
+  
+  windowh = (i->height) + (TITLE_BAR_HEIGHT + 40);
+
+  wd->imlibdata = im;
+  wd->type = DIALOG_TYPE_BUTTONLESS;
+  wd->wyes = wd->wno = wd->wcancel = wd->default_button = NULL;
+  wd->widget_list = NULL;
+  wd->xwin = xitk_window_create_dialog_window(im, ((title != NULL) ? title : "Notice"),
+					      0, 0, windoww, windowh);
+  wd->xwin->parent = wd;
+  
+  xitk_window_center_window(im, wd->xwin);
+  
+  wd->widget_list = NULL;
+
+  /* Draw text area */
+  {
+    xitk_pixmap_t  *bg;
+    int             width, height;
+    GC              gc;
+    
+    xitk_window_get_window_size(wd->xwin, &width, &height);
+    bg = xitk_image_create_xitk_pixmap(im, width, height);
+    
+    XLOCK(im->x.disp);
+    gc = XCreateGC(im->x.disp, (xitk_window_get_background(wd->xwin)), None, None);
+    XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg->pixmap,
+	      gc, 0, 0, width, height, 0, 0);
+    XCopyArea(im->x.disp, i->image->pixmap, bg->pixmap,
+	      i->image->gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
+    XUNLOCK(im->x.disp);
+
+    xitk_window_change_background(im, wd->xwin, bg->pixmap, width, height);
+
+    xitk_image_destroy_xitk_pixmap(bg);
+
+    XLOCK(im->x.disp);
+    XFreeGC(im->x.disp, gc);
+    XUNLOCK(im->x.disp);
+
+    xitk_image_free_image(im, &i);
+
+  }
+
+  XLOCK(im->x.disp);
+  XMapRaised(im->x.disp, (xitk_window_get_window(wd->xwin)));
+  XUNLOCK(im->x.disp);
+
+  wd->key = xitk_register_event_handler("xitk_nobtn", 
+					(xitk_window_get_window(wd->xwin)),
+					_window_handle_event,
+					NULL,
+					NULL,
+					NULL,
+					(void *)wd);
+  
+  return wd->xwin;
+}
+/*
+ * Create a window error, containing an error message.
+ */
+xitk_window_t *xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *button_label,
+							xitk_state_callback_t cb, void *userdata, 
+							int window_width, int align, char *message, ...) {
+  xitk_dialog_t              *wd;
+  xitk_labelbutton_widget_t   lb;
+  int                         windoww = window_width, windowh;
+  xitk_image_t               *i;
+  int                         bwidth = 100, bx, by;
+
+  if((im == NULL) || (window_width == 0) || (message == NULL))
+    return NULL;
+
+  wd = (xitk_dialog_t *) xitk_xmalloc(sizeof(xitk_dialog_t));
+
+  {
+    va_list   args;
+    char     *buf;
+    int       n, size = 100;
+    
+    if((buf = xitk_xmalloc(size)) == NULL) 
+      return NULL;
+    
+    while(1) {
+
+      va_start(args, message);
+      n = vsnprintf(buf, size, message, args);
+      va_end(args);
+      
+      if(n > -1 && n < size)
+	break;
+      
+      if(n > -1)
+	size = n + 1;
+      else
+	size *= 2;
+
+      if((buf = realloc(buf, size)) == NULL)
+	return NULL;
     }
     
     i = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
@@ -700,7 +854,9 @@ void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *
   wd->yescallback = cb;
   wd->userdata = userdata;
   wd->xwin = xitk_window_create_dialog_window(im, ((title != NULL) ? title : "Notice"),
-					      0, 0, windoww, windowh);  
+					      0, 0, windoww, windowh);
+  wd->xwin->parent = wd;
+
   xitk_window_center_window(im, wd->xwin);
   
   wd->widget_list                = xitk_widget_list_new();
@@ -722,7 +878,7 @@ void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *
   XITK_WIDGET_INIT(&lb, im);
   lb.button_type       = CLICK_BUTTON;
   lb.label             = button_label;
-  lb.align             = LABEL_ALIGN_CENTER;
+  lb.align             = ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
   lb.userdata          = (void*)wd;
@@ -737,25 +893,26 @@ void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *
 
   /* Draw text area */
   {
-    Pixmap   bg;
-    int      width, height;
-    GC       gc;
+    xitk_pixmap_t  *bg;
+    int             width, height;
+    GC              gc;
     
     xitk_window_get_window_size(wd->xwin, &width, &height);
-    bg = xitk_image_create_pixmap(im, width, height);
+    bg = xitk_image_create_xitk_pixmap(im, width, height);
     
     XLOCK(im->x.disp);
-    gc = XCreateGC(im->x.disp, im->x.base_window, None, None);
-    XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg,
+    gc = XCreateGC(im->x.disp, (xitk_window_get_background(wd->xwin)), None, None);
+    XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg->pixmap,
 	      gc, 0, 0, width, height, 0, 0);
-    XCopyArea(im->x.disp, i->image, bg,
-	      gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
+    XCopyArea(im->x.disp, i->image->pixmap, bg->pixmap,
+	      i->image->gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
     XUNLOCK(im->x.disp);
 
-    xitk_window_change_background(im, wd->xwin, bg, width, height);
+    xitk_window_change_background(im, wd->xwin, bg->pixmap, width, height);
+
+    xitk_image_destroy_xitk_pixmap(bg);
 
     XLOCK(im->x.disp);
-    XFreePixmap(im->x.disp, bg);
     XFreeGC(im->x.disp, gc);
     XUNLOCK(im->x.disp);
 
@@ -775,18 +932,22 @@ void xitk_window_dialog_one_button_with_width(ImlibData *im, char *title, char *
 					wd->widget_list,
 					(void *)wd);
 
+  xitk_enable_and_show_widget(wd->wyes);
+
+  return wd->xwin;
 }
 
-void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
-				      xitk_state_callback_t cb, void *userdata, 
-				      int window_width, int align, char *message, ...) {
-
-  va_list   args;
-  char     *buf;
-  int       n, size = 100;
+xitk_window_t *xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
+						xitk_state_callback_t cb, void *userdata, 
+						int window_width, int align, char *message, ...) {
+  
+  va_list        args;
+  char          *buf;
+  int            n, size = 100;
+  xitk_window_t *xw = NULL;
   
   if((buf = xitk_xmalloc(size)) == NULL) 
-    return;
+    return NULL;
   
   while(1) {
 
@@ -803,27 +964,28 @@ void xitk_window_dialog_ok_with_width(ImlibData *im, char *title,
       size *= 2;
     
     if((buf = realloc(buf, size)) == NULL)
-      return;
+      return NULL;
   }
   
   {
     char buf2[(strlen(buf) * 2) + 1];
     xitk_subst_special_chars(buf, buf2);
-    xitk_window_dialog_one_button_with_width(im, title, _("OK"), cb, userdata, window_width,
-					     align, buf2);
+    xw = xitk_window_dialog_one_button_with_width(im, title, _("OK"), cb, userdata, window_width,
+						  align, buf2);
   }
   XITK_FREE(buf);
+  return xw;
 }
 
 /*
  * Create an interactive window, containing 'yes', 'no', 'cancel' buttons.
  */
-void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
-					       char *button1_label, char *button2_label,
-					       xitk_state_callback_t cb1, 
-					       xitk_state_callback_t cb2, 
-					       void *userdata, 
-					       int window_width, int align, char *message, ...) {
+xitk_window_t *xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
+							 char *button1_label, char *button2_label,
+							 xitk_state_callback_t cb1, 
+							 xitk_state_callback_t cb2, 
+							 void *userdata, 
+							 int window_width, int align, char *message, ...) {
   xitk_dialog_t              *wd;
   xitk_labelbutton_widget_t   lb;
   int                         windoww = window_width, windowh;
@@ -831,7 +993,7 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
   int                         bwidth = 100, bx1, bx2, by;
 
   if((im == NULL) || (window_width == 0) || (message == NULL))
-    return;
+    return NULL;
 
   wd = (xitk_dialog_t *) xitk_xmalloc(sizeof(xitk_dialog_t));
 
@@ -841,7 +1003,7 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
     int       n, size = 100;
     
     if((buf = xitk_xmalloc(size)) == NULL) 
-      return;
+      return NULL;
     
     while(1) {
       
@@ -858,7 +1020,7 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
 	size *= 2;
 
       if((buf = realloc(buf, size)) == NULL)
-	return;
+	return NULL;
     }
     
     i = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
@@ -874,6 +1036,7 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
   wd->userdata = userdata;
   wd->xwin = xitk_window_create_dialog_window(im, ((title != NULL) ? title : _("Question?")), 
 					      0, 0, windoww, windowh);  
+  wd->xwin->parent = wd;
   xitk_window_center_window(im, wd->xwin);
   
   wd->widget_list                = xitk_widget_list_new();
@@ -895,7 +1058,7 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
   XITK_WIDGET_INIT(&lb, im);
   lb.button_type       = CLICK_BUTTON;
   lb.label             = button1_label;
-  lb.align             = LABEL_ALIGN_CENTER;
+  lb.align             = ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
   lb.userdata          = (void*)wd;
@@ -909,7 +1072,7 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = button2_label;
-  lb.align             = LABEL_ALIGN_CENTER;
+  lb.align             = ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
   lb.userdata          = (void*)wd;
@@ -925,25 +1088,26 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
 
   /* Draw text area */
   {
-    Pixmap   bg;
-    int      width, height;
-    GC       gc;
+    xitk_pixmap_t *bg;
+    int            width, height;
+    GC             gc;
     
     xitk_window_get_window_size(wd->xwin, &width, &height);
-    bg = xitk_image_create_pixmap(im, width, height);
+    bg = xitk_image_create_xitk_pixmap(im, width, height);
     
     XLOCK(im->x.disp);
-    gc = XCreateGC(im->x.disp, im->x.base_window, None, None);
-    XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg,
+    gc = XCreateGC(im->x.disp, (xitk_window_get_background(wd->xwin)), None, None);
+    XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg->pixmap,
 	      gc, 0, 0, width, height, 0, 0);
-    XCopyArea(im->x.disp, i->image, bg,
-	      gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
+    XCopyArea(im->x.disp, i->image->pixmap, bg->pixmap,
+	      i->image->gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
     XUNLOCK(im->x.disp);
 
-    xitk_window_change_background(im, wd->xwin, bg, width, height);
+    xitk_window_change_background(im, wd->xwin, bg->pixmap, width, height);
+
+    xitk_image_destroy_xitk_pixmap(bg);
 
     XLOCK(im->x.disp);
-    XFreePixmap(im->x.disp, bg);
     XFreeGC(im->x.disp, gc);
     XUNLOCK(im->x.disp);
 
@@ -962,22 +1126,27 @@ void xitk_window_dialog_two_buttons_with_width(ImlibData *im, char *title,
 					NULL,
 					wd->widget_list,
 					(void *)wd);
+  xitk_enable_and_show_widget(wd->wyes);
+  xitk_enable_and_show_widget(wd->wno);
+
+  return wd->xwin;
 }
 
 /*
  *
  */
-void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
-					 xitk_state_callback_t ycb, 
-					 xitk_state_callback_t ncb, 
-					 void *userdata, 
-					 int window_width, int align, char *message, ...) {
-  va_list   args;
-  char     *buf;
-  int       n, size = 100;
+xitk_window_t *xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
+						   xitk_state_callback_t ycb, 
+						   xitk_state_callback_t ncb, 
+						   void *userdata, 
+						   int window_width, int align, char *message, ...) {
+  va_list        args;
+  char          *buf;
+  int            n, size = 100;
+  xitk_window_t *xw = NULL;
   
   if((buf = xitk_xmalloc(size)) == NULL) 
-    return;
+    return NULL;
   
   while(1) {
     
@@ -994,32 +1163,33 @@ void xitk_window_dialog_yesno_with_width(ImlibData *im, char *title,
       size *= 2;
     
     if((buf = realloc(buf, size)) == NULL)
-      return;
+      return NULL;
   }
   
   {
     char buf2[(strlen(buf) * 2) + 1];
     
     xitk_subst_special_chars(buf, buf2);
-    xitk_window_dialog_two_buttons_with_width(im, title, _("Yes"), _("No"), 
-					      ycb, ncb, userdata, window_width, align, buf2);
+    xw = xitk_window_dialog_two_buttons_with_width(im, title, _("Yes"), _("No"), 
+						   ycb, ncb, userdata, window_width, align, buf2);
   }
 
   XITK_FREE(buf);
+  return xw;
 }
 
 /*
  * Create an interactive window, containing 'yes', 'no', 'cancel' buttons.
  */
-void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
-						 char *button1_label,
-						 char *button2_label,
-						 char *button3_label,
-						 xitk_state_callback_t cb1, 
-						 xitk_state_callback_t cb2, 
-						 xitk_state_callback_t cb3, 
-						 void *userdata, 
-						 int window_width, int align, char *message, ...) {
+xitk_window_t *xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
+							   char *button1_label,
+							   char *button2_label,
+							   char *button3_label,
+							   xitk_state_callback_t cb1, 
+							   xitk_state_callback_t cb2, 
+							   xitk_state_callback_t cb3, 
+							   void *userdata, 
+							   int window_width, int align, char *message, ...) {
   xitk_dialog_t              *wd;
   xitk_labelbutton_widget_t   lb;
   int                         windoww = window_width, windowh;
@@ -1027,7 +1197,7 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
   int                         bwidth = 100, bx1, bx2, bx3, by;
 
   if((im == NULL) || (window_width == 0) || (message == NULL))
-    return;
+    return NULL;
 
   wd = (xitk_dialog_t *) xitk_xmalloc(sizeof(xitk_dialog_t));
 
@@ -1037,7 +1207,7 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
     int       n, size = 100;
     
     if((buf = xitk_xmalloc(size)) == NULL) 
-      return;
+      return NULL;
     
     while(1) {
       
@@ -1054,7 +1224,7 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
 	size *= 2;
 
       if((buf = realloc(buf, size)) == NULL)
-	return;
+	return NULL;
     }
     
     i = xitk_image_create_image_from_string(im, DEFAULT_FONT_12, windoww - 40, align, buf);
@@ -1072,6 +1242,8 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
   wd->xwin           = xitk_window_create_dialog_window(im, ((title != NULL) 
 							     ? title : _("Question?")), 
 							0, 0, windoww, windowh);  
+  wd->xwin->parent = wd;
+
   xitk_window_center_window(im, wd->xwin);
   
   wd->widget_list                = xitk_widget_list_new();
@@ -1094,7 +1266,7 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
   XITK_WIDGET_INIT(&lb, im);
   lb.button_type       = CLICK_BUTTON;
   lb.label             = button1_label;
-  lb.align             = LABEL_ALIGN_CENTER;
+  lb.align             = ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
   lb.userdata          = (void*)wd;
@@ -1108,7 +1280,7 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = button2_label;
-  lb.align             = LABEL_ALIGN_CENTER;
+  lb.align             = ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
   lb.userdata          = (void*)wd;
@@ -1122,7 +1294,7 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
 
   lb.button_type       = CLICK_BUTTON;
   lb.label             = button3_label;
-  lb.align             = LABEL_ALIGN_CENTER;
+  lb.align             = ALIGN_CENTER;
   lb.callback          = _xitk_window_destroy_window;
   lb.state_callback    = NULL;
   lb.userdata          = (void*)wd;
@@ -1138,25 +1310,25 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
 
   /* Draw text area */
   {
-    Pixmap   bg;
-    int      width, height;
-    GC       gc;
+    xitk_pixmap_t *bg;
+    int            width, height;
+    GC             gc;
     
     xitk_window_get_window_size(wd->xwin, &width, &height);
-    bg = xitk_image_create_pixmap(im, width, height);
+    bg = xitk_image_create_xitk_pixmap(im, width, height);
     
     XLOCK(im->x.disp);
-    gc = XCreateGC(im->x.disp, im->x.base_window, None, None);
-    XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg,
+    gc = XCreateGC(im->x.disp, (xitk_window_get_background(wd->xwin)), None, None);
+    XCopyArea(im->x.disp, (xitk_window_get_background(wd->xwin)), bg->pixmap,
 	      gc, 0, 0, width, height, 0, 0);
-    XCopyArea(im->x.disp, i->image, bg,
-	      gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
+    XCopyArea(im->x.disp, i->image->pixmap, bg->pixmap,
+	      i->image->gc, 0, 0, i->width, i->height, 20, (TITLE_BAR_HEIGHT + 20));
     XUNLOCK(im->x.disp);
 
-    xitk_window_change_background(im, wd->xwin, bg, width, height);
+    xitk_window_change_background(im, wd->xwin, bg->pixmap, width, height);
 
+    xitk_image_destroy_xitk_pixmap(bg);
     XLOCK(im->x.disp);
-    XFreePixmap(im->x.disp, bg);
     XFreeGC(im->x.disp, gc);
     XUNLOCK(im->x.disp);
 
@@ -1175,20 +1347,26 @@ void xitk_window_dialog_three_buttons_with_width(ImlibData *im, char *title,
 					NULL,
 					wd->widget_list,
 					(void *)wd);
+  xitk_enable_and_show_widget(wd->wyes);
+  xitk_enable_and_show_widget(wd->wno);
+  xitk_enable_and_show_widget(wd->wcancel);
+
+  return wd->xwin;
 }
 
-void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
-					       xitk_state_callback_t ycb, 
-					       xitk_state_callback_t ncb, 
-					       xitk_state_callback_t ccb, 
-					       void *userdata, 
-					       int window_width, int align, char *message, ...) {
-  va_list   args;
-  char     *buf;
-  int       n, size = 100;
+xitk_window_t *xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
+							 xitk_state_callback_t ycb, 
+							 xitk_state_callback_t ncb, 
+							 xitk_state_callback_t ccb, 
+							 void *userdata, 
+							 int window_width, int align, char *message, ...) {
+  va_list        args;
+  char          *buf;
+  int            n, size = 100;
+  xitk_window_t *xw = NULL;
   
   if((buf = xitk_xmalloc(size)) == NULL) 
-    return;
+    return NULL;
   
   while(1) {
     
@@ -1205,15 +1383,16 @@ void xitk_window_dialog_yesnocancel_with_width(ImlibData *im, char *title,
       size *= 2;
     
     if((buf = realloc(buf, size)) == NULL)
-      return;
+      return NULL;
   }
   
   {
     char buf2[(strlen(buf) * 2) + 1];
     
     xitk_subst_special_chars(buf, buf2);
-    xitk_window_dialog_three_buttons_with_width(im, title, _("Yes"), _("No"), _("Cancel"),
-						ycb, ncb, ccb, userdata, window_width, align, buf2);
+    xw = xitk_window_dialog_three_buttons_with_width(im, title, _("Yes"), _("No"), _("Cancel"),
+						     ycb, ncb, ccb, userdata, window_width, align, buf2);
   }
   XITK_FREE(buf);
+  return xw;
 }
